@@ -15,7 +15,11 @@ namespace test
 					for (int y = 0; y < bmp.Height; y++) {
 						for (int x = 0; x < bmp.Width; x++) {
 							Color c = bmp.GetPixel(x, y);
-							int p = (c.B << 24) | (c.G << 16) | (c.R << 8) | c.A;
+							
+							int r = c.R * c.A / 255;
+							int g = c.G * c.A / 255;
+							int b = c.B * c.A / 255;
+							int p = (r << 24) | (g << 16) | (b << 8) | c.A;
 							sw.Write("0x" + ((uint)p).ToString("X8") + ",");
 						}
 						sw.WriteLine();
@@ -26,7 +30,7 @@ namespace test
 		
 		public static void Main(string[] args) {
 			using (StreamWriter sw = new StreamWriter(dst)) {
-				sw.WriteLine("/* Generated using misc/mac_icon_gen.cs */");
+				sw.WriteLine("/* Generated using misc/macOS/mac_icon_gen.cs */");
 				sw.WriteLine("");
 				sw.WriteLine("static const unsigned int CCIcon_Data[] = {");
 				DumpIcon(sw, 64, 64);
