@@ -1,10 +1,14 @@
 #ifndef CC_ANIMATIONS_H
 #define CC_ANIMATIONS_H
+#include "Core.h"
 /* 
 Contains everything relating to texture animations (including default water/lava ones)
 Copyright 2014-2023 ClassiCube | Licensed under BSD-3
 */
+CC_BEGIN_HEADER
 
 struct IGameComponent;
 extern struct IGameComponent Animations_Component;
+
+CC_END_HEADER
 #endif
