@@ -13,7 +13,6 @@
 #include "Physics.h"
 #include "Model.h"
 #include "Audio.h"
-#include "Bitmap.h"
 
 /*########################################################################################################################*
 *----------------------------------------------------AnimatedComponent----------------------------------------------------*
@@ -58,10 +57,10 @@ void AnimatedComp_Init(struct AnimatedComp* anim) {
 	anim->BobStrength = 1.0f; anim->BobStrengthO = 1.0f; anim->BobStrengthN = 1.0f;
 }
 
-void AnimatedComp_Update(struct Entity* e, Vec3 oldPos, Vec3 newPos, double delta) {
+void AnimatedComp_Update(struct Entity* e, Vec3 oldPos, Vec3 newPos, float delta) {
 	struct AnimatedComp* anim = &e->Anim;
-	float dx = newPos.X - oldPos.X;
-	float dz = newPos.Z - oldPos.Z;
+	float dx = newPos.x - oldPos.x;
+	float dz = newPos.z - oldPos.z;
 	float distance = Math_SqrtF(dx * dx + dz * dz);
 	int i;
 
@@ -72,9 +71,9 @@ void AnimatedComp_Update(struct Entity* e, Vec3 oldPos, Vec3 newPos, double delt
 	if (distance > 0.05f) {
 		walkDelta = distance * 2 * (float)(20 * delta);
 		anim->WalkTimeN += walkDelta;
-		anim->SwingN += (float)delta * 3;
+		anim->SwingN += delta * 3;
 	} else {
-		anim->SwingN -= (float)delta * 3;
+		anim->SwingN -= delta * 3;
 	}
 	Math_Clamp(anim->SwingN, 0.0f, 1.0f);
 
@@ -121,8 +120,7 @@ void TiltComp_Init(struct TiltComp* anim) {
 	anim->VelTiltStrengthO = 1.0f; anim->VelTiltStrengthN = 1.0f;
 }
 
-void TiltComp_Update(struct TiltComp* anim, double delta) {
-	struct LocalPlayer* p = &LocalPlayer_Instance;
+void TiltComp_Update(struct LocalPlayer* p, struct TiltComp* anim, float delta) {
 	int i;
 
 	anim->VelTiltStrengthO = anim->VelTiltStrengthN;
@@ -132,8 +130,7 @@ void TiltComp_Update(struct TiltComp* anim, double delta) {
 	}
 }
 
-void TiltComp_GetCurrent(struct TiltComp* anim, float t) {
-	struct LocalPlayer* p = &LocalPlayer_Instance;
+void TiltComp_GetCurrent(struct LocalPlayer* p, struct TiltComp* anim, float t) {
 	struct AnimatedComp* pAnim = &p->Base.Anim;
 
 	anim->VelTiltStrength = Math_Lerp(anim->VelTiltStrengthO, anim->VelTiltStrengthN, t);
@@ -428,8 +425,7 @@ void NetInterpComp_AdvanceState(struct NetInterpComp* interp, struct Entity* e)
 /*########################################################################################################################*
 *-----------------------------------------------LocalInterpolationComponent-----------------------------------------------*
 *#########################################################################################################################*/
-static void LocalInterpComp_SetPosition(struct LocationUpdate* update, int mode) {
-	struct Entity* e = &LocalPlayer_Instance.Base;
+static void LocalInterpComp_SetPosition(struct Entity* e, struct LocationUpdate* update, int mode) {
 	float yOffset;
 
 	if (mode == LU_POS_ABSOLUTE_INSTANT || mode == LU_POS_ABSOLUTE_SMOOTH) {
@@ -442,8 +438,8 @@ static void LocalInterpComp_SetPosition(struct LocationUpdate* update, int mode)
 	}
 
 	/* If server sets Y position exactly on ground, push up a tiny bit */
-	yOffset = e->next.pos.Y - Math_Floor(e->next.pos.Y);
-	if (yOffset < ENTITY_ADJUSTMENT) e->next.pos.Y += ENTITY_ADJUSTMENT;
+	yOffset = e->next.pos.y - Math_Floor(e->next.pos.y);
+	if (yOffset < ENTITY_ADJUSTMENT) e->next.pos.y += ENTITY_ADJUSTMENT;
 
 	if (mode == LU_POS_ABSOLUTE_INSTANT) { 
 		e->prev.pos = e->next.pos; e->Position = e->next.pos; 
@@ -456,15 +452,14 @@ static void LocalInterpComp_Angle(float* prev, float* next, float value, cc_bool
 	if (!interpolate) *prev = value;
 }
 
-void LocalInterpComp_SetLocation(struct InterpComp* interp, struct LocationUpdate* update) {
-	struct Entity* e = &LocalPlayer_Instance.Base;
+void LocalInterpComp_SetLocation(struct InterpComp* interp, struct LocationUpdate* update, struct Entity* e) {
 	struct EntityLocation* prev = &e->prev;
 	struct EntityLocation* next = &e->next;
 	cc_uint8 flags      = update->flags;
 	cc_bool interpolate = flags & LU_ORI_INTERPOLATE;
 
 	if (flags & LU_HAS_POS) {
-		LocalInterpComp_SetPosition(update, flags & LU_POS_MODEMASK);
+		LocalInterpComp_SetPosition(e, update, flags & LU_POS_MODEMASK);
 	}
 	if (flags & LU_HAS_PITCH) {
 		LocalInterpComp_Angle(&prev->pitch, &next->pitch, update->pitch, interpolate);
@@ -500,221 +495,6 @@ void LocalInterpComp_AdvanceState(struct InterpComp* interp, struct Entity* e) {
 }
 
 
-/*########################################################################################################################*
-*-----------------------------------------------------ShadowComponent-----------------------------------------------------*
-*#########################################################################################################################*/
-cc_bool ShadowComponent_BoundShadowTex;
-GfxResourceID ShadowComponent_ShadowTex;
-static float shadow_radius, shadow_uvScale;
-struct ShadowData { float Y; BlockID Block; cc_uint8 A; };
-
-static cc_bool lequal(float a, float b) { return a < b || Math_AbsF(a - b) < 0.001f; }
-static void ShadowComponent_DrawCoords(struct VertexTextured** vertices, struct Entity* e, struct ShadowData* data, float x1, float z1, float x2, float z2) {
-	PackedCol col;
-	struct VertexTextured* v;
-	Vec3 cen;
-	float u1, v1, u2, v2;
-
-	if (lequal(x2, x1) || lequal(z2, z1)) return;
-	cen = e->Position;
-
-	u1 = (x1 - cen.X) * shadow_uvScale + 0.5f;
-	v1 = (z1 - cen.Z) * shadow_uvScale + 0.5f;
-	u2 = (x2 - cen.X) * shadow_uvScale + 0.5f;
-	v2 = (z2 - cen.Z) * shadow_uvScale + 0.5f;
-	if (u2 <= 0.0f || v2 <= 0.0f || u1 >= 1.0f || v1 >= 1.0f) return;
-
-	x1 = max(x1, cen.X - shadow_radius); u1 = u1 >= 0.0f ? u1 : 0.0f;
-	z1 = max(z1, cen.Z - shadow_radius); v1 = v1 >= 0.0f ? v1 : 0.0f;
-	x2 = min(x2, cen.X + shadow_radius); u2 = u2 <= 1.0f ? u2 : 1.0f;
-	z2 = min(z2, cen.Z + shadow_radius); v2 = v2 <= 1.0f ? v2 : 1.0f;
-
-	v   = *vertices;
-	col = PackedCol_Make(255, 255, 255, data->A);
-
-	v->X = x1; v->Y = data->Y; v->Z = z1; v->Col = col; v->U = u1; v->V = v1; v++;
-	v->X = x2; v->Y = data->Y; v->Z = z1; v->Col = col; v->U = u2; v->V = v1; v++;
-	v->X = x2; v->Y = data->Y; v->Z = z2; v->Col = col; v->U = u2; v->V = v2; v++;
-	v->X = x1; v->Y = data->Y; v->Z = z2; v->Col = col; v->U = u1; v->V = v2; v++;
-
-	*vertices = v;
-}
-
-static void ShadowComponent_DrawSquareShadow(struct VertexTextured** vertices, float y, float x, float z) {
-	PackedCol col = PackedCol_Make(255, 255, 255, 220);
-	float     uv1 = 63/128.0f, uv2 = 64/128.0f;
-	struct VertexTextured* v = *vertices;
-
-	v->X = x;     v->Y = y; v->Z = z;     v->Col = col; v->U = uv1; v->V = uv1; v++;
-	v->X = x + 1; v->Y = y; v->Z = z;     v->Col = col; v->U = uv2; v->V = uv1; v++;
-	v->X = x + 1; v->Y = y; v->Z = z + 1; v->Col = col; v->U = uv2; v->V = uv2; v++;
-	v->X = x;     v->Y = y; v->Z = z + 1; v->Col = col; v->U = uv1; v->V = uv2; v++;
-
-	*vertices = v;
-}
-
-/* Shadow may extend down multiple blocks vertically */
-/* If so, shadow on a block must be 'chopped up' to avoid a shadow underneath block above this one */
-static void ShadowComponent_DrawCircle(struct VertexTextured** vertices, struct Entity* e, struct ShadowData* data, float x, float z) {
-	Vec3 min, max, nMin, nMax;
-	int i;
-	x = (float)Math_Floor(x); z = (float)Math_Floor(z);
-	min = Blocks.MinBB[data[0].Block]; max = Blocks.MaxBB[data[0].Block];
-
-	ShadowComponent_DrawCoords(vertices, e, &data[0], x + min.X, z + min.Z, x + max.X, z + max.Z);
-	for (i = 1; i < 4; i++) {
-		if (data[i].Block == BLOCK_AIR) return;
-		nMin = Blocks.MinBB[data[i].Block]; nMax = Blocks.MaxBB[data[i].Block];
-
-		ShadowComponent_DrawCoords(vertices, e, &data[i], x +  min.X, z + nMin.Z, x +  max.X, z +  min.Z);
-		ShadowComponent_DrawCoords(vertices, e, &data[i], x +  min.X, z +  max.Z, x +  max.X, z + nMax.Z);
-
-		ShadowComponent_DrawCoords(vertices, e, &data[i], x + nMin.X, z + nMin.Z, x +  min.X, z + nMax.Z);
-		ShadowComponent_DrawCoords(vertices, e, &data[i], x +  max.X, z + nMin.Z, x + nMax.X, z + nMax.Z);
-		min = nMin; max = nMax;
-	}
-}
-
-static void ShadowComponent_CalcAlpha(float playerY, struct ShadowData* data) {
-	float height = playerY - data->Y;
-	if (height <= 6.0f) {
-		data->A = (cc_uint8)(160 - 160 * height / 6.0f);
-		data->Y += 1.0f / 64.0f; return;
-	}
-
-	data->A = 0;
-	if (height <= 16.0f)      data->Y += 1.0f / 64.0f;
-	else if (height <= 32.0f) data->Y += 1.0f / 16.0f;
-	else if (height <= 96.0f) data->Y += 1.0f / 8.0f;
-	else data->Y += 1.0f / 4.0f;
-}
-
-static cc_bool ShadowComponent_GetBlocks(struct Entity* e, int x, int y, int z, struct ShadowData* data) {
-	struct ShadowData zeroData = { 0 };
-	struct ShadowData* cur;
-	float posY, topY;
-	cc_bool outside;
-	BlockID block; cc_uint8 draw;
-	int i;
-
-	for (i = 0; i < 4; i++) { data[i] = zeroData; }
-	cur     = data;
-	posY    = e->Position.Y;
-	outside = !World_ContainsXZ(x, z);
-
-	for (i = 0; y >= 0 && i < 4; y--) {
-		if (!outside) {
-			block = World_GetBlock(x, y, z);
-		} else if (y == Env.EdgeHeight - 1) {
-			block = Blocks.Draw[Env.EdgeBlock] == DRAW_GAS  ? BLOCK_AIR : BLOCK_BEDROCK;
-		} else if (y == Env_SidesHeight - 1) {
-			block = Blocks.Draw[Env.SidesBlock] == DRAW_GAS ? BLOCK_AIR : BLOCK_BEDROCK;
-		} else {
-			block = BLOCK_AIR;
-		}
-
-		draw = Blocks.Draw[block];
-		if (draw == DRAW_GAS || draw == DRAW_SPRITE || Blocks.IsLiquid[block]) continue;
-		topY = y + Blocks.MaxBB[block].Y;
-		if (topY >= posY + 0.01f) continue;
-
-		cur->Block = block; cur->Y = topY;
-		ShadowComponent_CalcAlpha(posY, cur);
-		i++; cur++;
-
-		/* Check if the casted shadow will continue on further down. */
-		if (Blocks.MinBB[block].X == 0.0f && Blocks.MaxBB[block].X == 1.0f &&
-			Blocks.MinBB[block].Z == 0.0f && Blocks.MaxBB[block].Z == 1.0f) return true;
-	}
-
-	if (i < 4) {
-		cur->Block = Env.EdgeBlock; cur->Y = 0.0f;
-		ShadowComponent_CalcAlpha(posY, cur);
-		i++; cur++;
-	}
-	return true;
-}
-
-#define sh_size 128
-#define sh_half (sh_size / 2)
-static void ShadowComponent_MakeTex(void) {
-	BitmapCol pixels[sh_size * sh_size];
-	BitmapCol color = BitmapCol_Make(0, 0, 0, 200);
-	struct Bitmap bmp;
-	cc_uint32 x, y;
-
-	Bitmap_Init(bmp, sh_size, sh_size, pixels);
-	for (y = 0; y < sh_size; y++) {
-		BitmapCol* row = Bitmap_GetRow(&bmp, y);
-
-		for (x = 0; x < sh_size; x++) {
-			double dist = 
-				(sh_half - (x + 0.5)) * (sh_half - (x + 0.5)) +
-				(sh_half - (y + 0.5)) * (sh_half - (y + 0.5));
-			row[x] = dist < sh_half * sh_half ? color : 0;
-		}
-	}
-	Gfx_RecreateTexture(&ShadowComponent_ShadowTex, &bmp, 0, false);
-}
-
-void ShadowComponent_Draw(struct Entity* e) {
-	struct VertexTextured vertices[128];
-	struct VertexTextured* ptr;
-	struct ShadowData data[4];
-	GfxResourceID vb;
-	Vec3 pos;
-	float radius;
-	int y, count;
-	int x1, z1, x2, z2;
-
-	pos = e->Position;
-	if (pos.Y < 0.0f) return;
-	y = min((int)pos.Y, World.MaxY);
-
-	radius = 7.0f * min(e->ModelScale.Y, 1.0f) * e->Model->shadowScale;
-	shadow_radius  = radius / 16.0f;
-	shadow_uvScale = 16.0f / (radius * 2.0f);
-
-	/* TODO: Should shadow component use its own VB? */
-	ptr = vertices;
-	if (Entities.ShadowsMode == SHADOW_MODE_SNAP_TO_BLOCK) {
-		vb = Gfx_texVb;
-		x1 = Math_Floor(pos.X); z1 = Math_Floor(pos.Z);
-		if (!ShadowComponent_GetBlocks(e, x1, y, z1, data)) return;
-
-		ShadowComponent_DrawSquareShadow(&ptr, data[0].Y, x1, z1);
-	} else {
-		vb = Models.Vb;
-		x1 = Math_Floor(pos.X - shadow_radius); z1 = Math_Floor(pos.Z - shadow_radius);
-		x2 = Math_Floor(pos.X + shadow_radius); z2 = Math_Floor(pos.Z + shadow_radius);
-
-		if (ShadowComponent_GetBlocks(e, x1, y, z1, data) && data[0].A > 0) {
-			ShadowComponent_DrawCircle(&ptr, e, data, (float)x1, (float)z1);
-		}
-		if (x1 != x2 && ShadowComponent_GetBlocks(e, x2, y, z1, data) && data[0].A > 0) {
-			ShadowComponent_DrawCircle(&ptr, e, data, (float)x2, (float)z1);
-		}
-		if (z1 != z2 && ShadowComponent_GetBlocks(e, x1, y, z2, data) && data[0].A > 0) {
-			ShadowComponent_DrawCircle(&ptr, e, data, (float)x1, (float)z2);
-		}
-		if (x1 != x2 && z1 != z2 && ShadowComponent_GetBlocks(e, x2, y, z2, data) && data[0].A > 0) {
-			ShadowComponent_DrawCircle(&ptr, e, data, (float)x2, (float)z2);
-		}
-	}
-
-	if (ptr == vertices) return;
-	if (!ShadowComponent_ShadowTex) ShadowComponent_MakeTex();
-
-	if (!ShadowComponent_BoundShadowTex) {
-		Gfx_BindTexture(ShadowComponent_ShadowTex);
-		ShadowComponent_BoundShadowTex = true;
-	}
-
-	count = (int)(ptr - vertices);
-	Gfx_UpdateDynamicVb_IndexedTris(vb, vertices, count);
-}
-
-
 /*########################################################################################################################*
 *---------------------------------------------------CollisionsComponent---------------------------------------------------*
 *#########################################################################################################################*/
@@ -725,21 +505,21 @@ cc_bool Collisions_HitHorizontal(struct CollisionsComp* comp) {
 #define COLLISIONS_ADJ 0.001f
 
 static void Collisions_ClipX(struct Entity* e, Vec3* size, struct AABB* entityBB, struct AABB* extentBB) {
-	e->Velocity.X = 0.0f;
-	entityBB->Min.X = e->Position.X - size->X / 2; extentBB->Min.X = entityBB->Min.X;
-	entityBB->Max.X = e->Position.X + size->X / 2; extentBB->Max.X = entityBB->Max.X;
+	e->Velocity.x = 0.0f;
+	entityBB->Min.x = e->Position.x - size->x / 2; extentBB->Min.x = entityBB->Min.x;
+	entityBB->Max.x = e->Position.x + size->x / 2; extentBB->Max.x = entityBB->Max.x;
 }
 
 static void Collisions_ClipY(struct Entity* e, Vec3* size, struct AABB* entityBB, struct AABB* extentBB) {
-	e->Velocity.Y = 0.0f;
-	entityBB->Min.Y = e->Position.Y;               extentBB->Min.Y = entityBB->Min.Y;
-	entityBB->Max.Y = e->Position.Y + size->Y;     extentBB->Max.Y = entityBB->Max.Y;
+	e->Velocity.y = 0.0f;
+	entityBB->Min.y = e->Position.y;               extentBB->Min.y = entityBB->Min.y;
+	entityBB->Max.y = e->Position.y + size->y;     extentBB->Max.y = entityBB->Max.y;
 }
 
 static void Collisions_ClipZ(struct Entity* e, Vec3* size, struct AABB* entityBB, struct AABB* extentBB) {
-	e->Velocity.Z = 0.0f;
-	entityBB->Min.Z = e->Position.Z - size->Z / 2; extentBB->Min.Z = entityBB->Min.Z;
-	entityBB->Max.Z = e->Position.Z + size->Z / 2; extentBB->Max.Z = entityBB->Max.Z;
+	e->Velocity.z = 0.0f;
+	entityBB->Min.z = e->Position.z - size->z / 2; extentBB->Min.z = entityBB->Min.z;
+	entityBB->Max.z = e->Position.z + size->z / 2; extentBB->Max.z = entityBB->Max.z;
 }
 
 static cc_bool Collisions_CanSlideThrough(struct AABB* adjFinalBB) {
@@ -752,9 +532,9 @@ static cc_bool Collisions_CanSlideThrough(struct AABB* adjFinalBB) {
 	IVec3_Floor(&bbMin, &adjFinalBB->Min);
 	IVec3_Floor(&bbMax, &adjFinalBB->Max);
 
-	for (y = bbMin.Y; y <= bbMax.Y; y++) { v.Y = (float)y;
-		for (z = bbMin.Z; z <= bbMax.Z; z++) { v.Z = (float)z;
-			for (x = bbMin.X; x <= bbMax.X; x++) { v.X = (float)x;
+	for (y = bbMin.y; y <= bbMax.y; y++) { v.y = (float)y;
+		for (z = bbMin.z; z <= bbMax.z; z++) { v.z = (float)z;
+			for (x = bbMin.x; x <= bbMax.x; x++) { v.x = (float)x;
 
 				block = World_GetPhysicsBlock(x, y, z);
 				Vec3_Add(&blockBB.Min, &v, &Blocks.MinBB[block]);
@@ -770,24 +550,24 @@ static cc_bool Collisions_CanSlideThrough(struct AABB* adjFinalBB) {
 
 static cc_bool Collisions_DidSlide(struct CollisionsComp* comp, struct AABB* blockBB, Vec3* size,
 									struct AABB* finalBB, struct AABB* entityBB, struct AABB* extentBB) {
-	float yDist = blockBB->Max.Y - entityBB->Min.Y;
+	float yDist = blockBB->Max.y - entityBB->Min.y;
 	struct AABB adjBB;
 
 	if (yDist > 0.0f && yDist <= comp->StepSize + 0.01f) {
-		float blockBB_MinX = max(blockBB->Min.X, blockBB->Max.X - size->X / 2);
-		float blockBB_MaxX = min(blockBB->Max.X, blockBB->Min.X + size->X / 2);
-		float blockBB_MinZ = max(blockBB->Min.Z, blockBB->Max.Z - size->Z / 2);
-		float blockBB_MaxZ = min(blockBB->Max.Z, blockBB->Min.Z + size->Z / 2);
+		float blockBB_MinX = max(blockBB->Min.x, blockBB->Max.x - size->x / 2);
+		float blockBB_MaxX = min(blockBB->Max.x, blockBB->Min.x + size->x / 2);
+		float blockBB_MinZ = max(blockBB->Min.z, blockBB->Max.z - size->z / 2);
+		float blockBB_MaxZ = min(blockBB->Max.z, blockBB->Min.z + size->z / 2);
 		
-		adjBB.Min.X = min(finalBB->Min.X, blockBB_MinX + COLLISIONS_ADJ);
-		adjBB.Max.X = max(finalBB->Max.X, blockBB_MaxX - COLLISIONS_ADJ);
-		adjBB.Min.Y = blockBB->Max.Y + COLLISIONS_ADJ;
-		adjBB.Max.Y = adjBB.Min.Y    + size->Y;
-		adjBB.Min.Z = min(finalBB->Min.Z, blockBB_MinZ + COLLISIONS_ADJ);
-		adjBB.Max.Z = max(finalBB->Max.Z, blockBB_MaxZ - COLLISIONS_ADJ);
+		adjBB.Min.x = min(finalBB->Min.x, blockBB_MinX + COLLISIONS_ADJ);
+		adjBB.Max.x = max(finalBB->Max.x, blockBB_MaxX - COLLISIONS_ADJ);
+		adjBB.Min.y = blockBB->Max.y + COLLISIONS_ADJ;
+		adjBB.Max.y = adjBB.Min.y    + size->y;
+		adjBB.Min.z = min(finalBB->Min.z, blockBB_MinZ + COLLISIONS_ADJ);
+		adjBB.Max.z = max(finalBB->Max.z, blockBB_MaxZ - COLLISIONS_ADJ);
 
 		if (!Collisions_CanSlideThrough(&adjBB)) return false;
-		comp->Entity->Position.Y = adjBB.Min.Y;
+		comp->Entity->Position.y = adjBB.Min.y;
 		comp->Entity->OnGround = true;
 		Collisions_ClipY(comp->Entity, size, entityBB, extentBB);
 		return true;
@@ -798,7 +578,7 @@ static cc_bool Collisions_DidSlide(struct CollisionsComp* comp, struct AABB* blo
 static void Collisions_ClipXMin(struct CollisionsComp* comp, struct AABB* blockBB, struct AABB* entityBB,
 								cc_bool wasOn, struct AABB* finalBB, struct AABB* extentBB, Vec3* size) {
 	if (!wasOn || !Collisions_DidSlide(comp, blockBB, size, finalBB, entityBB, extentBB)) {
-		comp->Entity->Position.X = blockBB->Min.X - size->X / 2 - COLLISIONS_ADJ;
+		comp->Entity->Position.x = blockBB->Min.x - size->x / 2 - COLLISIONS_ADJ;
 		Collisions_ClipX(comp->Entity, size, entityBB, extentBB);
 		comp->HitXMin = true;
 	}
@@ -807,7 +587,7 @@ static void Collisions_ClipXMin(struct CollisionsComp* comp, struct AABB* blockB
 static void Collisions_ClipXMax(struct CollisionsComp* comp, struct AABB* blockBB, struct AABB* entityBB, 
 								cc_bool wasOn, struct AABB* finalBB, struct AABB* extentBB, Vec3* size) {
 	if (!wasOn || !Collisions_DidSlide(comp, blockBB, size, finalBB, entityBB, extentBB)) {
-		comp->Entity->Position.X = blockBB->Max.X + size->X / 2 + COLLISIONS_ADJ;
+		comp->Entity->Position.x = blockBB->Max.x + size->x / 2 + COLLISIONS_ADJ;
 		Collisions_ClipX(comp->Entity, size, entityBB, extentBB);
 		comp->HitXMax = true;
 	}
@@ -816,7 +596,7 @@ static void Collisions_ClipXMax(struct CollisionsComp* comp, struct AABB* blockB
 static void Collisions_ClipZMax(struct CollisionsComp* comp, struct AABB* blockBB, struct AABB* entityBB, 
 								cc_bool wasOn, struct AABB* finalBB, struct AABB* extentBB, Vec3* size) {
 	if (!wasOn || !Collisions_DidSlide(comp, blockBB, size, finalBB, entityBB, extentBB)) {
-		comp->Entity->Position.Z = blockBB->Max.Z + size->Z / 2 + COLLISIONS_ADJ;
+		comp->Entity->Position.z = blockBB->Max.z + size->z / 2 + COLLISIONS_ADJ;
 		Collisions_ClipZ(comp->Entity, size, entityBB, extentBB);
 		comp->HitZMax = true;
 	}
@@ -825,7 +605,7 @@ static void Collisions_ClipZMax(struct CollisionsComp* comp, struct AABB* blockB
 static void Collisions_ClipZMin(struct CollisionsComp* comp, struct AABB* blockBB, struct AABB* entityBB,
 								cc_bool wasOn, struct AABB* finalBB, struct AABB* extentBB, Vec3* size) {
 	if (!wasOn || !Collisions_DidSlide(comp, blockBB, size, finalBB, entityBB, extentBB)) {
-		comp->Entity->Position.Z = blockBB->Min.Z - size->Z / 2 - COLLISIONS_ADJ;
+		comp->Entity->Position.z = blockBB->Min.z - size->z / 2 - COLLISIONS_ADJ;
 		Collisions_ClipZ(comp->Entity, size, entityBB, extentBB);
 		comp->HitZMin = true;
 	}
@@ -833,14 +613,14 @@ static void Collisions_ClipZMin(struct CollisionsComp* comp, struct AABB* blockB
 
 static void Collisions_ClipYMin(struct CollisionsComp* comp, struct AABB* blockBB, struct AABB* entityBB, 
 								struct AABB* extentBB, Vec3* size) {
-	comp->Entity->Position.Y = blockBB->Min.Y - size->Y - COLLISIONS_ADJ;
+	comp->Entity->Position.y = blockBB->Min.y - size->y - COLLISIONS_ADJ;
 	Collisions_ClipY(comp->Entity, size, entityBB, extentBB);
 	comp->HitYMin = true;
 }
 
 static void Collisions_ClipYMax(struct CollisionsComp* comp, struct AABB* blockBB, struct AABB* entityBB, 
 								struct AABB* extentBB, Vec3* size) {
-	comp->Entity->Position.Y = blockBB->Max.Y + COLLISIONS_ADJ;
+	comp->Entity->Position.y = blockBB->Max.y + COLLISIONS_ADJ;
 	comp->Entity->OnGround = true;
 	Collisions_ClipY(comp->Entity, size, entityBB, extentBB);
 	comp->HitYMax = true;
@@ -868,8 +648,8 @@ static void Collisions_CollideWithReachableBlocks(struct CollisionsComp* comp, i
 	for (i = 0; i < count; i++) {
 		/* Unpack the block and coordinate data */
 		state  = Searcher_States[i];
-		bPos.X = state.X >> 3; bPos.Y = state.Y >> 4; bPos.Z = state.Z >> 3;
-		block  = (state.X & 0x7) | (state.Y & 0xF) << 3 | (state.Z & 0x7) << 7;
+		bPos.x = state.x >> 3; bPos.y = state.y >> 4; bPos.z = state.z >> 3;
+		block  = (state.x & 0x7) | (state.y & 0xF) << 3 | (state.z & 0x7) << 7;
 
 		Vec3_Add(&blockBB.Min, &Blocks.MinBB[block], &bPos);
 		Vec3_Add(&blockBB.Max, &Blocks.MaxBB[block], &bPos);
@@ -883,41 +663,41 @@ static void Collisions_CollideWithReachableBlocks(struct CollisionsComp* comp, i
 
 		/* Calculate the location of the entity when it collides with this block */
 		v = entity->Velocity; 
-		v.X *= tx; v.Y *= ty; v.Z *= tz;
+		v.x *= tx; v.y *= ty; v.z *= tz;
 		/* Inlined ABBB_Offset */
 		Vec3_Add(&finalBB.Min, &entityBB->Min, &v);
 		Vec3_Add(&finalBB.Max, &entityBB->Max, &v);
 
 		/* if we have hit the bottom of a block, we need to change the axis we test first */
 		if (!comp->HitYMin) {
-			if (finalBB.Min.Y + COLLISIONS_ADJ >= blockBB.Max.Y) {
+			if (finalBB.Min.y + COLLISIONS_ADJ >= blockBB.Max.y) {
 				Collisions_ClipYMax(comp, &blockBB, entityBB, extentBB, &size);
-			} else if (finalBB.Max.Y - COLLISIONS_ADJ <= blockBB.Min.Y) {
+			} else if (finalBB.Max.y - COLLISIONS_ADJ <= blockBB.Min.y) {
 				Collisions_ClipYMin(comp, &blockBB, entityBB, extentBB, &size);
-			} else if (finalBB.Min.X + COLLISIONS_ADJ >= blockBB.Max.X) {
+			} else if (finalBB.Min.x + COLLISIONS_ADJ >= blockBB.Max.x) {
 				Collisions_ClipXMax(comp, &blockBB, entityBB, wasOn, &finalBB, extentBB, &size);
-			} else if (finalBB.Max.X - COLLISIONS_ADJ <= blockBB.Min.X) {
+			} else if (finalBB.Max.x - COLLISIONS_ADJ <= blockBB.Min.x) {
 				Collisions_ClipXMin(comp, &blockBB, entityBB, wasOn, &finalBB, extentBB, &size);
-			} else if (finalBB.Min.Z + COLLISIONS_ADJ >= blockBB.Max.Z) {
+			} else if (finalBB.Min.z + COLLISIONS_ADJ >= blockBB.Max.z) {
 				Collisions_ClipZMax(comp, &blockBB, entityBB, wasOn, &finalBB, extentBB, &size);
-			} else if (finalBB.Max.Z - COLLISIONS_ADJ <= blockBB.Min.Z) {
+			} else if (finalBB.Max.z - COLLISIONS_ADJ <= blockBB.Min.z) {
 				Collisions_ClipZMin(comp, &blockBB, entityBB, wasOn, &finalBB, extentBB, &size);
 			}
 			continue;
 		}
 
 		/* if flying or falling, test the horizontal axes first */
-		if (finalBB.Min.X + COLLISIONS_ADJ >= blockBB.Max.X) {
+		if (finalBB.Min.x + COLLISIONS_ADJ >= blockBB.Max.x) {
 			Collisions_ClipXMax(comp, &blockBB, entityBB, wasOn, &finalBB, extentBB, &size);
-		} else if (finalBB.Max.X - COLLISIONS_ADJ <= blockBB.Min.X) {
+		} else if (finalBB.Max.x - COLLISIONS_ADJ <= blockBB.Min.x) {
 			Collisions_ClipXMin(comp, &blockBB, entityBB, wasOn, &finalBB, extentBB, &size);
-		} else if (finalBB.Min.Z + COLLISIONS_ADJ >= blockBB.Max.Z) {
+		} else if (finalBB.Min.z + COLLISIONS_ADJ >= blockBB.Max.z) {
 			Collisions_ClipZMax(comp, &blockBB, entityBB, wasOn, &finalBB, extentBB, &size);
-		} else if (finalBB.Max.Z - COLLISIONS_ADJ <= blockBB.Min.Z) {
+		} else if (finalBB.Max.z - COLLISIONS_ADJ <= blockBB.Min.z) {
 			Collisions_ClipZMin(comp, &blockBB, entityBB, wasOn, &finalBB, extentBB, &size);
-		} else if (finalBB.Min.Y + COLLISIONS_ADJ >= blockBB.Max.Y) {
+		} else if (finalBB.Min.y + COLLISIONS_ADJ >= blockBB.Max.y) {
 			Collisions_ClipYMax(comp, &blockBB, entityBB, extentBB, &size);
-		} else if (finalBB.Max.Y - COLLISIONS_ADJ <= blockBB.Min.Y) {
+		} else if (finalBB.Max.y - COLLISIONS_ADJ <= blockBB.Min.y) {
 			Collisions_ClipYMin(comp, &blockBB, entityBB, extentBB, &size);
 		}
 	}
@@ -960,14 +740,14 @@ void PhysicsComp_UpdateVelocityState(struct PhysicsComp* comp) {
 	cc_bool pastJumpPoint;
 
 	if (hacks->Floating) {
-		entity->Velocity.Y = 0.0f; /* eliminate the effect of gravity */
+		entity->Velocity.y = 0.0f; /* eliminate the effect of gravity */
 		dir = (hacks->FlyingUp || comp->Jumping) ? 1 : (hacks->FlyingDown ? -1 : 0);
 
-		entity->Velocity.Y += 0.12f * dir;
-		if (hacks->Speeding     && hacks->CanSpeed) entity->Velocity.Y += 0.12f * dir;
-		if (hacks->HalfSpeeding && hacks->CanSpeed) entity->Velocity.Y += 0.06f * dir;
-	} else if (comp->Jumping && Entity_TouchesAnyRope(entity) && entity->Velocity.Y > 0.02f) {
-		entity->Velocity.Y = 0.02f;
+		entity->Velocity.y += 0.12f * dir;
+		if (hacks->Speeding     && hacks->CanSpeed) entity->Velocity.y += 0.12f * dir;
+		if (hacks->HalfSpeeding && hacks->CanSpeed) entity->Velocity.y += 0.06f * dir;
+	} else if (comp->Jumping && Entity_TouchesAnyRope(entity) && entity->Velocity.y > 0.02f) {
+		entity->Velocity.y = 0.02f;
 	}
 
 	if (!comp->Jumping) { comp->CanLiquidJump = false; return; }
@@ -976,38 +756,38 @@ void PhysicsComp_UpdateVelocityState(struct PhysicsComp* comp) {
 
 	if (touchWater || touchLava) {
 		Entity_GetBounds(entity, &bounds);
-		feetY = Math_Floor(bounds.Min.Y); bodyY = feetY + 1;
-		headY = Math_Floor(bounds.Max.Y);
+		feetY = Math_Floor(bounds.Min.y); bodyY = feetY + 1;
+		headY = Math_Floor(bounds.Max.y);
 		if (bodyY > headY) bodyY = headY;
 
-		bounds.Max.Y = bounds.Min.Y = feetY;
+		bounds.Max.y = bounds.Min.y = feetY;
 		liquidFeet   = Entity_TouchesAny(&bounds, PhysicsComp_TouchesLiquid);
-		bounds.Min.Y = min(bodyY, headY);
-		bounds.Max.Y = max(bodyY, headY);
+		bounds.Min.y = min(bodyY, headY);
+		bounds.Max.y = max(bodyY, headY);
 		liquidRest   = Entity_TouchesAny(&bounds, PhysicsComp_TouchesLiquid);
 
-		pastJumpPoint = liquidFeet && !liquidRest && (Math_Mod1(entity->Position.Y) >= 0.4f);
+		pastJumpPoint = liquidFeet && !liquidRest && (Math_Mod1(entity->Position.y) >= 0.4f);
 		if (!pastJumpPoint) {
 			comp->CanLiquidJump = true;
-			entity->Velocity.Y += 0.04f;
-			if (hacks->Speeding     && hacks->CanSpeed) entity->Velocity.Y += 0.04f;
-			if (hacks->HalfSpeeding && hacks->CanSpeed) entity->Velocity.Y += 0.02f;
+			entity->Velocity.y += 0.04f;
+			if (hacks->Speeding     && hacks->CanSpeed) entity->Velocity.y += 0.04f;
+			if (hacks->HalfSpeeding && hacks->CanSpeed) entity->Velocity.y += 0.02f;
 		} else if (pastJumpPoint) {
 			/* either A) climb up solid on side B) jump bob in water */
 			if (Collisions_HitHorizontal(comp->Collisions)) {
-				entity->Velocity.Y += touchLava ? 0.30f : 0.13f;
+				entity->Velocity.y += touchLava ? 0.30f : 0.13f;
 			} else if (comp->CanLiquidJump) {
-				entity->Velocity.Y += touchLava ? 0.20f : 0.10f;
+				entity->Velocity.y += touchLava ? 0.20f : 0.10f;
 			}
 			comp->CanLiquidJump = false;
 		}
 	} else if (comp->UseLiquidGravity) {
-		entity->Velocity.Y += 0.04f;
-		if (hacks->Speeding     && hacks->CanSpeed) entity->Velocity.Y += 0.04f;
-		if (hacks->HalfSpeeding && hacks->CanSpeed) entity->Velocity.Y += 0.02f;
+		entity->Velocity.y += 0.04f;
+		if (hacks->Speeding     && hacks->CanSpeed) entity->Velocity.y += 0.04f;
+		if (hacks->HalfSpeeding && hacks->CanSpeed) entity->Velocity.y += 0.02f;
 		comp->CanLiquidJump = false;
 	} else if (Entity_TouchesAnyRope(entity)) {
-		entity->Velocity.Y += (hacks->Speeding && hacks->CanSpeed) ? 0.15f : 0.10f;
+		entity->Velocity.y += (hacks->Speeding && hacks->CanSpeed) ? 0.15f : 0.10f;
 		comp->CanLiquidJump = false;
 	} else if (entity->OnGround) {
 		PhysicsComp_DoNormalJump(comp);
@@ -1019,9 +799,9 @@ void PhysicsComp_DoNormalJump(struct PhysicsComp* comp) {
 	struct HacksComp* hacks = comp->Hacks;
 	if (comp->JumpVel == 0.0f || hacks->MaxJumps <= 0) return;
 
-	entity->Velocity.Y = comp->JumpVel;
-	if (hacks->Speeding     && hacks->CanSpeed) entity->Velocity.Y += comp->JumpVel;
-	if (hacks->HalfSpeeding && hacks->CanSpeed) entity->Velocity.Y += comp->JumpVel / 2;
+	entity->Velocity.y = comp->JumpVel;
+	if (hacks->Speeding     && hacks->CanSpeed) entity->Velocity.y += comp->JumpVel;
+	if (hacks->HalfSpeeding && hacks->CanSpeed) entity->Velocity.y += comp->JumpVel / 2;
 	comp->CanLiquidJump = false;
 }
 
@@ -1031,15 +811,15 @@ static cc_bool PhysicsComp_OnIce(struct Entity* e) {
 	int feetX, feetY, feetZ;
 	BlockID feetBlock;
 
-	feetX = Math_Floor(e->Position.X);
-	feetY = Math_Floor(e->Position.Y - 0.01f);
-	feetZ = Math_Floor(e->Position.Z);
+	feetX = Math_Floor(e->Position.x);
+	feetY = Math_Floor(e->Position.y - 0.01f);
+	feetZ = Math_Floor(e->Position.z);
 
 	feetBlock = World_GetPhysicsBlock(feetX, feetY, feetZ);
 	if (Blocks.ExtendedCollide[feetBlock] == COLLIDE_ICE) return true;
 
 	Entity_GetBounds(e, &bounds);
-	bounds.Min.Y -= 0.01f; bounds.Max.Y = bounds.Min.Y;
+	bounds.Min.y -= 0.01f; bounds.Max.y = bounds.Min.y;
 	return Entity_TouchesAny(&bounds, PhysicsComp_TouchesSlipperyIce);
 }
 
@@ -1047,7 +827,7 @@ static void PhysicsComp_MoveHor(struct PhysicsComp* comp, Vec3 vel, float factor
 	struct Entity* entity;
 	float dist;
 
-	dist = Math_SqrtF(vel.X * vel.X + vel.Z * vel.Z);
+	dist = Math_SqrtF(vel.x * vel.x + vel.z * vel.z);
 	if (dist < 0.00001f) return;
 	if (dist < 1.0f) dist = 1.0f;
 
@@ -1059,16 +839,16 @@ static void PhysicsComp_MoveHor(struct PhysicsComp* comp, Vec3 vel, float factor
 
 static void PhysicsComp_Move(struct PhysicsComp* comp, Vec3 drag, float gravity, float yMul) {
 	struct Entity* entity = comp->Entity;
-	entity->Velocity.Y *= yMul;
+	entity->Velocity.y *= yMul;
 
 	if (!comp->Hacks->Noclip) {
 		Collisions_MoveAndWallSlide(comp->Collisions);
 	}
 	Vec3_AddBy(&entity->Position, &entity->Velocity);
 
-	entity->Velocity.Y /= yMul;
+	entity->Velocity.y /= yMul;
 	Vec3_Mul3By(&entity->Velocity, &drag);
-	entity->Velocity.Y -= gravity;
+	entity->Velocity.y -= gravity;
 }
 
 static void PhysicsComp_MoveFlying(struct PhysicsComp* comp, Vec3 vel, float factor, Vec3 drag, float gravity, float yMul) {
@@ -1077,13 +857,13 @@ static void PhysicsComp_MoveFlying(struct PhysicsComp* comp, Vec3 vel, float fac
 	float yVel;
 
 	PhysicsComp_MoveHor(comp, vel, factor);
-	yVel = Math_SqrtF(entity->Velocity.X * entity->Velocity.X + entity->Velocity.Z * entity->Velocity.Z);
+	yVel = Math_SqrtF(entity->Velocity.x * entity->Velocity.x + entity->Velocity.z * entity->Velocity.z);
 	/* make horizontal speed the same as vertical speed */
-	if ((vel.X != 0.0f || vel.Z != 0.0f) && yVel > 0.001f) {
-		entity->Velocity.Y = 0.0f;
+	if ((vel.x != 0.0f || vel.z != 0.0f) && yVel > 0.001f) {
+		entity->Velocity.y = 0.0f;
 		yMul = 1.0f;
-		if (hacks->FlyingUp || comp->Jumping) entity->Velocity.Y += yVel;
-		if (hacks->FlyingDown)                entity->Velocity.Y -= yVel;
+		if (hacks->FlyingUp || comp->Jumping) entity->Velocity.y += yVel;
+		if (hacks->FlyingDown)                entity->Velocity.y -= yVel;
 	}
 	PhysicsComp_Move(comp, drag, gravity, yMul);
 }
@@ -1105,13 +885,13 @@ static float PhysicsComp_LowestModifier(struct PhysicsComp* comp, struct AABB* b
 	IVec3_Floor(&bbMin, &bounds->Min);
 	IVec3_Floor(&bbMax, &bounds->Max);	
 
-	bbMin.X = max(bbMin.X, 0); bbMax.X = min(bbMax.X, World.MaxX);
-	bbMin.Y = max(bbMin.Y, 0); bbMax.Y = min(bbMax.Y, World.MaxY);
-	bbMin.Z = max(bbMin.Z, 0); bbMax.Z = min(bbMax.Z, World.MaxZ);
+	bbMin.x = max(bbMin.x, 0); bbMax.x = min(bbMax.x, World.MaxX);
+	bbMin.y = max(bbMin.y, 0); bbMax.y = min(bbMax.y, World.MaxY);
+	bbMin.z = max(bbMin.z, 0); bbMax.z = min(bbMax.z, World.MaxZ);
 	
-	for (y = bbMin.Y; y <= bbMax.Y; y++) { v.Y = (float)y;
-		for (z = bbMin.Z; z <= bbMax.Z; z++) { v.Z = (float)z;
-			for (x = bbMin.X; x <= bbMax.X; x++) { v.X = (float)x;
+	for (y = bbMin.y; y <= bbMax.y; y++) { v.y = (float)y;
+		for (z = bbMin.z; z <= bbMax.z; z++) { v.z = (float)z;
+			for (x = bbMin.x; x <= bbMax.x; x++) { v.x = (float)x;
 				block = World_GetBlock(x, y, z);
 
 				if (block == BLOCK_AIR) continue;
@@ -1146,7 +926,7 @@ static float PhysicsComp_GetBaseSpeed(struct PhysicsComp* comp) {
 	comp->UseLiquidGravity = false;
 
 	baseModifier  = PhysicsComp_LowestModifier(comp, &bounds, false);
-	bounds.Min.Y -= 0.5f/16.0f; /* also check block standing on */
+	bounds.Min.y -= 0.5f/16.0f; /* also check block standing on */
 	solidModifier = PhysicsComp_LowestModifier(comp, &bounds, true);
 
 	if (baseModifier == MATH_LARGENUM && solidModifier == MATH_LARGENUM) return 1.0f;
@@ -1201,13 +981,13 @@ void PhysicsComp_PhysicsTick(struct PhysicsComp* comp, Vec3 vel) {
 
 		if (PhysicsComp_OnIce(entity) && !hacks->Floating) {
 			/* limit components to +-0.25f by rescaling vector to [-0.25, 0.25] */
-			if (Math_AbsF(entity->Velocity.X) > 0.25f || Math_AbsF(entity->Velocity.Z) > 0.25f) {
-				float xScale = Math_AbsF(0.25f / entity->Velocity.X);
-				float zScale = Math_AbsF(0.25f / entity->Velocity.Z);
+			if (Math_AbsF(entity->Velocity.x) > 0.25f || Math_AbsF(entity->Velocity.z) > 0.25f) {
+				float xScale = Math_AbsF(0.25f / entity->Velocity.x);
+				float zScale = Math_AbsF(0.25f / entity->Velocity.z);
 
 				float scale = min(xScale, zScale);
-				entity->Velocity.X *= scale;
-				entity->Velocity.Z *= scale;
+				entity->Velocity.x *= scale;
+				entity->Velocity.z *= scale;
 			}
 		} else if (entity->OnGround || hacks->Flying) {
 			Vec3_Mul3By(&entity->Velocity, &entity->Model->groundFriction); /* air drag or ground friction */
@@ -1222,14 +1002,14 @@ static double PhysicsComp_YPosAt(int t, float u) {
 	/* x(t, u) = Σv(t, u) from 0 to t (since we work in discrete timesteps) */
 	/* plugging into Wolfram Alpha gives 1 equation as */
 	/* (0.98^t) * (-49u - 196) - 4t + 50u + 196 */
-	double a = Math_Exp(-0.0202027 * t); /* ~0.98^t */
+	double a = Math_Exp2(-0.02914633510256746 * t); /* ~0.98^t */
 	return a * (-49 * u - 196) - 4 * t + 50 * u + 196;
 }
 
 double PhysicsComp_CalcMaxHeight(float u) {
 	/* equation below comes from solving diff(x(t, u))= 0 */
 	/* We only work in discrete timesteps, so test both rounded up and down */
-	double t = 49.49831645 * Math_Log(0.247483075 * u + 0.9899323);
+	double t = 34.30961849 * Math_Log2(0.247483075 * u + 0.9899323);
 	double value_floor = PhysicsComp_YPosAt((int)t,     u);
 	double value_ceil  = PhysicsComp_YPosAt((int)t + 1, u);
 	return max(value_floor, value_ceil);
@@ -1255,7 +1035,7 @@ void PhysicsComp_DoEntityPush(struct Entity* entity) {
 	Vec3 dir;
 	float dist, pushStrength;
 	int id;
-	dir.Y = 0.0f;
+	dir.y = 0.0f;
 
 	for (id = 0; id < ENTITIES_MAX_COUNT; id++) {
 		other = Entities.List[id];
@@ -1263,13 +1043,13 @@ void PhysicsComp_DoEntityPush(struct Entity* entity) {
 		if (!other->Model->pushes)     continue;
 
 		yIntersects =
-			entity->Position.Y <= (other->Position.Y  + other->Size.Y) &&
-			 other->Position.Y <= (entity->Position.Y + entity->Size.Y);
+			entity->Position.y <= (other->Position.y  + other->Size.y) &&
+			 other->Position.y <= (entity->Position.y + entity->Size.y);
 		if (!yIntersects) continue;
 
-		dir.X = other->Position.X - entity->Position.X;
-		dir.Z = other->Position.Z - entity->Position.Z;
-		dist = dir.X * dir.X + dir.Z * dir.Z;
+		dir.x = other->Position.x - entity->Position.x;
+		dir.z = other->Position.z - entity->Position.z;
+		dist = dir.x * dir.x + dir.z * dir.z;
 		if (dist < 0.002f || dist > 1.0f) continue; /* TODO: range needs to be lower? */
 
 		Vec3_Normalise(&dir);
@@ -1284,7 +1064,7 @@ void PhysicsComp_DoEntityPush(struct Entity* entity) {
 /*########################################################################################################################*
 *----------------------------------------------------SoundsComponent------------------------------------------------------*
 *#########################################################################################################################*/
-static Vec3 sounds_lastPos = { -1e25f, -1e25f, -1e25f };
+static Vec3 sounds_lastPos = { -87.1234f, -99.5678f, -100.91237f };
 static cc_bool  sounds_anyNonAir;
 static cc_uint8 sounds_type;
 
@@ -1322,19 +1102,19 @@ static void SoundComp_GetSound(struct LocalPlayer* p) {
 	if (sounds_type != SOUND_NONE) return;
 
 	/* then check block standing on (feet) */
-	pos = p->Base.next.pos; pos.Y -= 0.01f;
+	pos = p->Base.next.pos; pos.y -= 0.01f;
 	IVec3_Floor(&coords, &pos);
-	blockUnder = World_SafeGetBlock(coords.X, coords.Y, coords.Z);
-	maxY = coords.Y + Blocks.MaxBB[blockUnder].Y;
+	blockUnder = World_SafeGetBlock(coords.x, coords.y, coords.z);
+	maxY = coords.y + Blocks.MaxBB[blockUnder].y;
 
 	typeUnder    = Blocks.StepSounds[blockUnder];
 	collideUnder = Blocks.Collide[blockUnder];
-	if (maxY >= pos.Y && collideUnder == COLLIDE_SOLID && typeUnder != SOUND_NONE) {
+	if (maxY >= pos.y && collideUnder == COLLIDE_SOLID && typeUnder != SOUND_NONE) {
 		sounds_anyNonAir = true; sounds_type = typeUnder; return;
 	}
 
 	/* then check all solid blocks at feet */
-	bounds.Max.Y = bounds.Min.Y = pos.Y;
+	bounds.Max.y = bounds.Min.y = pos.y;
 	Entity_TouchesAny(&bounds, Sounds_CheckSolid);
 }
 
@@ -1350,18 +1130,17 @@ static cc_bool SoundComp_ShouldPlay(struct LocalPlayer* p, Vec3 soundPos) {
 
 	/* have our legs just crossed over the '0' point? */
 	if (Camera.Active->isThirdPerson) {
-		oldLegRot = (float)Math_Cos(p->Base.Anim.WalkTimeO);
-		newLegRot = (float)Math_Cos(p->Base.Anim.WalkTimeN);
+		oldLegRot = Math_CosF(p->Base.Anim.WalkTimeO);
+		newLegRot = Math_CosF(p->Base.Anim.WalkTimeN);
 	} else {
-		oldLegRot = (float)Math_Sin(p->Base.Anim.WalkTimeO);
-		newLegRot = (float)Math_Sin(p->Base.Anim.WalkTimeN);
+		oldLegRot = Math_SinF(p->Base.Anim.WalkTimeO);
+		newLegRot = Math_SinF(p->Base.Anim.WalkTimeN);
 	}
 	return Math_Sign(oldLegRot) != Math_Sign(newLegRot);
 }
 
-void SoundComp_Tick(cc_bool wasOnGround) {
-	struct LocalPlayer* p = &LocalPlayer_Instance;
-	Vec3 soundPos         = p->Base.next.pos;
+void SoundComp_Tick(struct LocalPlayer* p, cc_bool wasOnGround) {
+	Vec3 soundPos = p->Base.next.pos;
 
 	SoundComp_GetSound(p);
 	if (!sounds_anyNonAir) soundPos = Vec3_BigPos();
