@@ -18,15 +18,15 @@
 #include "Camera.h"
 #include "Particle.h"
 #include "Options.h"
+#include "Entity.h"
 
 cc_bool EnvRenderer_Legacy, EnvRenderer_Minimal;
 
 static float CalcBlendFactor(float x) {
-	/* return -0.05 + 0.22 * (Math_Log(x) * 0.25f); */
-	double blend = -0.13 + 0.28 * (Math_Log(x) * 0.25);
-	if (blend < 0.0) blend = 0.0;
-	if (blend > 1.0) blend = 1.0;
-	return (float)blend;
+	float blend = -0.13f + 0.28f * ((float)Math_Log2(x) * 0.17329f);
+	if (blend < 0.0f) blend = 0.0f;
+	if (blend > 1.0f) blend = 1.0f;
+	return blend;
 }
 
 #define EnvRenderer_AxisSize() (EnvRenderer_Legacy ? 128 : 2048)
@@ -40,21 +40,25 @@ static int CalcNumVertices(int axis1Len, int axis2Len) {
 /*########################################################################################################################*
 *------------------------------------------------------------Fog----------------------------------------------------------*
 *#########################################################################################################################*/
-static void CalcFog(float* density, PackedCol* color) {
+static cc_bool CameraInsideBlock(BlockID block, IVec3* coords) {
+	struct AABB blockBB;
 	Vec3 pos;
+	IVec3_ToVec3(&pos, coords); /* pos = coords; */
+
+	Vec3_Add(&blockBB.Min, &pos, &Blocks.MinBB[block]);
+	Vec3_Add(&blockBB.Max, &pos, &Blocks.MaxBB[block]);
+	return AABB_ContainsPoint(&blockBB, &Camera.CurrentPos);
+}
+
+static void CalcFog(float* density, PackedCol* color) {
 	IVec3 coords;
 	BlockID block;
-	struct AABB blockBB;
 	float blend;
 
 	IVec3_Floor(&coords, &Camera.CurrentPos); /* coords = floor(camera_pos); */
-	IVec3_ToVec3(&pos, &coords);              /* pos = coords; */
+	block = World_SafeGetBlock(coords.x, coords.y, coords.z);
 
-	block = World_SafeGetBlock(coords.X, coords.Y, coords.Z);
-	Vec3_Add(&blockBB.Min, &pos, &Blocks.MinBB[block]);
-	Vec3_Add(&blockBB.Max, &pos, &Blocks.MaxBB[block]);
-
-	if (AABB_ContainsPoint(&blockBB, &Camera.CurrentPos) && Blocks.FogDensity[block]) {
+	if (Blocks.FogDensity[block] && CameraInsideBlock(block, &coords)) {
 		*density = Blocks.FogDensity[block];
 		*color   = Blocks.FogCol[block];
 	} else {
@@ -73,7 +77,7 @@ static void UpdateFogMinimal(float fogDensity) {
 		/* Exp fog mode: f = e^(-density*coord) */
 		/* Solve coord for f = 0.05 (good approx for fog end) */
 		/*   i.e. log(0.05) = -density * coord */
-		#define LOG_005 -2.99573227355399
+		#define LOG_005 -2.99573227355399f
 
 		dist = (int)(LOG_005 / -fogDensity);
 		Game_SetViewDistance(min(dist, Game_UserViewDistance));
@@ -83,7 +87,7 @@ static void UpdateFogMinimal(float fogDensity) {
 }
 
 static void UpdateFogNormal(float fogDensity, PackedCol fogColor) {
-	double density;
+	float density;
 
 	if (fogDensity != 0.0f) {
 		Gfx_SetFogMode(FOG_EXP);
@@ -96,10 +100,10 @@ static void UpdateFogNormal(float fogDensity, PackedCol fogColor) {
 		   0.99=z/end   --> z=end*0.99
 		     therefore
 		  d = -ln(0.01)/(end*0.99) */
-		#define LOG_001 -4.60517018598809
+		#define LOG_001 -4.60517018598809f
 
-		density = -(LOG_001) / (Game_ViewDistance * 0.99);
-		Gfx_SetFogDensity((float)density);
+		density = -LOG_001 / (Game_ViewDistance * 0.99f);
+		Gfx_SetFogDensity(density);
 	} else {
 		Gfx_SetFogMode(FOG_LINEAR);
 		Gfx_SetFogEnd((float)Game_ViewDistance);
@@ -114,7 +118,7 @@ void EnvRenderer_UpdateFog(void) {
 	if (!World.Loaded) return;
 
 	CalcFog(&fogDensity, &fogColor);
-	Gfx_ClearCol(fogColor);
+	Gfx_ClearColor(fogColor);
 
 	if (EnvRenderer_Minimal) {
 		UpdateFogMinimal(fogDensity);
@@ -164,10 +168,10 @@ static void DrawCloudsY(int x1, int z1, int x2, int z2, int y, struct VertexText
 			u1 = (float)x1 / 2048.0f + offset; u2 = (float)x2 / 2048.0f + offset;
 			v1 = (float)z1 / 2048.0f + offset; v2 = (float)z2 / 2048.0f + offset;
 
-			v->X = (float)x1; v->Y = yy; v->Z = (float)z1; v->Col = col; v->U = u1; v->V = v1; v++;
-			v->X = (float)x1; v->Y = yy; v->Z = (float)z2; v->Col = col; v->U = u1; v->V = v2; v++;
-			v->X = (float)x2; v->Y = yy; v->Z = (float)z2; v->Col = col; v->U = u2; v->V = v2; v++;
-			v->X = (float)x2; v->Y = yy; v->Z = (float)z1; v->Col = col; v->U = u2; v->V = v1; v++;
+			v->x = (float)x1; v->y = yy; v->z = (float)z1; v->Col = col; v->U = u1; v->V = v1; v++;
+			v->x = (float)x1; v->y = yy; v->z = (float)z2; v->Col = col; v->U = u1; v->V = v2; v++;
+			v->x = (float)x2; v->y = yy; v->z = (float)z2; v->Col = col; v->U = u2; v->V = v2; v++;
+			v->x = (float)x2; v->y = yy; v->z = (float)z1; v->Col = col; v->U = u2; v->V = v1; v++;
 		}
 	}
 }
@@ -205,7 +209,7 @@ void EnvRenderer_RenderSky(void) {
 	if (!sky_vb || EnvRenderer_ShouldRenderSkybox()) return;
 
 	normY = (float)World.Height + 8.0f;
-	skyY  = max(Camera.CurrentPos.Y + 8.0f, normY);
+	skyY  = max(Camera.CurrentPos.y + 8.0f, normY);
 	Gfx_SetVertexFormat(VERTEX_FORMAT_COLOURED);
 	Gfx_BindVb(sky_vb);
 
@@ -215,8 +219,8 @@ void EnvRenderer_RenderSky(void) {
 		m  = Gfx.View;
 		dy = skyY - normY; 
 		/* inlined Y translation matrix multiply */
-		m.row4.X += dy * m.row2.X; m.row4.Y += dy * m.row2.Y;
-		m.row4.Z += dy * m.row2.Z; m.row4.W += dy * m.row2.W;
+		m.row4.x += dy * m.row2.x; m.row4.y += dy * m.row2.y;
+		m.row4.z += dy * m.row2.z; m.row4.w += dy * m.row2.w;
 
 		Gfx_LoadMatrix(MATRIX_VIEW, &m);
 		Gfx_DrawVb_IndexedTris(sky_vertices);
@@ -236,10 +240,10 @@ static void DrawSkyY(int x1, int z1, int x2, int z2, int y, struct VertexColoure
 			z2 = z1 + axisSize;
 			if (z2 > endZ) z2 = endZ;
 
-			v->X = (float)x1; v->Y = (float)y; v->Z = (float)z1; v->Col = col; v++;
-			v->X = (float)x1; v->Y = (float)y; v->Z = (float)z2; v->Col = col; v++;
-			v->X = (float)x2; v->Y = (float)y; v->Z = (float)z2; v->Col = col; v++;
-			v->X = (float)x2; v->Y = (float)y; v->Z = (float)z1; v->Col = col; v++;
+			v->x = (float)x1; v->y = (float)y; v->z = (float)z1; v->Col = col; v++;
+			v->x = (float)x1; v->y = (float)y; v->z = (float)z2; v->Col = col; v++;
+			v->x = (float)x2; v->y = (float)y; v->z = (float)z2; v->Col = col; v++;
+			v->x = (float)x2; v->y = (float)y; v->z = (float)z1; v->Col = col; v++;
 		}
 	}
 }
@@ -272,37 +276,7 @@ static GfxResourceID skybox_tex, skybox_vb;
 #define SKYBOX_COUNT (6 * 4)
 cc_bool EnvRenderer_ShouldRenderSkybox(void) { return skybox_tex && !EnvRenderer_Minimal; }
 
-void EnvRenderer_RenderSkybox(void) {
-	struct Matrix m, rotX, rotY, view;
-	float rotTime;
-	Vec3 pos;
-	if (!skybox_vb) return;
-
-	Gfx_SetDepthWrite(false);
-	Gfx_BindTexture(skybox_tex);
-	Gfx_SetVertexFormat(VERTEX_FORMAT_TEXTURED);
-
-	/* Base skybox rotation */
-	rotTime = (float)(Game.Time * 2 * MATH_PI); /* So speed of 1 rotates whole skybox every second */
-	Matrix_RotateY(&rotY, Env.SkyboxHorSpeed * rotTime);
-	Matrix_RotateX(&rotX, Env.SkyboxVerSpeed * rotTime);
-	Matrix_Mul(&m, &rotY, &rotX);
-
-	/* Rotate around camera */
-	pos = Camera.CurrentPos;
-	Vec3_Set(Camera.CurrentPos, 0,0,0);
-	Camera.Active->GetView(&view); Matrix_MulBy(&m, &view);
-	Camera.CurrentPos = pos;
-
-	Gfx_LoadMatrix(MATRIX_VIEW, &m);
-	Gfx_BindVb(skybox_vb);
-	Gfx_DrawVb_IndexedTris(SKYBOX_COUNT);
-
-	Gfx_LoadMatrix(MATRIX_VIEW, &Gfx.View);
-	Gfx_SetDepthWrite(true);
-}
-
-static void UpdateSkybox(void) {
+static void AllocateSkyboxVB(void) {
 	static const struct VertexTextured vertices[SKYBOX_COUNT] = {
 		/* Front quad */
 		{ -1, -1, -1,  0, 0.25f, 1.00f }, {  1, -1, -1,  0, 0.50f, 1.00f },
@@ -326,10 +300,6 @@ static void UpdateSkybox(void) {
 	struct VertexTextured* data;
 	int i;
 
-	Gfx_DeleteVb(&skybox_vb);
-	if (Gfx.LostContext)     return;
-	if (EnvRenderer_Minimal) return;
-
 	data = (struct VertexTextured*)Gfx_RecreateAndLockVb(&skybox_vb,
 										VERTEX_FORMAT_TEXTURED, SKYBOX_COUNT);
 	Mem_Copy(data, vertices, sizeof(vertices));
@@ -337,17 +307,50 @@ static void UpdateSkybox(void) {
 	Gfx_UnlockVb(skybox_vb);
 }
 
+void EnvRenderer_RenderSkybox(void) {
+	struct Matrix m, rotX, rotY, view;
+	float rotTime;
+	Vec3 pos;
+	if (!skybox_vb) AllocateSkyboxVB();
+
+	Gfx_SetDepthWrite(false);
+	Gfx_BindTexture(skybox_tex);
+	Gfx_SetVertexFormat(VERTEX_FORMAT_TEXTURED);
+
+	/* Base skybox rotation */
+	rotTime = (float)(Game.Time * 2 * MATH_PI); /* So speed of 1 rotates whole skybox every second */
+	Matrix_RotateY(&rotY, Env.SkyboxHorSpeed * rotTime);
+	Matrix_RotateX(&rotX, Env.SkyboxVerSpeed * rotTime);
+	Matrix_Mul(&m, &rotY, &rotX);
+
+	/* Rotate around camera */
+	pos = Camera.CurrentPos;
+	Vec3_Set(Camera.CurrentPos, 0,0,0);
+	Camera.Active->GetView(&view); 
+	Matrix_MulBy(&m, &view);
+	Camera.CurrentPos = pos;
+
+	Gfx_LoadMatrix(MATRIX_VIEW, &m);
+	Gfx_BindVb(skybox_vb);
+	Gfx_DrawVb_IndexedTris(SKYBOX_COUNT);
+
+	Gfx_LoadMatrix(MATRIX_VIEW, &Gfx.View);
+	Gfx_SetDepthWrite(true);
+}
 
 /*########################################################################################################################*
 *----------------------------------------------------------Weather--------------------------------------------------------*
 *#########################################################################################################################*/
 cc_int16* Weather_Heightmap;
 static GfxResourceID rain_tex, snow_tex, weather_vb;
-static double weather_accumulator;
+static float weather_accumulator;
 static IVec3 lastPos;
 
 #define WEATHER_EXTENT 4
-#define WEATHER_VERTS_COUNT 8 * (WEATHER_EXTENT * 2 + 1) * (WEATHER_EXTENT * 2 + 1)
+#define WEATHER_VERTS  8 /* 2 quads per tile */
+#define WEATHER_RANGE  (WEATHER_EXTENT * 2 + 1)
+
+#define WEATHER_VERTS_COUNT WEATHER_RANGE * WEATHER_RANGE * WEATHER_VERTS
 #define Weather_Pack(x, z) ((x) * World.Length + (z))
 
 static void InitWeatherHeightmap(void) {
@@ -396,7 +399,7 @@ static float GetRainHeight(int x, int z) {
 	height = Weather_Heightmap[hIndex];
 
 	y = height == Int16_MaxValue ? CalcRainHeightAt(x, World.MaxY, z, hIndex) : height;
-	return y == -1 ? 0 : y + Blocks.MaxBB[World_GetBlock(x, y, z)].Y;
+	return y == -1 ? 0 : y + Blocks.MaxBB[World_GetBlock(x, y, z)].y;
 }
 
 void EnvRenderer_OnBlockChanged(int x, int y, int z, BlockID oldBlock, BlockID newBlock) {
@@ -428,104 +431,127 @@ static float CalcRainAlphaAt(float x) {
 	return 178 + falloff * Env.WeatherFade;
 }
 
+struct RainCoord { int dx, dz; float y; };
 static RNGState snowDirRng;
-void EnvRenderer_RenderWeather(double deltaTime) {
-	struct VertexTextured vertices[WEATHER_VERTS_COUNT];
+
+void EnvRenderer_RenderWeather(float delta) {
+	struct RainCoord coords[WEATHER_RANGE * WEATHER_RANGE];
+	int i, weather, numCoords = 0;
 	struct VertexTextured* v;
-	int weather, vCount;
-	IVec3 pos;
 	cc_bool moved, particles;
 	float speed, vOffsetBase, vOffset;
+	IVec3 pos;
 
-	PackedCol col;
+	PackedCol color;
 	int dist, dx, dz, x, z;
 	float alpha, y, height;
-	float uOffset1, uOffset2;
+	float uOffset1, uOffset2, uSpeed;
 	float worldV, v1, v2, vPlane1Offset;
 	float x1,y1,z1, x2,y2,z2;
 
 	weather = Env.Weather;
 	if (weather == WEATHER_SUNNY) return;
-	if (!Weather_Heightmap) InitWeatherHeightmap();
-	Gfx_BindTexture(weather == WEATHER_RAINY ? rain_tex : snow_tex);
+
+	if (!Weather_Heightmap) 
+		InitWeatherHeightmap();
+	if (!weather_vb)
+		weather_vb = Gfx_CreateDynamicVb(VERTEX_FORMAT_TEXTURED, WEATHER_VERTS_COUNT);
 
 	IVec3_Floor(&pos, &Camera.CurrentPos);
-	moved   = pos.X != lastPos.X || pos.Y != lastPos.Y || pos.Z != lastPos.Z;
+	moved   = pos.x != lastPos.x || pos.y != lastPos.y || pos.z != lastPos.z;
 	lastPos = pos;
 
 	/* Rain should extend up by 64 blocks, or to the top of the world. */
-	pos.Y += 64;
-	pos.Y = max(World.Height, pos.Y);
+	pos.y += 64;
+	pos.y = max(World.Height, pos.y);
 
-	speed         = (weather == WEATHER_RAINY ? 1.0f : 0.2f) * Env.WeatherSpeed;
-	vOffsetBase   = (float)Game.Time * speed;
-	vPlane1Offset = weather == WEATHER_RAINY ? 0 : 0.25f; /* Offset v on 1 plane while snowing to avoid the unnatural mirrored texture effect */
-	particles     = weather == WEATHER_RAINY;
-	weather_accumulator += deltaTime;
-
-	v   = vertices;
-	col = Env.SunCol;
+	weather_accumulator += delta;
+	particles = weather == WEATHER_RAINY && (weather_accumulator >= 0.25f || moved);
 
 	for (dx = -WEATHER_EXTENT; dx <= WEATHER_EXTENT; dx++) {
 		for (dz = -WEATHER_EXTENT; dz <= WEATHER_EXTENT; dz++) {
-			x = pos.X + dx; z = pos.Z + dz;
+			x = pos.x + dx; z = pos.z + dz;
 
 			y = GetRainHeight(x, z);
-			height = pos.Y - y;
-			if (height <= 0) continue;
+			if (pos.y <= y) continue;
+			if (particles) Particles_RainSnowEffect((float)x, y, (float)z);
 
-			if (particles && (weather_accumulator >= 0.25 || moved)) {
-				Particles_RainSnowEffect((float)x, y, (float)z);
-			}
-
-			dist  = dx * dx + dz * dz;
-			alpha = CalcRainAlphaAt((float)dist);
-			Math_Clamp(alpha, 0.0f, 255.0f);
-			col   = (col & PACKEDCOL_RGB_MASK) | PackedCol_A_Bits(alpha);
-
-			uOffset1 = 0;
-			uOffset2 = 0;
-			if (weather == WEATHER_SNOWY) {
-				Random_Seed(&snowDirRng, (x + 1217 * z) & 0x7fffffff);
-				/* Multiply horizontal speed by a random float from -1 to 1 */
-				uOffset1 = ((float)Game.Time * (Random_Float(&snowDirRng) * 2 + -1)) * Env.WeatherSpeed * 0.5f;
-				uOffset2 = ((float)Game.Time * (Random_Float(&snowDirRng) * 2 + -1)) * Env.WeatherSpeed * 0.5f;
-				/* Multiply vertical speed by a random float from 1.0 to 0.25 */
-				vOffset = vOffsetBase * (float)(Random_Float(&snowDirRng) * (1.0f - 0.25f) + 0.25f);
-			} else {
-				vOffset = vOffsetBase;
-			}
-			
-			worldV = vOffset + (z & 1) / 2.0f - (x & 0x0F) / 16.0f;
-			v1 = y            / 6.0f + worldV; 
-			v2 = (y + height) / 6.0f + worldV;
-			x1 = (float)x;       y1 = (float)y;            z1 = (float)z;
-			x2 = (float)(x + 1); y2 = (float)(y + height); z2 = (float)(z + 1);
-
-			v->X = x1; v->Y = y1; v->Z = z1; v->Col = col; v->U = 0.0f + uOffset1; v->V = v1+vPlane1Offset; v++;
-			v->X = x1; v->Y = y2; v->Z = z1; v->Col = col; v->U = 0.0f + uOffset1; v->V = v2+vPlane1Offset; v++;
-			v->X = x2; v->Y = y2; v->Z = z2; v->Col = col; v->U = 1.0f + uOffset1; v->V = v2+vPlane1Offset; v++;
-			v->X = x2; v->Y = y1; v->Z = z2; v->Col = col; v->U = 1.0f + uOffset1; v->V = v1+vPlane1Offset; v++;
-
-			v->X = x2; v->Y = y1; v->Z = z1; v->Col = col; v->U = 1.0f + uOffset2; v->V = v1; v++;
-			v->X = x2; v->Y = y2; v->Z = z1; v->Col = col; v->U = 1.0f + uOffset2; v->V = v2; v++;
-			v->X = x1; v->Y = y2; v->Z = z2; v->Col = col; v->U = 0.0f + uOffset2; v->V = v2; v++;
-			v->X = x1; v->Y = y1; v->Z = z2; v->Col = col; v->U = 0.0f + uOffset2; v->V = v1; v++;
+			coords[numCoords].dx = dx;
+			coords[numCoords].y  = y;
+			coords[numCoords].dz = dz;
+			numCoords++;
 		}
 	}
 
-	if (particles && (weather_accumulator >= 0.25f || moved)) {
-		weather_accumulator = 0;
-	}
-	if (v == vertices) return;
+	Gfx_BindTexture(weather == WEATHER_RAINY ? rain_tex : snow_tex);
+	if (particles) weather_accumulator = 0;
+	if (!numCoords) return;
 
 	Gfx_SetAlphaTest(false);
 	Gfx_SetDepthWrite(false);
 	Gfx_SetAlphaArgBlend(true);
 
 	Gfx_SetVertexFormat(VERTEX_FORMAT_TEXTURED);
-	vCount = (int)(v - vertices);
-	Gfx_UpdateDynamicVb_IndexedTris(weather_vb, vertices, vCount);
+	v = (struct VertexTextured*)Gfx_LockDynamicVb(weather_vb, 
+										VERTEX_FORMAT_TEXTURED, numCoords * WEATHER_VERTS);
+
+	color = Env.SunCol;
+	speed = (weather == WEATHER_RAINY ? 1.0f : 0.2f) * Env.WeatherSpeed;
+
+	vOffsetBase   = (float)Game.Time * speed;
+	vPlane1Offset = weather == WEATHER_RAINY  ? 0 : 0.25f; /* Offset v on 1 plane while snowing to avoid the unnatural mirrored texture effect */
+
+	for (i = 0; i < numCoords; i++)
+	{
+		dx = coords[i].dx;
+		y  = coords[i].y;
+		dz = coords[i].dz;
+
+		height = pos.y - y;
+
+		dist  = dx * dx + dz * dz;
+		alpha = CalcRainAlphaAt((float)dist);
+		Math_Clamp(alpha, 0.0f, 255.0f);
+		color = (color & PACKEDCOL_RGB_MASK) | PackedCol_A_Bits(alpha);
+
+		x = dx + pos.x;
+		z = dz + pos.z;
+
+		uOffset1 = 0;
+		uOffset2 = 0;
+		if (weather == WEATHER_SNOWY) {
+			Random_Seed(&snowDirRng, (x + 1217 * z) & 0x7fffffff);
+
+			/* Multiply horizontal speed by a random float from -1 to 1 */
+			uSpeed   = (float)Game.Time * Env.WeatherSpeed * 0.5f;
+			uOffset1 = uSpeed * (Random_Float(&snowDirRng) * 2 + -1);
+			uOffset2 = uSpeed * (Random_Float(&snowDirRng) * 2 + -1);
+
+			/* Multiply vertical speed by a random float from 1.0 to 0.25 */
+			vOffset = vOffsetBase * (float)(Random_Float(&snowDirRng) * (1.0f - 0.25f) + 0.25f);
+		} else {
+			vOffset = vOffsetBase;
+		}
+		
+		worldV = vOffset + (z & 1) / 2.0f - (x & 0x0F) / 16.0f;
+		v1 = y            / 6.0f + worldV; 
+		v2 = (y + height) / 6.0f + worldV;
+		x1 = (float)x;       y1 = (float)y;            z1 = (float)z;
+		x2 = (float)(x + 1); y2 = (float)(y + height); z2 = (float)(z + 1);
+
+		v->x = x1; v->y = y1; v->z = z1; v->Col = color; v->U = uOffset1;        v->V = v1 + vPlane1Offset; v++;
+		v->x = x1; v->y = y2; v->z = z1; v->Col = color; v->U = uOffset1;        v->V = v2 + vPlane1Offset; v++;
+		v->x = x2; v->y = y2; v->z = z2; v->Col = color; v->U = uOffset1 + 1.0f; v->V = v2 + vPlane1Offset; v++;
+		v->x = x2; v->y = y1; v->z = z2; v->Col = color; v->U = uOffset1 + 1.0f; v->V = v1 + vPlane1Offset; v++;
+
+		v->x = x2; v->y = y1; v->z = z1; v->Col = color; v->U = uOffset2 + 1.0f; v->V = v1; v++;
+		v->x = x2; v->y = y2; v->z = z1; v->Col = color; v->U = uOffset2 + 1.0f; v->V = v2; v++;
+		v->x = x1; v->y = y2; v->z = z2; v->Col = color; v->U = uOffset2;        v->V = v2; v++;
+		v->x = x1; v->y = y1; v->z = z2; v->Col = color; v->U = uOffset2;        v->V = v1; v++;
+	}
+
+	Gfx_UnlockDynamicVb(weather_vb);
+	Gfx_DrawVb_IndexedTris(numCoords * WEATHER_VERTS);
 
 	Gfx_SetAlphaArgBlend(false);
 	Gfx_SetDepthWrite(true);
@@ -564,7 +590,7 @@ void EnvRenderer_RenderMapEdges(void) {
 	/* Do not draw water when player cannot see it */
 	/* Fixes some 'depth bleeding through' issues with 16 bit depth buffers on large maps */
 	int yVisible = min(0, Env_SidesHeight);
-	if (Camera.CurrentPos.Y < yVisible && sides_vb) return;
+	if (Camera.CurrentPos.y < yVisible && sides_vb) return;
 
 	RenderBorders(Env.EdgeBlock, edges_vb, edges_tex, edges_vertices);
 }
@@ -579,7 +605,7 @@ static void MakeBorderTex(GfxResourceID* texId, BlockID block) {
 
 static Rect2D EnvRenderer_Rect(int x, int y, int width, int height) {
 	Rect2D r;
-	r.X = x; r.Y = y; r.Width = width; r.Height = height; 
+	r.x = x; r.y = y; r.width = width; r.height = height; 
 	return r;
 }
 
@@ -597,8 +623,8 @@ static void UpdateBorderTextures(void) {
 	MakeBorderTex(&sides_tex, Env.SidesBlock);
 }
 
-#define Borders_HorOffset(block) (Blocks.RenderMinBB[block].X - Blocks.MinBB[block].X)
-#define Borders_YOffset(block)   (Blocks.RenderMinBB[block].Y - Blocks.MinBB[block].Y)
+#define Borders_HorOffset(block) (Blocks.RenderMinBB[block].x - Blocks.MinBB[block].x)
+#define Borders_YOffset(block)   (Blocks.RenderMinBB[block].y - Blocks.MinBB[block].y)
 
 static void DrawBorderX(int x, int z1, int z2, int y1, int y2, PackedCol color, struct VertexTextured** vertices) {
 	int endZ = z2, endY = y2, startY = y1, axisSize = EnvRenderer_AxisSize();
@@ -614,10 +640,10 @@ static void DrawBorderX(int x, int z1, int z2, int y1, int y2, PackedCol color,
 			if (y2 > endY) y2 = endY;
 
 			u2   = (float)z2 - (float)z1;      v2   = (float)y2 - (float)y1;
-			v->X = (float)x; v->Y = (float)y1; v->Z = (float)z1; v->Col = color; v->U = 0;  v->V = v2; v++;
-			v->X = (float)x; v->Y = (float)y2; v->Z = (float)z1; v->Col = color; v->U = 0;  v->V = 0;  v++;
-			v->X = (float)x; v->Y = (float)y2; v->Z = (float)z2; v->Col = color; v->U = u2; v->V = 0;  v++;
-			v->X = (float)x; v->Y = (float)y1; v->Z = (float)z2; v->Col = color; v->U = u2; v->V = v2; v++;
+			v->x = (float)x; v->y = (float)y1; v->z = (float)z1; v->Col = color; v->U = 0;  v->V = v2; v++;
+			v->x = (float)x; v->y = (float)y2; v->z = (float)z1; v->Col = color; v->U = 0;  v->V = 0;  v++;
+			v->x = (float)x; v->y = (float)y2; v->z = (float)z2; v->Col = color; v->U = u2; v->V = 0;  v++;
+			v->x = (float)x; v->y = (float)y1; v->z = (float)z2; v->Col = color; v->U = u2; v->V = v2; v++;
 		}
 	}
 	*vertices = v;
@@ -637,10 +663,10 @@ static void DrawBorderZ(int z, int x1, int x2, int y1, int y2, PackedCol color,
 			if (y2 > endY) y2 = endY;
 
 			u2   = (float)x2 - (float)x1;       v2   = (float)y2 - (float)y1;
-			v->X = (float)x1; v->Y = (float)y1; v->Z = (float)z; v->Col = color; v->U = 0;  v->V = v2; v++;
-			v->X = (float)x1; v->Y = (float)y2; v->Z = (float)z; v->Col = color; v->U = 0;  v->V = 0;  v++;
-			v->X = (float)x2; v->Y = (float)y2; v->Z = (float)z; v->Col = color; v->U = u2; v->V = 0;  v++;
-			v->X = (float)x2; v->Y = (float)y1; v->Z = (float)z; v->Col = color; v->U = u2; v->V = v2; v++;
+			v->x = (float)x1; v->y = (float)y1; v->z = (float)z; v->Col = color; v->U = 0;  v->V = v2; v++;
+			v->x = (float)x1; v->y = (float)y2; v->z = (float)z; v->Col = color; v->U = 0;  v->V = 0;  v++;
+			v->x = (float)x2; v->y = (float)y2; v->z = (float)z; v->Col = color; v->U = u2; v->V = 0;  v++;
+			v->x = (float)x2; v->y = (float)y1; v->z = (float)z; v->Col = color; v->U = u2; v->V = v2; v++;
 		}
 	}
 	*vertices = v;
@@ -661,10 +687,10 @@ static void DrawBorderY(int x1, int z1, int x2, int z2, float y, PackedCol color
 			if (z2 > endZ) z2 = endZ;
 
 			u2   = (float)x2 - (float)x1;         v2   = (float)z2 - (float)z1;
-			v->X = (float)x1 + offset; v->Y = yy; v->Z = (float)z1 + offset; v->Col = color; v->U = 0;  v->V = 0;  v++;
-			v->X = (float)x1 + offset; v->Y = yy; v->Z = (float)z2 + offset; v->Col = color; v->U = 0;  v->V = v2; v++;
-			v->X = (float)x2 + offset; v->Y = yy; v->Z = (float)z2 + offset; v->Col = color; v->U = u2; v->V = v2; v++;
-			v->X = (float)x2 + offset; v->Y = yy; v->Z = (float)z1 + offset; v->Col = color; v->U = u2; v->V = 0;  v++;
+			v->x = (float)x1 + offset; v->y = yy; v->z = (float)z1 + offset; v->Col = color; v->U = 0;  v->V = 0;  v++;
+			v->x = (float)x1 + offset; v->y = yy; v->z = (float)z2 + offset; v->Col = color; v->U = 0;  v->V = v2; v++;
+			v->x = (float)x2 + offset; v->y = yy; v->z = (float)z2 + offset; v->Col = color; v->U = u2; v->V = v2; v++;
+			v->x = (float)x2 + offset; v->y = yy; v->z = (float)z1 + offset; v->Col = color; v->U = u2; v->V = 0;  v++;
 		}
 	}
 	*vertices = v;
@@ -688,7 +714,7 @@ static void UpdateMapSides(void) {
 	sides_vertices = 0;
 	for (i = 0; i < 4; i++) {
 		r = rects[i];
-		sides_vertices += CalcNumVertices(r.Width, r.Height); /* YQuads outside */
+		sides_vertices += CalcNumVertices(r.width, r.height); /* YQuads outside */
 	}
 
 	y = Env_SidesHeight;
@@ -698,13 +724,13 @@ static void UpdateMapSides(void) {
 	data = (struct VertexTextured*)Gfx_RecreateAndLockVb(&sides_vb,
 										VERTEX_FORMAT_TEXTURED, sides_vertices);
 
-	sides_fullBright = Blocks.FullBright[block];
+	sides_fullBright = Blocks.Brightness[block];
 	color = sides_fullBright ? PACKEDCOL_WHITE : Env.ShadowCol;
 	Block_Tint(color, block)
 
 	for (i = 0; i < 4; i++) {
 		r = rects[i];
-		DrawBorderY(r.X, r.Y, r.X + r.Width, r.Y + r.Height, (float)y, color,
+		DrawBorderY(r.x, r.y, r.x + r.width, r.y + r.height, (float)y, color,
 			0, Borders_YOffset(block), &data);
 	}
 
@@ -739,19 +765,19 @@ static void UpdateMapEdges(void) {
 	edges_vertices = 0;
 	for (i = 0; i < 4; i++) {
 		r = rects[i];
-		edges_vertices += CalcNumVertices(r.Width, r.Height); /* YPlanes outside */
+		edges_vertices += CalcNumVertices(r.width, r.height); /* YPlanes outside */
 	}
 	data = (struct VertexTextured*)Gfx_RecreateAndLockVb(&edges_vb,
 										VERTEX_FORMAT_TEXTURED, edges_vertices);
 
-	edges_fullBright = Blocks.FullBright[block];
+	edges_fullBright = Blocks.Brightness[block];
 	color = edges_fullBright ? PACKEDCOL_WHITE : Env.SunCol;
 	Block_Tint(color, block)
 
 	y = (float)Env.EdgeHeight;
 	for (i = 0; i < 4; i++) {
 		r = rects[i];
-		DrawBorderY(r.X, r.Y, r.X + r.Width, r.Y + r.Height, y, color,
+		DrawBorderY(r.x, r.y, r.x + r.width, r.y + r.height, y, color,
 			Borders_HorOffset(block), Borders_YOffset(block), &data);
 	}
 	Gfx_UnlockVb(edges_vb);
@@ -762,22 +788,22 @@ static void UpdateMapEdges(void) {
 *---------------------------------------------------------General---------------------------------------------------------*
 *#########################################################################################################################*/
 static void CloudsPngProcess(struct Stream* stream, const cc_string* name) {
-	Game_UpdateTexture(&clouds_tex, stream, name, NULL);
+	Game_UpdateTexture(&clouds_tex, stream, name, NULL, NULL);
 }
 static struct TextureEntry clouds_entry = { "clouds.png", CloudsPngProcess };
 
 static void SkyboxPngProcess(struct Stream* stream, const cc_string* name) {
-	Game_UpdateTexture(&skybox_tex, stream, name, NULL);
+	Game_UpdateTexture(&skybox_tex, stream, name, NULL, NULL);
 }
 static struct TextureEntry skybox_entry = { "skybox.png", SkyboxPngProcess };
 
 static void SnowPngProcess(struct Stream* stream, const cc_string* name) {
-	Game_UpdateTexture(&snow_tex, stream, name, NULL);
+	Game_UpdateTexture(&snow_tex, stream, name, NULL, NULL);
 }
 static struct TextureEntry snow_entry = { "snow.png", SnowPngProcess };
 
 static void RainPngProcess(struct Stream* stream, const cc_string* name) {
-	Game_UpdateTexture(&rain_tex, stream, name, NULL);
+	Game_UpdateTexture(&rain_tex, stream, name, NULL, NULL);
 }
 static struct TextureEntry rain_entry = { "rain.png", RainPngProcess };
 
@@ -808,13 +834,12 @@ static void UpdateAll(void) {
 	UpdateMapEdges();
 	UpdateClouds();
 	UpdateSky();
-	UpdateSkybox();
+	Gfx_DeleteVb(&skybox_vb);
 	EnvRenderer_UpdateFog();
 
 	Gfx_DeleteDynamicVb(&weather_vb);
+	/* TODO: Unnecessary to delete the weather VB? */
 	if (Gfx.LostContext) return;
-	/* TODO: Don't allocate unless used? */
-	Gfx_RecreateDynamicVb(&weather_vb, VERTEX_FORMAT_TEXTURED, WEATHER_VERTS_COUNT);
 	/* TODO: Don't need to do this on every new map */
 	UpdateBorderTextures();
 }
@@ -873,7 +898,7 @@ static void OnEnvVariableChanged(void* obj, int envVar) {
 		UpdateSky();
 		UpdateClouds();
 	} else if (envVar == ENV_VAR_SKYBOX_COLOR) {
-		UpdateSkybox();
+		Gfx_DeleteVb(&skybox_vb);
 	}
 }
 
@@ -891,10 +916,12 @@ static void OnInit(void) {
 	EnvRenderer_Legacy  = flags & ENV_LEGACY;
 	EnvRenderer_Minimal = flags & ENV_MINIMAL;
 
+#ifndef CC_BUILD_LOW_VRAM
 	TextureEntry_Register(&clouds_entry);
 	TextureEntry_Register(&skybox_entry);
 	TextureEntry_Register(&snow_entry);
 	TextureEntry_Register(&rain_entry);
+#endif
 
 	Event_Register_(&TextureEvents.PackChanged,  NULL, OnTexturePackChanged);
 	Event_Register_(&TextureEvents.AtlasChanged, NULL, OnTerrainAtlasChanged);
