@@ -15,12 +15,14 @@
 #include "Logger.h"
 #include "Entity.h"
 #include "Chat.h"
+#include "Commands.h"
 #include "Drawer2D.h"
 #include "Model.h"
 #include "Particle.h"
 #include "Http.h"
 #include "Inventory.h"
 #include "Input.h"
+#include "InputHandler.h"
 #include "Server.h"
 #include "TexturePack.h"
 #include "Screens.h"
@@ -28,7 +30,7 @@
 #include "AxisLinesRenderer.h"
 #include "EnvRenderer.h"
 #include "HeldBlockRenderer.h"
-#include "PickedPosRenderer.h"
+#include "SelOutlineRenderer.h"
 #include "Menus.h"
 #include "Audio.h"
 #include "Stream.h"
@@ -38,21 +40,26 @@
 #include "Animations.h"
 #include "SystemFonts.h"
 #include "Formats.h"
+#include "EntityRenderers.h"
 
 struct _GameData Game;
-cc_uint64 Game_FrameStart;
+static cc_uint64 frameStart;
 cc_bool Game_UseCPEBlocks;
 
 struct RayTracer Game_SelectedPos;
-int Game_ViewDistance = 512, Game_UserViewDistance = 512;
-int Game_MaxViewDistance = DEFAULT_MAX_VIEWDIST;
+int Game_ViewDistance     = DEFAULT_VIEWDIST;
+int Game_UserViewDistance = DEFAULT_VIEWDIST;
+int Game_MaxViewDistance  = DEFAULT_MAX_VIEWDIST;
 
 int     Game_FpsLimit, Game_Vertices;
 cc_bool Game_SimpleArmsAnim;
+static cc_bool gameRunning;
+static float gfx_minFrameMs;
 
 cc_bool Game_ClassicMode, Game_ClassicHacks;
 cc_bool Game_AllowCustomBlocks;
 cc_bool Game_AllowServerTextures;
+cc_bool Game_Anaglyph3D;
 
 cc_bool Game_ViewBobbing, Game_HideGui;
 cc_bool Game_BreakableLiquids, Game_ScreenshotRequested;
@@ -60,8 +67,11 @@ struct GameVersion Game_Version;
 
 static char usernameBuffer[STRING_SIZE];
 static char mppassBuffer[STRING_SIZE];
-cc_string Game_Username = String_FromArray(usernameBuffer);
-cc_string Game_Mppass   = String_FromArray(mppassBuffer);
+cc_string Game_Username  = String_FromArray(usernameBuffer);
+cc_string Game_Mppass    = String_FromArray(mppassBuffer);
+#ifdef CC_BUILD_SPLITSCREEN
+int Game_NumStates = 1;
+#endif
 
 const char* const FpsLimit_Names[FPS_LIMIT_COUNT] = {
 	"LimitVSync", "Limit30FPS", "Limit60FPS", "Limit120FPS", "Limit144FPS", "LimitNone",
@@ -131,16 +141,16 @@ static void CycleViewDistanceBackwards(const short* viewDists, int count) {
 	Game_UserSetViewDistance(viewDists[count - 1]);
 }
 
-static const short normDists[10]   = { 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096 };
-static const short classicDists[4] = { 8, 32, 128, 512 };
+static const short normalDists[]  = { 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096 };
+static const short classicDists[] = { 8, 32, 128, 512 };
 void Game_CycleViewDistance(void) {
-	const short* dists = Gui.ClassicMenu ? classicDists : normDists;
-	int count = Gui.ClassicMenu ? Array_Elems(classicDists) : Array_Elems(normDists);
+	const short* dists = Gui.ClassicMenu ? classicDists : normalDists;
+	int count = Gui.ClassicMenu ? Array_Elems(classicDists) : Array_Elems(normalDists);
 
 	if (Input_IsShiftPressed()) {
 		CycleViewDistanceBackwards(dists, count);
 	} else {
-		CycleViewDistanceForwards(dists, count);
+		CycleViewDistanceForwards(dists,  count);
 	}
 }
 
@@ -209,13 +219,18 @@ cc_bool Game_CanPick(BlockID block) {
 	return Blocks.Collide[block] != COLLIDE_LIQUID || Game_BreakableLiquids;
 }
 
-cc_bool Game_UpdateTexture(GfxResourceID* texId, struct Stream* src, const cc_string* file, cc_uint8* skinType) {
+cc_bool Game_UpdateTexture(GfxResourceID* texId, struct Stream* src, const cc_string* file,
+							cc_uint8* skinType, int* heightDivisor) {
 	struct Bitmap bmp;
 	cc_bool success;
 	cc_result res;
 	
 	res = Png_Decode(&bmp, src);
 	if (res) { Logger_SysWarn2(res, "decoding", file); }
+	
+	/* E.g. gui.png only need top half of the texture loaded */
+	if (heightDivisor && bmp.height >= *heightDivisor)
+		bmp.height /= *heightDivisor;
 
 	success = !res && Game_ValidateBitmap(file, &bmp);
 	if (success) {
@@ -229,6 +244,8 @@ cc_bool Game_UpdateTexture(GfxResourceID* texId, struct Stream* src, const cc_st
 
 cc_bool Game_ValidateBitmap(const cc_string* file, struct Bitmap* bmp) {
 	int maxWidth = Gfx.MaxTexWidth, maxHeight = Gfx.MaxTexHeight;
+	float texSize, maxSize;
+
 	if (!bmp->scan0) {
 		Chat_Add1("&cError loading %s from the texture pack.", file);
 		return false;
@@ -237,15 +254,29 @@ cc_bool Game_ValidateBitmap(const cc_string* file, struct Bitmap* bmp) {
 	if (bmp->width > maxWidth || bmp->height > maxHeight) {
 		Chat_Add1("&cUnable to use %s from the texture pack.", file);
 
-		Chat_Add4("&c Its size is (%i,%i), your GPU supports (%i,%i) at most.", 
-			&bmp->width, &bmp->height, &maxWidth, &maxHeight);
+		Chat_Add4("&c Its size is (%i,%i), your GPU supports (%i,%i) at most.",
+				&bmp->width, &bmp->height, &maxWidth, &maxHeight);
 		return false;
 	}
 
+	if (Gfx.MaxTexSize && (bmp->width * bmp->height > Gfx.MaxTexSize)) {
+		Chat_Add1("&cUnable to use %s from the texture pack.", file);
+		texSize = (bmp->width * bmp->height) / (1024.0f * 1024.0f);
+		maxSize = Gfx.MaxTexSize             / (1024.0f * 1024.0f);
+
+		Chat_Add2("&c Its size is %f3 MB, your GPU supports %f3 MB at most.",
+				&texSize, &maxSize);
+		return false;
+	}
+
+	return Game_ValidateBitmapPow2(file, bmp);
+}
+
+cc_bool Game_ValidateBitmapPow2(const cc_string* file, struct Bitmap* bmp) {
 	if (!Math_IsPowOf2(bmp->width) || !Math_IsPowOf2(bmp->height)) {
 		Chat_Add1("&cUnable to use %s from the texture pack.", file);
 
-		Chat_Add2("&c Its size is (%i,%i), which is not a power of two size.", 
+		Chat_Add2("&c Its size is (%i,%i), which is not a power of two size.",
 			&bmp->width, &bmp->height);
 		return false;
 	}
@@ -253,8 +284,8 @@ cc_bool Game_ValidateBitmap(const cc_string* file, struct Bitmap* bmp) {
 }
 
 void Game_UpdateDimensions(void) {
-	Game.Width  = max(WindowInfo.Width,  1);
-	Game.Height = max(WindowInfo.Height, 1);
+	Game.Width  = max(Window_Main.Width,  1);
+	Game.Height = max(Window_Main.Height, 1);
 }
 
 static void Game_OnResize(void* obj) {
@@ -278,13 +309,17 @@ static void HandleOnNewMapLoaded(void* obj) {
 }
 
 static void HandleInactiveChanged(void* obj) {
-	if (WindowInfo.Inactive) {
+	if (Window_Main.Inactive) {
 		Chat_AddOf(&Gfx_LowPerfMessage, MSG_TYPE_EXTRASTATUS_2);
-		Gfx_SetFpsLimit(false, 1000 / 1.0f);
+		Gfx_SetVSync(false);
+		Game_SetMinFrameTime(1000 / 1.0f);
+		Gfx.ReducedPerfMode = true;
 	} else {
 		Chat_AddOf(&String_Empty,       MSG_TYPE_EXTRASTATUS_2);
 		Game_SetFpsLimit(Game_FpsLimit);
-		Chat_AddRaw(LOWPERF_EXIT_MESSAGE);
+
+		Gfx.ReducedPerfMode         = false;
+		Gfx.ReducedPerfModeCooldown = 2;
 	}
 
 #ifdef CC_BUILD_WEB
@@ -302,16 +337,23 @@ static void Game_WarnFunc(const cc_string* msg) {
 }
 
 static void LoadOptions(void) {
-	Game_ClassicMode       = Options_GetBool(OPT_CLASSIC_MODE, false);
-	Game_ClassicHacks      = Options_GetBool(OPT_CLASSIC_HACKS, false);
-	Game_AllowCustomBlocks = Options_GetBool(OPT_CUSTOM_BLOCKS, true);
-	Game_SimpleArmsAnim    = Options_GetBool(OPT_SIMPLE_ARMS_ANIM, false);
-	Game_ViewBobbing       = Options_GetBool(OPT_VIEW_BOBBING, true);
+	Game_ClassicMode  = Options_GetBool(OPT_CLASSIC_MODE,  false);
+	Game_ClassicHacks = Options_GetBool(OPT_CLASSIC_HACKS, false);
+	Game_Anaglyph3D   = Options_GetBool(OPT_ANAGLYPH3D,    false);
+#if defined CC_BUILD_PS1 || defined CC_BUILD_SATURN
+	/* View bobbing requires per-frame matrix multiplications - costly on FPU less systems */
+	Game_ViewBobbing  = Options_GetBool(OPT_VIEW_BOBBING,  false);
+#else
+	Game_ViewBobbing  = Options_GetBool(OPT_VIEW_BOBBING,  true);
+#endif
+	
+	Game_AllowCustomBlocks   = !Game_ClassicMode && Options_GetBool(OPT_CUSTOM_BLOCKS,      true);
+	Game_SimpleArmsAnim      = !Game_ClassicMode && Options_GetBool(OPT_SIMPLE_ARMS_ANIM,   false);
+	Game_BreakableLiquids    = !Game_ClassicMode && Options_GetBool(OPT_MODIFIABLE_LIQUIDS, false);
+	Game_AllowServerTextures = !Game_ClassicMode && Options_GetBool(OPT_SERVER_TEXTURES,    true);
 
-	Game_ViewDistance     = Options_GetInt(OPT_VIEW_DISTANCE, 8, 4096, 512);
+	Game_ViewDistance     = Options_GetInt(OPT_VIEW_DISTANCE, 8, 4096, DEFAULT_VIEWDIST);
 	Game_UserViewDistance = Game_ViewDistance;
-	Game_BreakableLiquids = !Game_ClassicMode && Options_GetBool(OPT_MODIFIABLE_LIQUIDS, false);
-	Game_AllowServerTextures = Options_GetBool(OPT_SERVER_TEXTURES, true);
 	/* TODO: Do we need to support option to skip SSL */
 	/*cc_bool skipSsl = Options_GetBool("skip-ssl-check", false);
 	if (skipSsl) {
@@ -320,14 +362,13 @@ static void LoadOptions(void) {
 	}*/
 }
 
-#ifdef CC_BUILD_MINFILES
-static void LoadPlugins(void) { }
-#else
-static void LoadPlugin(const cc_string* path, void* obj) {
+#ifdef CC_BUILD_PLUGINS
+static void LoadPlugin(const cc_string* path, void* obj, int isDirectory) {
 	void* lib;
 	void* verSym;  /* EXPORT int Plugin_ApiVersion = GAME_API_VER; */
 	void* compSym; /* EXPORT struct IGameComponent Plugin_Component = { (whatever) } */
 	int ver;
+	if (isDirectory) return;
 
 	/* ignore accepted.txt, deskop.ini, .pdb files, etc */
 	if (!String_CaselessEnds(path, &DynamicLib_Ext)) return;
@@ -363,14 +404,17 @@ static void LoadPlugins(void) {
 	res = Directory_Enum(&dir, NULL, LoadPlugin);
 	if (res) Logger_SysWarn(res, "enumerating plugins directory");
 }
+#else
+static void LoadPlugins(void) { }
 #endif
 
-void Game_Free(void* obj);
+static void Game_PendingClose(void* obj) { gameRunning = false; }
 static void Game_Load(void) {
 	struct IGameComponent* comp;
 	Game_UpdateDimensions();
 	Game_SetFpsLimit(Options_GetEnum(OPT_FPS_LIMIT, 0, FpsLimit_Names, FPS_LIMIT_COUNT));
 	Gfx_Create();
+	
 	Logger_WarnFunc = Game_WarnFunc;
 	LoadOptions();
 	GameVersion_Load();
@@ -379,12 +423,13 @@ static void Game_Load(void) {
 	Event_Register_(&WorldEvents.NewMap,           NULL, HandleOnNewMap);
 	Event_Register_(&WorldEvents.MapLoaded,        NULL, HandleOnNewMapLoaded);
 	Event_Register_(&WindowEvents.Resized,         NULL, Game_OnResize);
-	Event_Register_(&WindowEvents.Closing,         NULL, Game_Free);
+	Event_Register_(&WindowEvents.Closing,         NULL, Game_PendingClose);
 	Event_Register_(&WindowEvents.InactiveChanged, NULL, HandleInactiveChanged);
 
 	Game_AddComponent(&World_Component);
 	Game_AddComponent(&Textures_Component);
 	Game_AddComponent(&Input_Component);
+	Game_AddComponent(&InputHandler_Component);
 	Game_AddComponent(&Camera_Component);
 	Game_AddComponent(&Gfx_Component);
 	Game_AddComponent(&Blocks_Component);
@@ -392,6 +437,7 @@ static void Game_Load(void) {
 	Game_AddComponent(&SystemFonts_Component);
 
 	Game_AddComponent(&Chat_Component);
+	Game_AddComponent(&Commands_Component);
 	Game_AddComponent(&Particles_Component);
 	Game_AddComponent(&TabList_Component);
 	Game_AddComponent(&Models_Component);
@@ -411,10 +457,11 @@ static void Game_Load(void) {
 	Game_AddComponent(&Selections_Component);
 	Game_AddComponent(&HeldBlockRenderer_Component);
 	/* Gfx_SetDepthWrite(true) */
-	Game_AddComponent(&PickedPosRenderer_Component);
+	Game_AddComponent(&SelOutlineRenderer_Component);
 	Game_AddComponent(&Audio_Component);
 	Game_AddComponent(&AxisLinesRenderer_Component);
 	Game_AddComponent(&Formats_Component);
+	Game_AddComponent(&EntityRenderers_Component);
 
 	LoadPlugins();
 	for (comp = comps_head; comp; comp = comp->next) {
@@ -442,24 +489,39 @@ void Game_SetFpsLimit(int method) {
 	case FPS_LIMIT_60:  minFrameTime = 1000/60.0f;  break;
 	case FPS_LIMIT_30:  minFrameTime = 1000/30.0f;  break;
 	}
-	Gfx_SetFpsLimit(method == FPS_LIMIT_VSYNC, minFrameTime);
+	Gfx_SetVSync(method == FPS_LIMIT_VSYNC);
+	Game_SetMinFrameTime(minFrameTime);
 }
 
-static void UpdateViewMatrix(void) {
-	Camera.Active->GetView(&Gfx.View);
-	FrustumCulling_CalcFrustumEquations(&Gfx.Projection, &Gfx.View);
-}
+#ifdef CC_BUILD_WEB
+extern void Window_SetMinFrameTime(float timeMS);
 
-static void Game_Render3D(double delta, float t) {
+void Game_SetMinFrameTime(float frameTimeMS) {
+	if (frameTimeMS) Window_SetMinFrameTime(frameTimeMS);
+}
+#else
+void Game_SetMinFrameTime(float frameTimeMS) {
+	gfx_minFrameMs = frameTimeMS;
+}
+#endif
+
+static void Render3DFrame(float delta, float t) {
+	struct Matrix mvp;
 	Vec3 pos;
-	if (EnvRenderer_ShouldRenderSkybox()) EnvRenderer_RenderSkybox();
 
+	Camera.Active->GetView(&Gfx.View);
+	/*Gfx_LoadMatrix(MATRIX_PROJ, &Gfx.Projection);
+	Gfx_LoadMatrix(MATRIX_VIEW, &Gfx.View);
+	FrustumCulling_CalcFrustumEquations(&Gfx.Projection, &Gfx.View);*/
+	Gfx_LoadMVP(&Gfx.View, &Gfx.Projection, &mvp);
+	FrustumCulling_CalcFrustumEquations(&mvp);
+
+	if (EnvRenderer_ShouldRenderSkybox()) EnvRenderer_RenderSkybox();
 	AxisLinesRenderer_Render();
 	Entities_RenderModels(delta, t);
-	Entities_RenderNames();
+	EntityNames_Render();
 
 	Particles_Render(t);
-	Camera.Active->GetPickedBlock(&Game_SelectedPos); /* TODO: only pick when necessary */
 	EnvRenderer_RenderSky();
 	EnvRenderer_RenderClouds();
 
@@ -467,14 +529,14 @@ static void Game_Render3D(double delta, float t) {
 	MapRenderer_RenderNormal(delta);
 	EnvRenderer_RenderMapSides();
 
-	Entities_DrawShadows();
-	if (Game_SelectedPos.Valid && !Game_HideGui) {
-		PickedPosRenderer_Render(&Game_SelectedPos, true);
+	EntityShadows_Render();
+	if (Game_SelectedPos.valid && !Game_HideGui) {
+		SelOutlineRenderer_Render(&Game_SelectedPos, true);
 	}
 
 	/* Render water over translucent blocks when under the water outside the map for proper alpha blending */
 	pos = Camera.CurrentPos;
-	if (pos.Y < Env.EdgeHeight && (pos.X < 0 || pos.Z < 0 || pos.X > World.Width || pos.Z > World.Length)) {
+	if (pos.y < Env.EdgeHeight && (pos.x < 0 || pos.z < 0 || pos.x > World.Width || pos.z > World.Length)) {
 		MapRenderer_RenderTranslucent(delta);
 		EnvRenderer_RenderMapEdges();
 	} else {
@@ -484,17 +546,28 @@ static void Game_Render3D(double delta, float t) {
 
 	/* Need to render again over top of translucent block, as the selection outline */
 	/* is drawn without writing to the depth buffer */
-	if (Game_SelectedPos.Valid && !Game_HideGui && Blocks.Draw[Game_SelectedPos.block] == DRAW_TRANSLUCENT) {
-		PickedPosRenderer_Render(&Game_SelectedPos, false);
+	if (Game_SelectedPos.valid && !Game_HideGui && Blocks.Draw[Game_SelectedPos.block] == DRAW_TRANSLUCENT) {
+		SelOutlineRenderer_Render(&Game_SelectedPos, false);
 	}
 
 	Selections_Render();
-	Entities_RenderHoveredNames();
-	Camera_KeyLookUpdate();
-	InputHandler_Tick();
+	EntityNames_RenderHovered();
 	if (!Game_HideGui) HeldBlockRenderer_Render(delta);
 }
 
+static void Render3D_Anaglyph(float delta, float t) {
+	struct Matrix proj = Gfx.Projection;
+	struct Matrix view = Gfx.View;
+
+	Gfx_Set3DLeft(&proj, &view);
+	Render3DFrame(delta, t);
+
+	Gfx_Set3DRight(&proj, &view);
+	Render3DFrame(delta, t);
+
+	Gfx_End3D(&proj, &view);
+}
+
 static void PerformScheduledTasks(double time) {
 	struct ScheduledTask* task;
 	int i;
@@ -516,7 +589,7 @@ void Game_TakeScreenshot(void) {
 	struct DateTime now;
 	cc_result res;
 #ifdef CC_BUILD_WEB
-	char str[NATIVE_STR_LEN];
+	cc_filepath str;
 #else
 	struct Stream stream;
 #endif
@@ -529,8 +602,8 @@ void Game_TakeScreenshot(void) {
 
 #ifdef CC_BUILD_WEB
 	extern void interop_TakeScreenshot(const char* path);
-	String_EncodeUtf8(str, &filename);
-	interop_TakeScreenshot(str);
+	Platform_EncodePath(&str, &filename);
+	interop_TakeScreenshot(&str);
 #else
 	if (!Utils_EnsureDirectory("screenshots")) return;
 	String_InitArray(path, pathBuffer);
@@ -540,7 +613,7 @@ void Game_TakeScreenshot(void) {
 	if (res) { Logger_SysWarn2(res, "creating", &path); return; }
 
 	res = Gfx_TakeScreenshot(&stream);
-	if (res) { 
+	if (res) {
 		Logger_SysWarn2(res, "saving to", &path); stream.Close(&stream); return;
 	}
 
@@ -554,9 +627,121 @@ void Game_TakeScreenshot(void) {
 #endif
 }
 
-static void Game_RenderFrame(double delta) {
+
+#ifdef CC_BUILD_WEB
+static void LimitFPS(void) {
+	/* Can't use Thread_Sleep on the web. (spinwaits instead of sleeping) */
+	/* Instead the web browser manages the frame timing */
+}
+#else
+static float gfx_targetTime, gfx_actualTime;
+
+static CC_INLINE float ElapsedMilliseconds(cc_uint64 beg, cc_uint64 end) {
+	cc_uint64 elapsed = Stopwatch_ElapsedMicroseconds(beg, end);
+	if (elapsed > 5000000) elapsed = 5000000;
+	
+	return (int)elapsed / 1000.0f;
+}
+
+/* Examines difference between expected and actual frame times, */
+/*  then sleeps if actual frame time is too fast */
+static void LimitFPS(void) {
+	cc_uint64 frameEnd, sleepEnd;
+	
+	frameEnd = Stopwatch_Measure();
+	gfx_actualTime += ElapsedMilliseconds(frameStart, frameEnd);
+	gfx_targetTime += gfx_minFrameMs;
+
+	/* going faster than FPS limit - sleep to slow down */
+	if (gfx_actualTime < gfx_targetTime) {
+		float cooldown = gfx_targetTime - gfx_actualTime;
+		Thread_Sleep((int)(cooldown + 0.5f));
+
+		/* also accumulate Thread_Sleep duration, as actual sleep */
+		/*  duration can significantly deviate from requested time */
+		/*  (e.g. requested 4ms, but actually slept for 8ms) */
+		sleepEnd = Stopwatch_Measure();
+		gfx_actualTime += ElapsedMilliseconds(frameEnd, sleepEnd);
+	}
+
+	/* reset accumulated time to avoid excessive FPS drift */
+	if (gfx_targetTime >= 1000) { gfx_actualTime = 0; gfx_targetTime = 0; }
+}
+#endif
+
+static CC_INLINE void Game_DrawFrame(float delta, float t) {
+	int i;
+
+	if (!Gui_GetBlocksWorld()) {
+		Camera.Active->GetPickedBlock(&Game_SelectedPos); /* TODO: only pick when necessary */
+		Camera_KeyLookUpdate(delta);
+		InputHandler_Tick();
+
+		if (Game_Anaglyph3D) {
+			Render3D_Anaglyph(delta, t);
+		} else {
+			Render3DFrame(delta, t);
+		}
+	} else {
+		RayTracer_SetInvalid(&Game_SelectedPos);
+	}
+
+	Gfx_Begin2D(Game.Width, Game.Height);
+	Gui_RenderGui(delta);
+	for (i = 0; i < Array_Elems(Game.Draw2DHooks); i++)
+	{
+		if (Game.Draw2DHooks[i]) Game.Draw2DHooks[i](delta);
+	}
+
+/* TODO find a better solution than this */
+#ifdef CC_BUILD_3DS
+	if (Game_Anaglyph3D) {
+		extern void Gfx_SetTopRight(void);
+		Gfx_SetTopRight();
+		Gui_RenderGui(delta);
+	}
+#endif
+	Gfx_End2D();
+}
+
+#ifdef CC_BUILD_SPLITSCREEN
+static void DrawSplitscreen(float delta, float t, int i, int x, int y, int w, int h) {
+	Gfx_SetViewport(x, y, w, h);
+	Gfx_SetScissor( x, y, w, h);
+	Game.CurrentState = i;
+	
+	Entities.CurPlayer = &LocalPlayer_Instances[i];
+	LocalPlayer_SetInterpPosition(Entities.CurPlayer, t);
+	Camera.CurrentPos = Camera.Active->GetPosition(t);
+	
+	Game_DrawFrame(delta, t);
+}
+
+int Game_MapState(int deviceIndex) {
+	if (Game_NumStates >= 4 && deviceIndex == 3) return 3;
+	if (Game_NumStates >= 3 && deviceIndex == 2) return 2;
+	if (Game_NumStates >= 2 && deviceIndex == 1) return 1;
+
+	return 0;
+}
+#endif
+
+static CC_INLINE void Game_RenderFrame(void) {
 	struct ScheduledTask entTask;
-	float t;
+	double deltaD;
+	float t, delta;
+
+	cc_uint64 render  = Stopwatch_Measure();
+	cc_uint64 elapsed = Stopwatch_ElapsedMicroseconds(frameStart, render);
+	/* avoid large delta with suspended process */
+	if (elapsed > 5000000) elapsed = 5000000;
+	
+	deltaD = (int)elapsed / (1000.0 * 1000.0);
+	delta  = (float)deltaD;
+	Window_ProcessEvents(delta);
+
+	if (delta <= 0.0f) return;
+	frameStart = render;
 
 	/* TODO: Should other tasks get called back too? */
 	/* Might not be such a good idea for the http_clearcache, */
@@ -566,6 +751,7 @@ static void Game_RenderFrame(double delta) {
 			Gfx_RecreateContext();
 			/* all good, context is back */
 		} else {
+			Game.Time += delta; /* TODO: Not set in two places? */
 			Server.Tick(NULL);
 			Thread_Sleep(16);
 			return;
@@ -573,49 +759,76 @@ static void Game_RenderFrame(double delta) {
 	}
 
 	Gfx_BeginFrame();
-	Gfx_BindIb(Gfx_defaultIb);
-	Game.Time += delta;
+	Gfx_BindIb(Gfx.DefaultIb);
+	Game.Time += deltaD;
 	Game_Vertices = 0;
+	Gamepad_Tick(delta);
 
-	Camera.Active->UpdateMouse(delta);
-	if (!WindowInfo.Focused && !Gui.InputGrab) Gui_ShowPauseMenu();
+#ifdef CC_BUILD_SPLITSCREEN
+	/* TODO: find a better solution */
+	for (int i = 0; i < Game_NumStates; i++)
+	{
+		Game.CurrentState  = i;
+		Entities.CurPlayer = &LocalPlayer_Instances[i];
+		Camera.Active->UpdateMouse(Entities.CurPlayer, delta);
+	}
+	Game.CurrentState  = 0;
+	Entities.CurPlayer = &LocalPlayer_Instances[0];
+#else
+	Camera.Active->UpdateMouse(Entities.CurPlayer, delta);
+#endif
 
-	if (KeyBind_IsPressed(KEYBIND_ZOOM_SCROLL) && !Gui.InputGrab) {
+	if (!Window_Main.Focused && !Gui.InputGrab) Gui_ShowPauseMenu();
+
+	if (Bind_IsTriggered[BIND_ZOOM_SCROLL] && !Gui.InputGrab) {
 		InputHandler_SetFOV(Camera.ZoomFov);
 	}
 
-	PerformScheduledTasks(delta);
+	PerformScheduledTasks(deltaD);
 	entTask = tasks[entTaskI];
 	t = (float)(entTask.accumulator / entTask.interval);
-	LocalPlayer_SetInterpPosition(t);
+	LocalPlayer_SetInterpPosition(Entities.CurPlayer, t);
 
 	Camera.CurrentPos = Camera.Active->GetPosition(t);
 	/* NOTE: EnvRenderer_UpdateFog also also sets clear color */
 	EnvRenderer_UpdateFog();
-	UpdateViewMatrix();
+	AudioBackend_Tick();
 
 	/* TODO: Not calling Gfx_EndFrame doesn't work with Direct3D9 */
-	if (WindowInfo.Inactive) return;
-	Gfx_Clear();
-
-	Gfx_LoadMatrix(MATRIX_PROJECTION, &Gfx.Projection);
-	Gfx_LoadMatrix(MATRIX_VIEW,       &Gfx.View);
-
-	if (!Gui_GetBlocksWorld()) {
-		Game_Render3D(delta, t);
-	} else {
-		RayTracer_SetInvalid(&Game_SelectedPos);
+	if (Window_Main.Inactive) return;
+	Gfx_ClearBuffers(GFX_BUFFER_COLOR | GFX_BUFFER_DEPTH);
+	
+#ifdef CC_BUILD_SPLITSCREEN
+	switch (Game_NumStates) {
+		case 1:
+			Game_DrawFrame(delta, t); break;
+		case 2:
+			DrawSplitscreen(delta, t, 0,  0,               0, Game.Width, Game.Height / 2);
+			DrawSplitscreen(delta, t, 1,  0, Game.Height / 2, Game.Width, Game.Height / 2);
+			break;
+		case 3:
+			DrawSplitscreen(delta, t, 0,              0,               0, Game.Width    , Game.Height / 2);
+			DrawSplitscreen(delta, t, 1,              0, Game.Height / 2, Game.Width / 2, Game.Height / 2);
+			DrawSplitscreen(delta, t, 2, Game.Width / 2, Game.Height / 2, Game.Width / 2, Game.Height / 2);
+			break;
+		case 4:
+			DrawSplitscreen(delta, t, 0,              0,               0, Game.Width / 2, Game.Height / 2);
+			DrawSplitscreen(delta, t, 1, Game.Width / 2,               0, Game.Width / 2, Game.Height / 2);
+			DrawSplitscreen(delta, t, 2,              0, Game.Height / 2, Game.Width / 2, Game.Height / 2);
+			DrawSplitscreen(delta, t, 3, Game.Width / 2, Game.Height / 2, Game.Width / 2, Game.Height / 2);
+			break;
 	}
-
-	Gfx_Begin2D(Game.Width, Game.Height);
-	Gui_RenderGui(delta);
-	Gfx_End2D();
+#else
+	Game_DrawFrame(delta, t);
+#endif
 
 	if (Game_ScreenshotRequested) Game_TakeScreenshot();
 	Gfx_EndFrame();
+	if (gfx_minFrameMs) LimitFPS();
 }
 
-void Game_Free(void* obj) {
+
+static void Game_Free(void) {
 	struct IGameComponent* comp;
 	/* Most components will call OnContextLost in their Free functions */
 	/* Set to false so components will always free managed textures too */
@@ -623,40 +836,36 @@ void Game_Free(void* obj) {
 	Event_UnregisterAll();
 	tasksCount = 0;
 
-	for (comp = comps_head; comp; comp = comp->next) {
+	for (comp = comps_head; comp; comp = comp->next)
+	{
 		if (comp->Free) comp->Free();
 	}
 
+	gameRunning     = false;
 	Logger_WarnFunc = Logger_DialogWarn;
 	Gfx_Free();
 	Options_SaveIfChanged();
 	Window_DisableRawMouse();
 }
 
-#define Game_DoFrameBody() \
-	render = Stopwatch_Measure();\
-	delta  = Stopwatch_ElapsedMicroseconds(Game_FrameStart, render) / (1000.0 * 1000.0);\
-	\
-	Window_ProcessEvents(delta);\
-	if (!WindowInfo.Exists) return;\
-	\
-	if (delta > 1.0) delta = 1.0; /* avoid large delta with suspended process */ \
-	if (delta > 0.0) { Game_FrameStart = render; Game_RenderFrame(delta); }
-
 #ifdef CC_BUILD_WEB
 void Game_DoFrame(void) {
-	cc_uint64 render; 
-	double delta;
-	Game_DoFrameBody()
+	if (gameRunning) {
+		Game_RenderFrame();
+	} else if (tasksCount) {
+		Game_Free();
+		Window_Free();
+	}	
 }
 
 static void Game_RunLoop(void) {
-	Game_FrameStart = Stopwatch_Measure();
 	/* Window_Web.c sets Game_DoFrame as the main loop callback function */
 	/* (i.e. web browser is in charge of calling Game_DoFrame, not us) */
 }
 
 cc_bool Game_ShouldClose(void) {
+	if (!gameRunning) return true;
+
 	if (Server.IsSinglePlayer) {
 		/* Close if map was saved within last 5 seconds */
 		return World.LastSave + 5 >= Game.Time;
@@ -669,11 +878,11 @@ cc_bool Game_ShouldClose(void) {
 }
 #else
 static void Game_RunLoop(void) {
-	cc_uint64 render;
-	double delta;
-
-	Game_FrameStart = Stopwatch_Measure();
-	for (;;) { Game_DoFrameBody() }
+	while (gameRunning)
+	{
+		Game_RenderFrame();
+	}
+	Game_Free();
 }
 #endif
 
@@ -681,8 +890,13 @@ void Game_Run(int width, int height, const cc_string* title) {
 	Window_Create3D(width, height);
 	Window_SetTitle(title);
 	Window_Show();
+	gameRunning = true;
+	Game.CurrentState = 0;
 
 	Game_Load();
 	Event_RaiseVoid(&WindowEvents.Resized);
+
+	frameStart = Stopwatch_Measure();
 	Game_RunLoop();
+	Window_Destroy();
 }
