@@ -1,145 +1,25 @@
+/* Silence deprecation warnings on modern macOS/iOS */
+#define GL_SILENCE_DEPRECATION
+#define GLES_SILENCE_DEPRECATION
+
 #include "Core.h"
-#if defined CC_BUILD_GL && !defined CC_BUILD_GLMODERN
+#if CC_GFX_BACKEND == CC_GFX_BACKEND_GL1
 #include "_GraphicsBase.h"
 #include "Errors.h"
-#include "Logger.h"
 #include "Window.h"
+#ifdef CC_BUILD_WIN
+	#define CC_BUILD_GL11_FALLBACK
+#endif
+
 /* The OpenGL backend is a bit of a mess, since it's really 2 backends in one:
  * - OpenGL 1.1 (completely lacking GPU, fallbacks to say Windows built-in software rasteriser)
  * - OpenGL 1.5 or OpenGL 1.2 + GL_ARB_vertex_buffer_object (default desktop backend)
 */
+#include "../misc/opengl/GLCommon.h"
 
-#if defined CC_BUILD_WIN
-/* Avoid pointless includes */
-#define WIN32_LEAN_AND_MEAN
-#define NOSERVICE
-#define NOMCX
-#define NOIME
-#include <windows.h>
-#define GLAPI WINGDIAPI
-#else
-#define GLAPI extern
-#define APIENTRY
-#endif
-/* === BEGIN OPENGL HEADERS === */
-typedef unsigned int GLenum;
-typedef unsigned char GLboolean;
-typedef signed char GLbyte;
-typedef int GLint;
-typedef int GLsizei;
-typedef unsigned char GLubyte;
-typedef unsigned int GLuint;
-typedef float GLfloat;
-typedef void GLvoid;
-
-#define GL_LEQUAL                0x0203
-#define GL_GREATER               0x0204
-
-#define GL_DEPTH_BUFFER_BIT      0x00000100
-#define GL_COLOR_BUFFER_BIT      0x00004000
-
-#define GL_LINES                 0x0001
-#define GL_TRIANGLES             0x0004
-
-#define GL_BLEND                 0x0BE2
-#define GL_SRC_ALPHA             0x0302
-#define GL_ONE_MINUS_SRC_ALPHA   0x0303
-
-#define GL_UNSIGNED_BYTE         0x1401
-#define GL_UNSIGNED_SHORT        0x1403
-#define GL_UNSIGNED_INT          0x1405
-#define GL_FLOAT                 0x1406
-#define GL_RGBA                  0x1908
-
-#define GL_FOG                   0x0B60
-#define GL_FOG_DENSITY           0x0B62
-#define GL_FOG_END               0x0B64
-#define GL_FOG_MODE              0x0B65
-#define GL_FOG_COLOR             0x0B66
-#define GL_LINEAR                0x2601
-#define GL_EXP                   0x0800
-#define GL_EXP2                  0x0801
-
-#define GL_CULL_FACE             0x0B44
-#define GL_DEPTH_TEST            0x0B71
-#define GL_MATRIX_MODE           0x0BA0
-#define GL_VIEWPORT              0x0BA2
-#define GL_ALPHA_TEST            0x0BC0
-#define GL_MAX_TEXTURE_SIZE      0x0D33
-#define GL_DEPTH_BITS            0x0D56
-
-#define GL_FOG_HINT              0x0C54
-#define GL_NICEST                0x1102
-#define GL_COMPILE               0x1300
-
-#define GL_MODELVIEW             0x1700
-#define GL_PROJECTION            0x1701
-#define GL_TEXTURE               0x1702
-
-#define GL_VENDOR                0x1F00
-#define GL_RENDERER              0x1F01
-#define GL_VERSION               0x1F02
-#define GL_EXTENSIONS            0x1F03
-
-#define GL_TEXTURE_2D            0x0DE1
-#define GL_NEAREST               0x2600
-#define GL_NEAREST_MIPMAP_LINEAR 0x2702
-#define GL_TEXTURE_MAG_FILTER    0x2800
-#define GL_TEXTURE_MIN_FILTER    0x2801
-
-#define GL_VERTEX_ARRAY          0x8074
-#define GL_COLOR_ARRAY           0x8076
-#define GL_TEXTURE_COORD_ARRAY   0x8078
-
-/* Not present in gl.h on Windows (only up to OpenGL 1.1) */
-#define GL_ARRAY_BUFFER          0x8892
-#define GL_ELEMENT_ARRAY_BUFFER  0x8893
-#define GL_STATIC_DRAW           0x88E4
-#define GL_DYNAMIC_DRAW          0x88E8
-
-GLAPI void APIENTRY glAlphaFunc(GLenum func, GLfloat ref);
-GLAPI void APIENTRY glBindTexture(GLenum target, GLuint texture);
-GLAPI void APIENTRY glBlendFunc(GLenum sfactor, GLenum dfactor);
-GLAPI void APIENTRY glCallList(GLuint list);
-GLAPI void APIENTRY glClear(GLuint mask);
-GLAPI void APIENTRY glClearColor(GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha);
-GLAPI void APIENTRY glColorMask(GLboolean red, GLboolean green, GLboolean blue, GLboolean alpha);
-GLAPI void APIENTRY glColorPointer(GLint size, GLenum type, GLsizei stride, const GLvoid* pointer);
-GLAPI void APIENTRY glDeleteLists(GLuint list, GLsizei range);
-GLAPI void APIENTRY glDeleteTextures(GLsizei n, const GLuint* textures);
-GLAPI void APIENTRY glDepthFunc(GLenum func);
-GLAPI void APIENTRY glDepthMask(GLboolean flag);
-GLAPI void APIENTRY glDisable(GLenum cap);
-GLAPI void APIENTRY glDisableClientState(GLenum array);
-GLAPI void APIENTRY glDrawArrays(GLenum mode, GLint first, GLsizei count);
-GLAPI void APIENTRY glDrawElements(GLenum mode, GLsizei count, GLenum type, const GLvoid* indices);
-GLAPI void APIENTRY glEnable(GLenum cap);
-GLAPI void APIENTRY glEnableClientState(GLenum array);
-GLAPI void APIENTRY glEndList(void);
-GLAPI void APIENTRY glFogf(GLenum pname, GLfloat param);
-GLAPI void APIENTRY glFogfv(GLenum pname, const GLfloat* params);
-GLAPI void APIENTRY glFogi(GLenum pname, GLint param);
-GLAPI void APIENTRY glFogiv(GLenum pname, const GLint* params);
-GLAPI GLuint APIENTRY glGenLists(GLsizei range);
-GLAPI void APIENTRY   glGenTextures(GLsizei n, GLuint* textures);
-GLAPI GLenum APIENTRY glGetError(void);
-GLAPI void APIENTRY glGetFloatv(GLenum pname, GLfloat* params);
-GLAPI void APIENTRY glGetIntegerv(GLenum pname, GLint* params);
-GLAPI const GLubyte* APIENTRY glGetString(GLenum name);
-GLAPI void APIENTRY glHint(GLenum target, GLenum mode);
-GLAPI void APIENTRY glLoadIdentity(void);
-GLAPI void APIENTRY glLoadMatrixf(const GLfloat* m);
-GLAPI void APIENTRY glMatrixMode(GLenum mode);
-GLAPI void APIENTRY glNewList(GLuint list, GLenum mode);
-GLAPI void APIENTRY glReadPixels(GLint x, GLint y, GLsizei width, GLsizei height, GLenum format, GLenum type, GLvoid* pixels);
-GLAPI void APIENTRY glTexCoordPointer(GLint size, GLenum type, GLsizei stride, const GLvoid* pointer);
-GLAPI void APIENTRY glTexImage2D(GLenum target, GLint level, GLint internalformat, GLsizei width, GLsizei height, GLint border, GLenum format, GLenum type, const GLvoid* pixels);
-GLAPI void APIENTRY glTexParameteri(GLenum target, GLenum pname, GLint param);
-GLAPI void APIENTRY glTexSubImage2D(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid* pixels);
-GLAPI void APIENTRY glVertexPointer(GLint size, GLenum type, GLsizei stride, const GLvoid* pointer);
-GLAPI void APIENTRY glViewport(GLint x, GLint y, GLsizei width, GLsizei height);
-/* === END OPENGL HEADERS === */
-
+/* e.g. GLAPI void APIENTRY glFunction(int args); */
+#define GL_FUNC(_retType, name) GLAPI _retType APIENTRY name
+#include "../misc/opengl/GL1Funcs.h"
 
 #if defined CC_BUILD_GL11
 static GLuint activeList;
@@ -154,16 +34,17 @@ static void (APIENTRY *_glGenBuffers)(GLsizei n, GLuint *buffers);
 static void (APIENTRY *_glBufferData)(GLenum target, cc_uintptr size, const GLvoid* data, GLenum usage);
 static void (APIENTRY *_glBufferSubData)(GLenum target, cc_uintptr offset, cc_uintptr size, const GLvoid* data);
 #endif
-#include "_GLShared.h"
 
-typedef void (*GL_SetupVBFunc)(void);
-typedef void (*GL_SetupVBRangeFunc)(int startVertex);
-static GL_SetupVBFunc gfx_setupVBFunc;
-static GL_SetupVBRangeFunc gfx_setupVBRangeFunc;
-/* Current format and size of vertices */
-static int gfx_stride, gfx_format = -1;
+static void GLContext_GetAll(const struct DynamicLibSym* syms, int count) {
+	int i;
+	for (i = 0; i < count; i++) 
+	{
+		*syms[i].symAddr = GLContext_GetAddress(syms[i].name);
+	}
+}
 
-#if defined CC_BUILD_WIN && !defined CC_BUILD_GL11
+
+#if defined CC_BUILD_GL11_FALLBACK && !defined CC_BUILD_GL11
 /* Note the following about calling OpenGL functions on Windows */
 /*  1) wglGetProcAddress returns a context specific address */
 /*  2) dllimport functions are implemented using indirect function pointers */
@@ -174,50 +55,64 @@ static int gfx_stride, gfx_format = -1;
 /*    call [glDrawElements]  --> opengl32.dll thunk--> GL driver thunk --> GL driver implementation */
 /*    call [_glDrawElements] --> GL driver thunk --> GL driver implementation */
 
-static void (APIENTRY *_glColorPointer)(GLint size,    GLenum type, GLsizei stride, const GLvoid* pointer);
-static void (APIENTRY *_glDrawElements)(GLenum mode, GLsizei count,    GLenum type, const GLvoid* indices);
-static void (APIENTRY *_glTexCoordPointer)(GLint size, GLenum type, GLsizei stride, const GLvoid* pointer);
-static void (APIENTRY *_glVertexPointer)(GLint size,   GLenum type, GLsizei stride, const GLvoid* pointer);
+/* e.g. typedef void (APIENTRY *FP_glFunction)(int args); */
+#undef  GL_FUNC
+#define GL_FUNC(_retType, name) typedef _retType (APIENTRY *FP_ ## name)
+#include "../misc/opengl/GL1Funcs.h"
 
+/* e.g. static void (APIENTRY *_glFunction)(int args); */
+#undef  GL_FUNC
+#define GL_FUNC(_retType, name) static _retType (APIENTRY *_ ## name)
+#include "../misc/opengl/GL1Funcs.h"
+
+#define GLSym(sym) { DYNAMICLIB_QUOTE(sym), (void**)&_ ## sym }
 static const struct DynamicLibSym coreFuncs[] = {
-	DynamicLib_Sym2("glColorPointer",    glColorPointer),
-	DynamicLib_Sym2("glTexCoordPointer", glTexCoordPointer), DynamicLib_Sym2("glDrawElements", glDrawElements),
-	DynamicLib_Sym2("glVertexPointer",  glVertexPointer)
+	GLSym(glColorPointer), GLSym(glTexCoordPointer), GLSym(glVertexPointer),
+
+	GLSym(glDrawArrays),   GLSym(glDrawElements),
+
+	GLSym(glBindTexture),  GLSym(glDeleteTextures), GLSym(glGenTextures),
+	GLSym(glTexImage2D),   GLSym(glTexSubImage2D),
+	GLSym(glDisableClientState), GLSym(glEnableClientState)
 };
+
 static void LoadCoreFuncs(void) {
 	GLContext_GetAll(coreFuncs, Array_Elems(coreFuncs));
 }
 #else
 #define _glColorPointer    glColorPointer
-#define _glDrawElements    glDrawElements
 #define _glTexCoordPointer glTexCoordPointer
 #define _glVertexPointer   glVertexPointer
+
+#define _glDrawArrays      glDrawArrays
+#define _glDrawElements    glDrawElements
+
+#define _glBindTexture    glBindTexture
+#define _glDeleteTextures glDeleteTextures
+#define _glGenTextures    glGenTextures
+#define _glTexImage2D     glTexImage2D
+#define _glTexSubImage2D  glTexSubImage2D
+
+#define _glDisableClientState glDisableClientState
+#define _glEnableClientState  glEnableClientState
 #endif
 
+typedef void (*GL_SetupVBFunc)(void);
+typedef void (*GL_SetupVBRangeFunc)(int startVertex);
+static GL_SetupVBFunc gfx_setupVBFunc;
+static GL_SetupVBRangeFunc gfx_setupVBRangeFunc;
+#include "_GLShared.h"
 
 /*########################################################################################################################*
 *-------------------------------------------------------Index buffers-----------------------------------------------------*
 *#########################################################################################################################*/
 #ifndef CC_BUILD_GL11
-static GfxResourceID GL_GenBuffer(void) {
-	GLuint id;
-	_glGenBuffers(1, &id);
-	return id;
-}
-
-static void GL_DelBuffer(GfxResourceID id) {
-	GLuint gl_id = (GLuint)id;
-	_glDeleteBuffers(1, &gl_id);
-}
-
-static GfxResourceID (*_genBuffer)(void)    = GL_GenBuffer;
-static void (*_delBuffer)(GfxResourceID id) = GL_DelBuffer;
-
 GfxResourceID Gfx_CreateIb2(int count, Gfx_FillIBFunc fillFunc, void* obj) {
 	cc_uint16 indices[GFX_MAX_INDICES];
-	GfxResourceID id = _genBuffer();
+	GfxResourceID id = NULL;
 	cc_uint32 size   = count * sizeof(cc_uint16);
 
+	_glGenBuffers(1, (GLuint*)&id);
 	fillFunc(indices, count, obj);
 	_glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, id);
 	_glBufferData(GL_ELEMENT_ARRAY_BUFFER, size, indices, GL_STATIC_DRAW);
@@ -229,7 +124,8 @@ void Gfx_BindIb(GfxResourceID ib) { _glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, ib);
 void Gfx_DeleteIb(GfxResourceID* ib) {
 	GfxResourceID id = *ib;
 	if (!id) return;
-	_delBuffer(id);
+
+	_glDeleteBuffers(1, (GLuint*)&id);
 	*ib = 0;
 }
 #else
@@ -243,18 +139,20 @@ void Gfx_DeleteIb(GfxResourceID* ib) { }
 *------------------------------------------------------Vertex buffers-----------------------------------------------------*
 *#########################################################################################################################*/
 #ifndef CC_BUILD_GL11
-GfxResourceID Gfx_CreateVb(VertexFormat fmt, int count) {
-	GfxResourceID id = _genBuffer();
+static GfxResourceID Gfx_AllocStaticVb(VertexFormat fmt, int count) {
+	GfxResourceID id = NULL;
+	_glGenBuffers(1, (GLuint*)&id);
 	_glBindBuffer(GL_ARRAY_BUFFER, id);
 	return id;
 }
 
-void Gfx_BindVb(GfxResourceID vb) { _glBindBuffer(GL_ARRAY_BUFFER, vb); }
+void Gfx_BindVb(GfxResourceID vb) { 
+	_glBindBuffer(GL_ARRAY_BUFFER, vb); 
+}
 
 void Gfx_DeleteVb(GfxResourceID* vb) {
 	GfxResourceID id = *vb;
-	if (!id) return;
-	_delBuffer(id);
+	if (id) _glDeleteBuffers(1, (GLuint*)&id);
 	*vb = 0;
 }
 
@@ -266,11 +164,13 @@ void Gfx_UnlockVb(GfxResourceID vb) {
 	_glBufferData(GL_ARRAY_BUFFER, tmpSize, tmpData, GL_STATIC_DRAW);
 }
 #else
-GfxResourceID Gfx_CreateVb(VertexFormat fmt, int count) { return glGenLists(1); }
-void Gfx_BindVb(GfxResourceID vb) { activeList = (GLuint)vb; }
+static GfxResourceID Gfx_AllocStaticVb(VertexFormat fmt, int count) { 
+	return glGenLists(1); 
+}
+void Gfx_BindVb(GfxResourceID vb) { activeList = ptr_to_uint(vb); }
 
 void Gfx_DeleteVb(GfxResourceID* vb) {
-	GLuint id = (GLuint)(*vb);
+	GLuint id = ptr_to_uint(*vb);
 	if (id) glDeleteLists(id, 1);
 	*vb = 0;
 }
@@ -317,19 +217,26 @@ GfxResourceID Gfx_CreateVb2(void* vertices, VertexFormat fmt, int count) {
 *--------------------------------------------------Dynamic vertex buffers-------------------------------------------------*
 *#########################################################################################################################*/
 #ifndef CC_BUILD_GL11
-GfxResourceID Gfx_CreateDynamicVb(VertexFormat fmt, int maxVertices) {
-	GfxResourceID id;
-	cc_uint32 size;
-	if (Gfx.LostContext) return 0;
-
-	id   = _genBuffer();
-	size = maxVertices * strideSizes[fmt];
+static GfxResourceID Gfx_AllocDynamicVb(VertexFormat fmt, int maxVertices) {
+	GfxResourceID id = NULL;
+	cc_uint32 size   = maxVertices * strideSizes[fmt];
 
+	_glGenBuffers(1, (GLuint*)&id);
 	_glBindBuffer(GL_ARRAY_BUFFER, id);
 	_glBufferData(GL_ARRAY_BUFFER, size, NULL, GL_DYNAMIC_DRAW);
 	return id;
 }
 
+void Gfx_BindDynamicVb(GfxResourceID vb) {
+	_glBindBuffer(GL_ARRAY_BUFFER, vb); 
+}
+
+void Gfx_DeleteDynamicVb(GfxResourceID* vb) {
+	GfxResourceID id = *vb;
+	if (id) _glDeleteBuffers(1, (GLuint*)&id);
+	*vb = 0;
+}
+
 void* Gfx_LockDynamicVb(GfxResourceID vb, VertexFormat fmt, int count) {
 	return FastAllocTempMem(count * strideSizes[fmt]);
 }
@@ -345,36 +252,127 @@ void Gfx_SetDynamicVbData(GfxResourceID vb, void* vertices, int vCount) {
 	_glBufferSubData(GL_ARRAY_BUFFER, 0, size, vertices);
 }
 #else
-GfxResourceID Gfx_CreateDynamicVb(VertexFormat fmt, int maxVertices) { 
-	return (GfxResourceID)Mem_Alloc(maxVertices, strideSizes[fmt], "creating dynamic vb");
+static GfxResourceID Gfx_AllocDynamicVb(VertexFormat fmt, int maxVertices) {
+	return (GfxResourceID)Mem_TryAlloc(maxVertices, strideSizes[fmt]);
 }
 
 void Gfx_BindDynamicVb(GfxResourceID vb) {
 	activeList      = gl_DYNAMICLISTID;
-	dynamicListData = (void*)vb;
+	dynamicListData = vb;
 }
 
 void Gfx_DeleteDynamicVb(GfxResourceID* vb) {
-	void* addr = (void*)(*vb);
+	void* addr = *vb;
 	if (addr) Mem_Free(addr);
 	*vb = 0;
 }
 
-void* Gfx_LockDynamicVb(GfxResourceID vb, VertexFormat fmt, int count) { return (void*)vb; }
+void* Gfx_LockDynamicVb(GfxResourceID vb, VertexFormat fmt, int count) { return vb; }
 void  Gfx_UnlockDynamicVb(GfxResourceID vb) { Gfx_BindDynamicVb(vb); }
 
 void Gfx_SetDynamicVbData(GfxResourceID vb, void* vertices, int vCount) {
 	Gfx_BindDynamicVb(vb);
-	Mem_Copy((void*)vb, vertices, vCount * gfx_stride);
+	Mem_Copy(vb, vertices, vCount * gfx_stride);
 }
 #endif
 
 
+/*########################################################################################################################*
+*----------------------------------------------------------Drawing--------------------------------------------------------*
+*#########################################################################################################################*/
+#ifdef CC_BUILD_GL11
+	/* point to client side dynamic array */
+	#define VB_PTR ((cc_uint8*)dynamicListData)
+	#define IB_PTR gl_indices
+#else
+	/* no client side array, use vertex buffer object */
+	#define VB_PTR 0
+	#define IB_PTR NULL
+#endif
+
+static void GL_SetupVbColoured(void) {
+	_glVertexPointer(3, GL_FLOAT,        SIZEOF_VERTEX_COLOURED, VB_PTR +  0);
+	_glColorPointer(4, GL_UNSIGNED_BYTE, SIZEOF_VERTEX_COLOURED, VB_PTR + 12);
+}
+
+static void GL_SetupVbTextured(void) {
+	_glVertexPointer(3, GL_FLOAT,        SIZEOF_VERTEX_TEXTURED, VB_PTR +  0);
+	_glColorPointer(4, GL_UNSIGNED_BYTE, SIZEOF_VERTEX_TEXTURED, VB_PTR + 12);
+	_glTexCoordPointer(2, GL_FLOAT,      SIZEOF_VERTEX_TEXTURED, VB_PTR + 16);
+}
+
+static void GL_SetupVbColoured_Range(int startVertex) {
+	cc_uint32 offset = startVertex * SIZEOF_VERTEX_COLOURED;
+	_glVertexPointer(3, GL_FLOAT,          SIZEOF_VERTEX_COLOURED, VB_PTR + offset +  0);
+	_glColorPointer(4, GL_UNSIGNED_BYTE,   SIZEOF_VERTEX_COLOURED, VB_PTR + offset + 12);
+}
+
+static void GL_SetupVbTextured_Range(int startVertex) {
+	cc_uint32 offset = startVertex * SIZEOF_VERTEX_TEXTURED;
+	_glVertexPointer(3,  GL_FLOAT,         SIZEOF_VERTEX_TEXTURED, VB_PTR + offset +  0);
+	_glColorPointer(4, GL_UNSIGNED_BYTE,   SIZEOF_VERTEX_TEXTURED, VB_PTR + offset + 12);
+	_glTexCoordPointer(2, GL_FLOAT,        SIZEOF_VERTEX_TEXTURED, VB_PTR + offset + 16);
+}
+
+void Gfx_SetVertexFormat(VertexFormat fmt) {
+	if (fmt == gfx_format) return;
+	gfx_format = fmt;
+	gfx_stride = strideSizes[fmt];
+
+	if (fmt == VERTEX_FORMAT_TEXTURED) {
+		_glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+		glEnable(GL_TEXTURE_2D);
+
+		gfx_setupVBFunc      = GL_SetupVbTextured;
+		gfx_setupVBRangeFunc = GL_SetupVbTextured_Range;
+	} else {
+		_glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+		glDisable(GL_TEXTURE_2D);
+
+		gfx_setupVBFunc      = GL_SetupVbColoured;
+		gfx_setupVBRangeFunc = GL_SetupVbColoured_Range;
+	}
+}
+
+void Gfx_DrawVb_Lines(int verticesCount) {
+	gfx_setupVBFunc();
+	_glDrawArrays(GL_LINES, 0, verticesCount);
+}
+
+void Gfx_DrawVb_IndexedTris_Range(int verticesCount, int startVertex) {
+#ifdef CC_BUILD_GL11
+	if (activeList != gl_DYNAMICLISTID) { glCallList(activeList); return; }
+#endif
+	gfx_setupVBRangeFunc(startVertex);
+	_glDrawElements(GL_TRIANGLES, ICOUNT(verticesCount), GL_UNSIGNED_SHORT, IB_PTR);
+}
+
+void Gfx_DrawVb_IndexedTris(int verticesCount) {
+#ifdef CC_BUILD_GL11
+	if (activeList != gl_DYNAMICLISTID) { glCallList(activeList); return; }
+#endif
+	gfx_setupVBFunc();
+	_glDrawElements(GL_TRIANGLES, ICOUNT(verticesCount), GL_UNSIGNED_SHORT, IB_PTR);
+}
+
+#ifdef CC_BUILD_GL11
+void Gfx_DrawIndexedTris_T2fC4b(int verticesCount, int startVertex) { glCallList(activeList); }
+#else
+void Gfx_DrawIndexedTris_T2fC4b(int verticesCount, int startVertex) {
+	cc_uint32 offset = startVertex * SIZEOF_VERTEX_TEXTURED;
+	_glVertexPointer(3, GL_FLOAT,        SIZEOF_VERTEX_TEXTURED, VB_PTR + offset +  0);
+	_glColorPointer(4, GL_UNSIGNED_BYTE, SIZEOF_VERTEX_TEXTURED, VB_PTR + offset + 12);
+	_glTexCoordPointer(2, GL_FLOAT,      SIZEOF_VERTEX_TEXTURED, VB_PTR + offset + 16);
+	_glDrawElements(GL_TRIANGLES,        ICOUNT(verticesCount),  GL_UNSIGNED_SHORT, IB_PTR);
+}
+#endif /* !CC_BUILD_GL11 */
+
+
 /*########################################################################################################################*
 *---------------------------------------------------------Textures--------------------------------------------------------*
 *#########################################################################################################################*/
 void Gfx_BindTexture(GfxResourceID texId) {
-	glBindTexture(GL_TEXTURE_2D, (GLuint)texId);
+	_glBindTexture(GL_TEXTURE_2D, ptr_to_uint(texId));
 }
 
 
@@ -429,15 +427,14 @@ void Gfx_SetFogMode(FogFunc func) {
 	gfx_fogMode = func;
 }
 
-void Gfx_SetTexturing(cc_bool enabled) { }
-
-void Gfx_SetAlphaTest(cc_bool enabled) { 
+static void SetAlphaTest(cc_bool enabled) {
 	if (enabled) { glEnable(GL_ALPHA_TEST); } else { glDisable(GL_ALPHA_TEST); }
 }
 
 void Gfx_DepthOnlyRendering(cc_bool depthOnly) {
 	cc_bool enabled = !depthOnly;
-	Gfx_SetColWriteMask(enabled, enabled, enabled, enabled);
+	SetColorWrite(enabled & gfx_colorMask[0], enabled & gfx_colorMask[1], 
+				  enabled & gfx_colorMask[2], enabled & gfx_colorMask[3]);
 	if (enabled) { glEnable(GL_TEXTURE_2D); } else { glDisable(GL_TEXTURE_2D); }
 }
 
@@ -450,21 +447,27 @@ static int lastMatrix;
 
 void Gfx_LoadMatrix(MatrixType type, const struct Matrix* matrix) {
 	if (type != lastMatrix) { lastMatrix = type; glMatrixMode(matrix_modes[type]); }
-	glLoadMatrixf((const float*)matrix);
+
+	if (matrix == &Matrix_Identity) {
+		glLoadIdentity();
+	} else {
+		glLoadMatrixf((const float*)matrix);
+	}
 }
 
-void Gfx_LoadIdentityMatrix(MatrixType type) {
-	if (type != lastMatrix) { lastMatrix = type; glMatrixMode(matrix_modes[type]); }
-	glLoadIdentity();
+void Gfx_LoadMVP(const struct Matrix* view, const struct Matrix* proj, struct Matrix* mvp) {
+	Gfx_LoadMatrix(MATRIX_VIEW, view);
+	Gfx_LoadMatrix(MATRIX_PROJ, proj);
+	Matrix_Mul(mvp, view, proj);
 }
 
 static struct Matrix texMatrix = Matrix_IdentityValue;
 void Gfx_EnableTextureOffset(float x, float y) {
-	texMatrix.row4.X = x; texMatrix.row4.Y = y;
+	texMatrix.row4.x = x; texMatrix.row4.y = y;
 	Gfx_LoadMatrix(2, &texMatrix);
 }
 
-void Gfx_DisableTextureOffset(void) { Gfx_LoadIdentityMatrix(2); }
+void Gfx_DisableTextureOffset(void) { Gfx_LoadMatrix(2, &Matrix_Identity); }
 
 
 /*########################################################################################################################*
@@ -473,8 +476,8 @@ void Gfx_DisableTextureOffset(void) { Gfx_LoadIdentityMatrix(2); }
 static void Gfx_FreeState(void) { FreeDefaultResources(); }
 static void Gfx_RestoreState(void) {
 	InitDefaultResources();
-	glEnableClientState(GL_VERTEX_ARRAY);
-	glEnableClientState(GL_COLOR_ARRAY);
+	_glEnableClientState(GL_VERTEX_ARRAY);
+	_glEnableClientState(GL_COLOR_ARRAY);
 	gfx_format = -1;
 
 	glHint(GL_FOG_HINT, GL_NICEST);
@@ -485,21 +488,30 @@ static void Gfx_RestoreState(void) {
 
 cc_bool Gfx_WarnIfNecessary(void) {
 	cc_string renderer = String_FromReadonly((const char*)glGetString(GL_RENDERER));
-
+	
 #ifdef CC_BUILD_GL11
 	Chat_AddRaw("&cYou are using the very outdated OpenGL backend.");
 	Chat_AddRaw("&cAs such you may experience poor performance.");
 	Chat_AddRaw("&cIt is likely you need to install video card drivers.");
 #endif
-	if (!String_ContainsConst(&renderer, "Intel")) return false;
 
-	Chat_AddRaw("&cIntel graphics cards are known to have issues with the OpenGL build.");
-	Chat_AddRaw("&cVSync may not work, and you may see disappearing clouds and map edges.");
-#ifdef CC_BUILD_WIN
-	Chat_AddRaw("&cTry downloading the Direct3D 9 build instead.");
-#endif
-	return true;
+	if (String_ContainsConst(&renderer, "llvmpipe")) {
+		Chat_AddRaw("&cSoftware rendering is being used, performance will greatly suffer.");
+		Chat_AddRaw("&cVSync may not work, and you may see disappearing clouds and map edges.");
+		Chat_AddRaw("&cYou may need to install video card drivers.");
+		return true;
+	}
+	if (String_ContainsConst(&renderer, "Intel")) {
+		Chat_AddRaw("&cIntel graphics cards are known to have issues with the OpenGL build.");
+		Chat_AddRaw("&cVSync may not work, and you may see disappearing clouds and map edges.");
+		#ifdef CC_BUILD_WIN
+		Chat_AddRaw("&cTry downloading the Direct3D 9 build instead.");
+		#endif
+		return true;
+	}
+	return false;
 }
+cc_bool Gfx_GetUIOptions(struct MenuOptionsScreen* s) { return false; }
 
 
 /*########################################################################################################################*
@@ -509,72 +521,184 @@ cc_bool Gfx_WarnIfNecessary(void) {
 static void GLBackend_Init(void) { MakeIndices(gl_indices, GFX_MAX_INDICES, NULL); }
 #else
 
-#if defined CC_BUILD_WIN
-/* On 32 bit windows, can replace the gl function drawing with these 1.1 fallbacks  */
-/*  (note that this only works on 32 bit system, as OpenGL IDs are 32 bit integers) */
+#ifdef CC_BUILD_GL11_FALLBACK
+static FP_glDrawElements    _realDrawElements;
+static FP_glColorPointer    _realColorPointer;
+static FP_glTexCoordPointer _realTexCoordPointer;
+static FP_glVertexPointer   _realVertexPointer;
 
-/* fake vertex buffer objects with client side pointers */
-typedef struct fake_buffer { cc_uint8* data; } fake_buffer;
-static fake_buffer* cur_ib;
-static fake_buffer* cur_vb;
-#define fake_GetBuffer(target) (target == GL_ELEMENT_ARRAY_BUFFER ? &cur_ib : &cur_vb);
+/* On Windows, can replace the GL function drawing with these 1.1 fallbacks */
+/* fake vertex buffer objects by using client side pointers instead */
+typedef struct legacy_buffer { cc_uint8* data; } legacy_buffer;
+static legacy_buffer* cur_ib;
+static legacy_buffer* cur_vb;
+#define legacy_GetBuffer(target) (target == GL_ELEMENT_ARRAY_BUFFER ? &cur_ib : &cur_vb);
 
-static void APIENTRY fake_bindBuffer(GLenum target, GfxResourceID src) {
-	fake_buffer** buffer = fake_GetBuffer(target);
-	*buffer = (fake_buffer*)src;
+static void APIENTRY legacy_genBuffer(GLsizei n, GLuint* buffer) {
+	GfxResourceID* dst = (GfxResourceID*)buffer;
+	*dst = Mem_TryAllocCleared(1, sizeof(legacy_buffer));
 }
 
-static GfxResourceID GenFakeBuffer(void) {
-	return (GfxResourceID)Mem_TryAllocCleared(1, sizeof(fake_buffer));
+static void APIENTRY legacy_deleteBuffer(GLsizei n, const GLuint* buffer) {
+	GfxResourceID* dst = (GfxResourceID*)buffer;
+	Mem_Free(*dst);
 }
 
-static void DelFakeBuffer(GfxResourceID id) {
-	Mem_Free((void*)id);
+static void APIENTRY legacy_bindBuffer(GLenum target, GfxResourceID src) {
+	legacy_buffer** buffer = legacy_GetBuffer(target);
+	*buffer = (legacy_buffer*)src;
 }
 
-static void APIENTRY fake_bufferData(GLenum target, cc_uintptr size, const GLvoid* data, GLenum usage) {
-	fake_buffer* buffer = *fake_GetBuffer(target);
+static void APIENTRY legacy_bufferData(GLenum target, cc_uintptr size, const GLvoid* data, GLenum usage) {
+	legacy_buffer* buffer = *legacy_GetBuffer(target);
 	Mem_Free(buffer->data);
 
 	buffer->data = Mem_TryAlloc(size, 1);
 	if (data) Mem_Copy(buffer->data, data, size);
 }
-static void APIENTRY fake_bufferSubData(GLenum target, cc_uintptr offset, cc_uintptr size, const GLvoid* data) {
-	fake_buffer* buffer = *fake_GetBuffer(target);
+
+static void APIENTRY legacy_bufferSubData(GLenum target, cc_uintptr offset, cc_uintptr size, const GLvoid* data) {
+	legacy_buffer* buffer = *legacy_GetBuffer(target);
 	Mem_Copy(buffer->data, data, size);
 }
 
-/* wglGetProcAddress doesn't work with OpenGL 1.1 software rasteriser, so call GL functions directly */
-static void APIENTRY fake_drawElements(GLenum mode, GLsizei count, GLenum type, const GLvoid* indices) {
-	glDrawElements(mode, count, type, (cc_uintptr)indices + cur_ib->data);
-}
-static void APIENTRY fake_colorPointer(GLint size, GLenum type, GLsizei stride, const GLvoid* pointer) {
-	glColorPointer(size,    type, stride, (cc_uintptr)pointer + cur_vb->data);
-}
-static void APIENTRY fake_texCoordPointer(GLint size, GLenum type, GLsizei stride, const GLvoid* pointer) {
-	glTexCoordPointer(size, type, stride, (cc_uintptr)pointer + cur_vb->data);
-}
-static void APIENTRY fake_vertexPointer(GLint size, GLenum type, GLsizei stride, const GLvoid* pointer) {
-	glVertexPointer(size,   type, stride, (cc_uintptr)pointer + cur_vb->data);
+
+struct GL10Texture {
+	int width, height;
+	unsigned char* pixels;
+};
+static struct GL10Texture* gl10_tex;
+
+static void APIENTRY gl10_bindTexture(GLenum target, GLuint texture) {
+	gl10_tex = (struct GL10Texture*)texture;
+	if (gl10_tex && gl10_tex->pixels) {
+		glTexImage2D(GL_TEXTURE_2D, 0, 4, gl10_tex->width, gl10_tex->height, 0, GL_RGBA, GL_UNSIGNED_BYTE, gl10_tex->pixels);
+	} else {
+		BitmapCol pixel = BITMAPCOLOR_WHITE;
+		glTexImage2D(GL_TEXTURE_2D, 0, 4, 1, 1, 0, GL_RGBA, GL_UNSIGNED_BYTE, &pixel);
+	}
 }
 
-static void OpenGL11Fallback(void) {
+static void APIENTRY gl10_deleteTexture(GLsizei n, const GLuint* textures) {
+	struct GL10Texture* tex = (struct GL10Texture*)textures[0];
+	if (tex->pixels) Mem_Free(tex->pixels);
+	if (tex) Mem_Free(tex);
+}
+
+static void APIENTRY gl10_genTexture(GLsizei n, GLuint* textures) {
+	textures[0] = (GLuint)Mem_AllocCleared(1, sizeof(struct GL10Texture), "GL 1.0 texture");
+}
+
+static void APIENTRY gl10_texImage(GLenum target, GLint level, GLint internalformat, GLsizei width, GLsizei height, GLint border, GLenum format, GLenum type, const GLvoid* pixels) {
+	int i;
+	gl10_tex->width  = width;
+	gl10_tex->height = height;
+	gl10_tex->pixels = Mem_Alloc(width * height, 4, "GL 1.0 pixels");
+
+	Mem_Copy(gl10_tex->pixels, pixels, width * height * 4);
+	for (i = 0; i < width * height * 4; i += 4) 
+	{
+		cc_uint8 t = gl10_tex->pixels[i + 2];
+		gl10_tex->pixels[i + 2] = gl10_tex->pixels[i + 0];
+		gl10_tex->pixels[i + 0] = t;
+	}
+}
+
+static void APIENTRY gl10_texSubImage(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid* pixels) {
+	/* TODO */
+}
+
+static void APIENTRY gl10_disableClientState(GLenum target) { }
+
+static void APIENTRY gl10_enableClientState(GLenum target) { }
+
+static cc_uint8* gl10_vb;
+static void APIENTRY gl10_drawElements(GLenum mode, GLsizei count, GLenum type, const GLvoid* indices) {
+	/* TODO */
+	int i;
+	glBegin(GL_QUADS);
+	count = (count * 4) / 6;
+
+	if (gfx_format == VERTEX_FORMAT_TEXTURED) {
+		struct VertexTextured* src = (struct VertexTextured*)gl10_vb;
+		for (i = 0; i < count; i++, src++) 
+		{
+			glColor4ub(PackedCol_R(src->Col), PackedCol_G(src->Col), PackedCol_B(src->Col), PackedCol_A(src->Col));
+			glTexCoord2f(src->U, src->V);
+			glVertex3f(src->x, src->y, src->z);
+		}
+	} else {
+		struct VertexColoured* src = (struct VertexColoured*)gl10_vb;
+		for (i = 0; i < count; i++, src++) 
+		{
+			glColor4ub(PackedCol_R(src->Col), PackedCol_G(src->Col), PackedCol_B(src->Col), PackedCol_A(src->Col));
+			glVertex3f(src->x, src->y, src->z);
+		}
+	}
+
+	glEnd();
+}
+static void APIENTRY gl10_colorPointer(GLint size, GLenum type, GLsizei stride, GLpointer offset) {
+}
+static void APIENTRY gl10_texCoordPointer(GLint size, GLenum type, GLsizei stride, GLpointer offset) {
+}
+static void APIENTRY gl10_vertexPointer(GLint size, GLenum type, GLsizei stride, GLpointer offset) {
+	gl10_vb = cur_vb->data + offset;
+}
+
+
+static void APIENTRY gl11_drawElements(GLenum mode, GLsizei count, GLenum type, const GLvoid* indices) {
+	_realDrawElements(mode, count, type, (cc_uintptr)indices + cur_ib->data);
+}
+static void APIENTRY gl11_colorPointer(GLint size, GLenum type, GLsizei stride, GLpointer offset) {
+	_realColorPointer(size,    type, stride, (cc_uintptr)cur_vb->data + offset);
+}
+static void APIENTRY gl11_texCoordPointer(GLint size, GLenum type, GLsizei stride, GLpointer offset) {
+	_realTexCoordPointer(size, type, stride, (cc_uintptr)cur_vb->data + offset);
+}
+static void APIENTRY gl11_vertexPointer(GLint size, GLenum type, GLsizei stride, GLpointer offset) {
+	_realVertexPointer(size,   type, stride, (cc_uintptr)cur_vb->data + offset);
+}
+
+
+static void FallbackOpenGL(void) {
 	Window_ShowDialog("Performance warning",
 		"Your system only supports only OpenGL 1.1\n" \
 		"This is usually caused by graphics drivers not being installed\n\n" \
 		"As such you will likely experience very poor performance");
 	customMipmapsLevels = false;
 		
-	_glBindBuffer = fake_bindBuffer; _delBuffer    = DelFakeBuffer;
-	_genBuffer    = GenFakeBuffer;   _glBufferData = fake_bufferData;
-	_glBufferSubData = fake_bufferSubData;
+	_glGenBuffers    = legacy_genBuffer;
+	_glDeleteBuffers = legacy_deleteBuffer;
+	_glBindBuffer    = legacy_bindBuffer;
+	_glBufferData    = legacy_bufferData;
+	_glBufferSubData = legacy_bufferSubData;
 
-	_glDrawElements    = fake_drawElements;    _glColorPointer  = fake_colorPointer;
-	_glTexCoordPointer = fake_texCoordPointer; _glVertexPointer = fake_vertexPointer;
+	_realDrawElements    = _glDrawElements;    _realColorPointer  = _glColorPointer;
+	_realTexCoordPointer = _glTexCoordPointer; _realVertexPointer = _glVertexPointer;
+
+	_glDrawElements    = gl11_drawElements;    _glColorPointer  = gl11_colorPointer;
+	_glTexCoordPointer = gl11_texCoordPointer; _glVertexPointer = gl11_vertexPointer;
+
+	/* OpenGL 1.0 fallback support */
+	if (_realDrawElements) return;
+	Window_ShowDialog("Performance warning", "OpenGL 1.0 only support, expect awful performance");
+
+	_glDrawElements    = gl10_drawElements;    _glColorPointer  = gl10_colorPointer;
+	_glTexCoordPointer = gl10_texCoordPointer; _glVertexPointer = gl10_vertexPointer;
+
+	_glBindTexture    = gl10_bindTexture;
+	_glGenTextures    = gl10_genTexture;
+	_glDeleteTextures = gl10_deleteTexture;
+	_glTexImage2D     = gl10_texImage;
+	_glTexSubImage2D  = gl10_texSubImage;
+
+	_glDisableClientState = gl10_disableClientState;
+	_glEnableClientState  = gl10_enableClientState;
 }
 #else
 /* No point in even trying for other systems */
-static void OpenGL11Fallback(void) {
+static void FallbackOpenGL(void) {
 	Logger_FailToStart("Only OpenGL 1.1 supported.\n\n" \
 		"Compile the game with CC_BUILD_GL11, or ask on the ClassiCube forums for it");
 }
@@ -597,10 +721,11 @@ static void GLBackend_Init(void) {
 
 	/* Version string is always: x.y. (and whatever afterwards) */
 	int major = ver[0] - '0', minor = ver[2] - '0';
-#ifdef CC_BUILD_WIN
+#ifdef CC_BUILD_GL11_FALLBACK
 	LoadCoreFuncs();
 #endif
 	customMipmapsLevels = true;
+	Gfx.BackendType     = CC_GFX_BACKEND_GL1;
 
 	/* Supported in core since 1.5 */
 	if (major > 1 || (major == 1 && minor >= 5)) {
@@ -608,99 +733,8 @@ static void GLBackend_Init(void) {
 	} else if (String_CaselessContains(&extensions, &vboExt)) {
 		GLContext_GetAll(arbVboFuncs,  Array_Elems(arbVboFuncs));
 	} else {
-		OpenGL11Fallback();
+		FallbackOpenGL();
 	}
 }
 #endif
-
-
-/*########################################################################################################################*
-*----------------------------------------------------------Drawing--------------------------------------------------------*
-*#########################################################################################################################*/
-#ifdef CC_BUILD_GL11
-/* point to client side dynamic array */
-#define VB_PTR ((cc_uint8*)dynamicListData)
-#define IB_PTR gl_indices
-#else
-/* no client side array, use vertex buffer object */
-#define VB_PTR 0
-#define IB_PTR NULL
-#endif
-
-static void GL_SetupVbColoured(void) {
-	_glVertexPointer(3, GL_FLOAT,        SIZEOF_VERTEX_COLOURED, (void*)(VB_PTR + 0));
-	_glColorPointer(4, GL_UNSIGNED_BYTE, SIZEOF_VERTEX_COLOURED, (void*)(VB_PTR + 12));
-}
-
-static void GL_SetupVbTextured(void) {
-	_glVertexPointer(3, GL_FLOAT,        SIZEOF_VERTEX_TEXTURED, (void*)(VB_PTR + 0));
-	_glColorPointer(4, GL_UNSIGNED_BYTE, SIZEOF_VERTEX_TEXTURED, (void*)(VB_PTR + 12));
-	_glTexCoordPointer(2, GL_FLOAT,      SIZEOF_VERTEX_TEXTURED, (void*)(VB_PTR + 16));
-}
-
-static void GL_SetupVbColoured_Range(int startVertex) {
-	cc_uint32 offset = startVertex * SIZEOF_VERTEX_COLOURED;
-	_glVertexPointer(3, GL_FLOAT,          SIZEOF_VERTEX_COLOURED, (void*)(VB_PTR + offset));
-	_glColorPointer(4, GL_UNSIGNED_BYTE,   SIZEOF_VERTEX_COLOURED, (void*)(VB_PTR + offset + 12));
-}
-
-static void GL_SetupVbTextured_Range(int startVertex) {
-	cc_uint32 offset = startVertex * SIZEOF_VERTEX_TEXTURED;
-	_glVertexPointer(3,  GL_FLOAT,         SIZEOF_VERTEX_TEXTURED, (void*)(VB_PTR + offset));
-	_glColorPointer(4, GL_UNSIGNED_BYTE,   SIZEOF_VERTEX_TEXTURED, (void*)(VB_PTR + offset + 12));
-	_glTexCoordPointer(2, GL_FLOAT,        SIZEOF_VERTEX_TEXTURED, (void*)(VB_PTR + offset + 16));
-}
-
-void Gfx_SetVertexFormat(VertexFormat fmt) {
-	if (fmt == gfx_format) return;
-	gfx_format = fmt;
-	gfx_stride = strideSizes[fmt];
-
-	if (fmt == VERTEX_FORMAT_TEXTURED) {
-		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-		glEnable(GL_TEXTURE_2D);
-
-		gfx_setupVBFunc      = GL_SetupVbTextured;
-		gfx_setupVBRangeFunc = GL_SetupVbTextured_Range;
-	} else {
-		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-		glDisable(GL_TEXTURE_2D);
-
-		gfx_setupVBFunc      = GL_SetupVbColoured;
-		gfx_setupVBRangeFunc = GL_SetupVbColoured_Range;
-	}
-}
-
-void Gfx_DrawVb_Lines(int verticesCount) {
-	gfx_setupVBFunc();
-	glDrawArrays(GL_LINES, 0, verticesCount);
-}
-
-void Gfx_DrawVb_IndexedTris_Range(int verticesCount, int startVertex) {
-#ifdef CC_BUILD_GL11
-	if (activeList != gl_DYNAMICLISTID) { glCallList(activeList); return; }
-#endif
-	gfx_setupVBRangeFunc(startVertex);
-	_glDrawElements(GL_TRIANGLES, ICOUNT(verticesCount), GL_UNSIGNED_SHORT, IB_PTR);
-}
-
-void Gfx_DrawVb_IndexedTris(int verticesCount) {
-#ifdef CC_BUILD_GL11
-	if (activeList != gl_DYNAMICLISTID) { glCallList(activeList); return; }
-#endif
-	gfx_setupVBFunc();
-	_glDrawElements(GL_TRIANGLES, ICOUNT(verticesCount), GL_UNSIGNED_SHORT, IB_PTR);
-}
-
-#ifdef CC_BUILD_GL11
-void Gfx_DrawIndexedTris_T2fC4b(int verticesCount, int startVertex) { glCallList(activeList); }
-#else
-void Gfx_DrawIndexedTris_T2fC4b(int verticesCount, int startVertex) {
-	cc_uint32 offset = startVertex * SIZEOF_VERTEX_TEXTURED;
-	_glVertexPointer(3, GL_FLOAT,        SIZEOF_VERTEX_TEXTURED, (void*)(VB_PTR + offset));
-	_glColorPointer(4, GL_UNSIGNED_BYTE, SIZEOF_VERTEX_TEXTURED, (void*)(VB_PTR + offset + 12));
-	_glTexCoordPointer(2, GL_FLOAT,      SIZEOF_VERTEX_TEXTURED, (void*)(VB_PTR + offset + 16));
-	_glDrawElements(GL_TRIANGLES,        ICOUNT(verticesCount),   GL_UNSIGNED_SHORT, IB_PTR);
-}
-#endif /* !CC_BUILD_GL11 */
 #endif
