@@ -17,25 +17,32 @@
 #include "Funcs.h"
 #include "Server.h"
 #include "TexturePack.h"
+#include "InputHandler.h"
 
 struct _GuiData Gui;
 struct Screen* Gui_Screens[GUI_MAX_SCREENS];
 static cc_uint8 priorities[GUI_MAX_SCREENS];
-
+#ifdef CC_BUILD_DUALSCREEN
+static struct Texture touchBgTex;
+#endif
 
 /*########################################################################################################################*
 *----------------------------------------------------------Gui------------------------------------------------------------*
 *#########################################################################################################################*/
 static CC_NOINLINE int GetWindowScale(void) {
-	float widthScale  = WindowInfo.Width  / 640.0f;
-	float heightScale = WindowInfo.Height / 480.0f;
+	float widthScale  = Window_Main.Width  * Window_Main.UIScaleX;
+	float heightScale = Window_Main.Height * Window_Main.UIScaleY;
 
 	/* Use larger UI scaling on mobile */
 	/* TODO move this DPI scaling elsewhere.,. */
-	if (!Input_TouchMode) {
+#ifndef CC_BUILD_DUALSCREEN
+	if (!Gui_TouchUI) {
+#endif
 		widthScale  /= DisplayInfo.ScaleX;
 		heightScale /= DisplayInfo.ScaleY;
+#ifndef CC_BUILD_DUALSCREEN
 	}
+#endif
 	return 1 + (int)(min(widthScale, heightScale));
 }
 
@@ -52,9 +59,16 @@ float Gui_GetInventoryScale(void) {
 }
 
 float Gui_GetChatScale(void) {
-	return Gui_Scale(GetWindowScale() * Gui.RawChatScale);
+	if (Gui.AutoScaleChat) return Gui_Scale(GetWindowScale() * Gui.RawChatScale);
+	return Gui.RawChatScale;
 }
 
+float Gui_GetCrosshairScale(void) {
+	float heightScale = Window_Main.Height * Window_Main.UIScaleY;
+	return Gui_Scale(heightScale) * Gui.RawCrosshairScale;
+}
+
+
 void Gui_MakeTitleFont(struct FontDesc* font) { Font_Make(font, 16, FONT_FLAGS_BOLD); }
 void Gui_MakeBodyFont(struct FontDesc* font)  { Font_Make(font, 16, FONT_FLAGS_NONE); }
 
@@ -90,6 +104,12 @@ void Gui_ShowDefault(void) {
 #endif
 }
 
+#ifdef CC_BUILD_TOUCH
+void Gui_SetTouchUI(cc_bool enabled) {
+	Gui.TouchUI = enabled; /* TODO toggle or not */
+}
+#endif
+
 static void LoadOptions(void) {
 	Gui.DefaultLines    = Game_ClassicMode ? 10 : 12;
 	Gui.Chatlines       = Options_GetInt(OPT_CHATLINES, 0, GUI_MAX_CHATLINES, Gui.DefaultLines);
@@ -106,7 +126,10 @@ static void LoadOptions(void) {
 	Gui.RawInventoryScale = Options_GetFloat(OPT_INVENTORY_SCALE, 0.25f, 5.0f, 1.0f);
 	Gui.RawHotbarScale    = Options_GetFloat(OPT_HOTBAR_SCALE,    0.25f, 5.0f, 1.0f);
 	Gui.RawChatScale      = Options_GetFloat(OPT_CHAT_SCALE,      0.25f, 5.0f, 1.0f);
+	Gui.RawCrosshairScale = Options_GetFloat(OPT_CROSSHAIR_SCALE, 0.25f, 5.0f, 1.0f);
 	Gui.RawTouchScale     = Options_GetFloat(OPT_TOUCH_SCALE,     0.25f, 5.0f, 1.0f);
+
+	Gui.AutoScaleChat     = Options_GetBool(OPT_CHAT_AUTO_SCALE, true);
 }
 
 static void LoseAllScreens(void) {
@@ -183,7 +206,7 @@ static void Gui_AddCore(struct Screen* s, int priority) {
 	s->VTABLE->Init(s);
 	s->VTABLE->ContextRecreated(s);
 	s->VTABLE->Layout(s);
-
+	
 	/* for selecting active button etc */
 	for (i = 0; i < Pointers_Count; i++) 
 	{
@@ -201,9 +224,9 @@ static int IndexOfScreen(struct Screen* s) {
 	return -1;
 }
 
-void Gui_RemoveCore(struct Screen* s) {
+static cc_bool Gui_RemoveCore(struct Screen* s) {
 	int i = IndexOfScreen(s);
-	if (i == -1) return;
+	if (i == -1) return false;
 
 	for (; i < Gui.ScreensCount - 1; i++) 
 	{
@@ -214,6 +237,7 @@ void Gui_RemoveCore(struct Screen* s) {
 
 	s->VTABLE->ContextLost(s);
 	s->VTABLE->Free(s);
+	return true;
 }
 
 CC_NOINLINE static void Gui_OnScreensChanged(void) {
@@ -222,18 +246,16 @@ CC_NOINLINE static void Gui_OnScreensChanged(void) {
 }
 
 void Gui_Remove(struct Screen* s) {
-	Gui_RemoveCore(s);
-	Gui_OnScreensChanged();
+	cc_bool removed = Gui_RemoveCore(s);
+	if (removed) Gui_OnScreensChanged();
 }
 
 void Gui_Add(struct Screen* s, int priority) {
-	int i;
+	struct Screen* existing;
 	Gui_RemoveCore(s);
-	/* Backwards loop since removing changes count and gui_screens */
-	for (i = Gui.ScreensCount - 1; i >= 0; i--) 
-	{
-		if (priorities[i] == priority) Gui_RemoveCore(Gui_Screens[i]);
-	}
+	
+	existing = Gui_GetScreen(priority);
+	if (existing) Gui_RemoveCore(existing);
 
 	Gui_AddCore(s, priority);
 	Gui_OnScreensChanged();
@@ -266,6 +288,15 @@ struct Screen* Gui_GetClosable(void) {
 	return NULL;
 }
 
+struct Screen* Gui_GetScreen(int priority) {
+	int i;
+	for (i = 0; i < Gui.ScreensCount; i++) 
+	{
+		if (priorities[i] == priority) return Gui_Screens[i];
+	}
+	return NULL;
+}
+
 void Gui_UpdateInputGrab(void) {
 	Gui.InputGrab = Gui_GetInputGrab();
 	Camera_CheckFocus();
@@ -279,10 +310,37 @@ void Gui_ShowPauseMenu(void) {
 	}
 }
 
-void Gui_RenderGui(double delta) {
+void Gui_ShowCinematicBars() {
+	int screenWidth = Window_Main.Width;
+	int screenHeight = Window_Main.Height;
+
+	// Ensure bar size is clamped between 0 and 1
+	if (Gui.BarSize < 0.0f) Gui.BarSize = 0.0f;
+	if (Gui.BarSize > 1.0f) Gui.BarSize = 1.0f;
+
+	// If bar size is 1, just draw 1 rectangle instead of 2
+	if (Gui.BarSize == 1.0f) {
+		Gfx_Draw2DGradient(0, 0, screenWidth, screenHeight, Gui.CinematicBarColor, Gui.CinematicBarColor);
+	} else {
+		// Calculate the height of each bar based on the bar size
+		int barHeight = (int)(screenHeight * Gui.BarSize / 2.0f);
+
+		Gfx_Draw2DGradient(0, 0, screenWidth, barHeight, Gui.CinematicBarColor, Gui.CinematicBarColor);
+		Gfx_Draw2DGradient(0, screenHeight - barHeight, screenWidth, barHeight, Gui.CinematicBarColor, Gui.CinematicBarColor);
+	}
+}
+
+void Gui_RenderGui(float delta) {
 	struct Screen* s;
 	int i;
 
+	Gfx_3DS_SetRenderScreen(BOTTOM_SCREEN);
+#ifdef CC_BUILD_DUALSCREEN
+	Texture_Render(&touchBgTex);
+#endif
+
+	if (Gui.BarSize > 0) Gui_ShowCinematicBars();
+
 	/* Draw back to front so highest priority screen is on top */
 	for (i = Gui.ScreensCount - 1; i >= 0; i--) 
 	{
@@ -292,6 +350,8 @@ void Gui_RenderGui(double delta) {
 		if (s->dirty) { s->VTABLE->BuildMesh(s); s->dirty = false; }
 		s->VTABLE->Render(s, delta);
 	}
+
+	Gfx_3DS_SetRenderScreen(TOP_SCREEN);
 }
 
 
@@ -336,8 +396,8 @@ void TextAtlas_Make(struct TextAtlas* atlas, const cc_string* chars, struct Font
 	Context2D_Free(&ctx);
 
 	atlas->uScale = 1.0f / (float)ctx.bmp.width;
-	atlas->tex.uv.U2 = atlas->offset * atlas->uScale;
-	atlas->tex.Width = atlas->offset;	
+	atlas->tex.uv.u2 = atlas->offset * atlas->uScale;
+	atlas->tex.width = atlas->offset;	
 }
 
 void TextAtlas_Free(struct TextAtlas* atlas) { Gfx_DeleteTexture(&atlas->tex.ID); }
@@ -346,9 +406,9 @@ void TextAtlas_Add(struct TextAtlas* atlas, int charI, struct VertexTextured** v
 	struct Texture part = atlas->tex;
 	int width = atlas->widths[charI];
 
-	part.X  = atlas->curX; part.Width = width;
-	part.uv.U1 = atlas->offsets[charI] * atlas->uScale;
-	part.uv.U2 = part.uv.U1 + width    * atlas->uScale;
+	part.x  = atlas->curX; part.width = width;
+	part.uv.u1 = atlas->offsets[charI] * atlas->uScale;
+	part.uv.u2 = part.uv.u1 + width    * atlas->uScale;
 
 	atlas->curX += width;	
 	Gfx_Make2DQuad(&part, PACKEDCOL_WHITE, vertices);
@@ -378,13 +438,23 @@ void Widget_SetLocation(void* widget, cc_uint8 horAnchor, cc_uint8 verAnchor, in
 	w->horAnchor = horAnchor; w->verAnchor = verAnchor;
 	w->xOffset = Display_ScaleX(xOffset);
 	w->yOffset = Display_ScaleY(yOffset);
-	Widget_Layout(w);
+	if (w->VTABLE) Widget_Layout(w);
 }
 
 void Widget_CalcPosition(void* widget) {
 	struct Widget* w = (struct Widget*)widget;
-	w->x = Gui_CalcPos(w->horAnchor, w->xOffset, w->width , WindowInfo.Width );
-	w->y = Gui_CalcPos(w->verAnchor, w->yOffset, w->height, WindowInfo.Height);
+	int windowWidth, windowHeight;
+	
+#ifdef CC_BUILD_DUALSCREEN
+	windowWidth  = (w->flags & WIDGET_FLAG_MAINSCREEN) ? Window_Main.Width  : Window_Alt.Width;
+	windowHeight = (w->flags & WIDGET_FLAG_MAINSCREEN) ? Window_Main.Height : Window_Alt.Height;
+#else
+	windowWidth  = Window_Main.Width;
+	windowHeight = Window_Main.Height;
+#endif
+	
+	w->x = Gui_CalcPos(w->horAnchor, w->xOffset, w->width , windowWidth );
+	w->y = Gui_CalcPos(w->verAnchor, w->yOffset, w->height, windowHeight);
 }
 
 void Widget_Reset(void* widget) {
@@ -397,6 +467,7 @@ void Widget_Reset(void* widget) {
 	w->verAnchor = ANCHOR_MIN;
 	w->xOffset = 0; w->yOffset = 0;
 	w->MenuClick = NULL;
+	w->meta.ptr  = NULL;
 }
 
 int Widget_Contains(void* widget, int x, int y) {
@@ -418,7 +489,7 @@ void Widget_SetDisabled(void* widget, int disabled) {
 /*########################################################################################################################*
 *-------------------------------------------------------Screen base-------------------------------------------------------*
 *#########################################################################################################################*/
-void Screen_Render2Widgets(void* screen, double delta) {
+void Screen_Render2Widgets(void* screen, float delta) {
 	struct Screen* s = (struct Screen*)screen;
 	struct Widget** widgets = s->widgets;
 	int i, offset = 0;
@@ -435,7 +506,8 @@ void Screen_Render2Widgets(void* screen, double delta) {
 
 void Screen_UpdateVb(void* screen) {
 	struct Screen* s = (struct Screen*)screen;
-	Gfx_RecreateDynamicVb(&s->vb, VERTEX_FORMAT_TEXTURED, s->maxVertices);
+	Gfx_DeleteDynamicVb(&s->vb);
+	s->vb = Gfx_CreateDynamicVb(VERTEX_FORMAT_TEXTURED, s->maxVertices);
 }
 
 struct VertexTextured* Screen_LockVb(void* screen) {
@@ -454,31 +526,32 @@ int Screen_DoPointerDown(void* screen, int id, int x, int y) {
 	{
 		struct Widget* w = widgets[i];
 		if (!w || !Widget_Contains(w, x, y)) continue;
-		if (w->flags & WIDGET_FLAG_DISABLED) return i;
+		if (w->flags & WIDGET_FLAG_DISABLED) break;
 
 		if (w->MenuClick) {
 			w->MenuClick(s, w);
 		} else {
 			Elem_HandlesPointerDown(w, id, x, y);
 		}
-		return i;
+		break;
 	}
-	return -1;
+	return i;
 }
 
-int Screen_Index(void* screen, void* widget) {
+int Screen_CalcDefaultMaxVertices(void* screen) {
 	struct Screen* s = (struct Screen*)screen;
 	struct Widget** widgets = s->widgets;
-	int i;
+	int i, count = 0;
 
-	struct Widget* w = (struct Widget*)widget;
-	for (i = 0; i < s->numWidgets; i++) 
+	for (i = 0; i < s->numWidgets; i++)
 	{
-		if (widgets[i] == w) return i;
+		if (!widgets[i]) continue;
+		count += widgets[i]->VTABLE->GetMaxVertices(widgets[i]);
 	}
-	return -1;
+	return count;
 }
 
+
 void Screen_BuildMesh(void* screen) {
 	struct Screen* s = (struct Screen*)screen;
 	struct Widget** widgets = s->widgets;
@@ -522,35 +595,75 @@ void Screen_ContextLost(void* screen) {
 	}
 }
 
-int  Screen_InputDown(void* screen, int key) { return key < CCKEY_F1 || key > CCKEY_F24; }
-void Screen_InputUp(void*   screen, int key) { }
+int  Screen_InputDown(void* screen, int key, struct InputDevice* device) { return key < CCKEY_F1 || key > CCKEY_F24; }
+void Screen_InputUp(void*   screen, int key, struct InputDevice* device) { }
 void Screen_PointerUp(void* s, int id, int x, int y) { }
 
+/*########################################################################################################################*
+*------------------------------------------------------Input handling-----------------------------------------------------*
+*#########################################################################################################################*/
+static void OnMouseWheel(void* obj, float delta) {
+	struct Screen* s;
+	int i;
+	
+	for (i = 0; i < Gui.ScreensCount; i++) {
+		s = Gui_Screens[i];
+		s->dirty = true;
+		if (s->VTABLE->HandlesMouseScroll(s, delta)) return;
+	}
+}
+
+static void OnPointerMove(void* obj, int idx) {
+	struct Screen* s;
+	int i, x = Pointers[idx].x, y = Pointers[idx].y;
+
+	for (i = 0; i < Gui.ScreensCount; i++) {
+		s = Gui_Screens[i];
+		s->dirty = true;
+		if (s->VTABLE->HandlesPointerMove(s, 1 << idx, x, y)) return;
+	}
+}
+
+static void OnAxisUpdate(void* obj, int port, int axis, float x, float y) {
+	struct Screen* s;
+	int i;
+	
+	for (i = 0; i < Gui.ScreensCount; i++) {
+		s = Gui_Screens[i];
+		if (!s->VTABLE->HandlesPadAxis) continue;
+
+		s->dirty = true;
+		if (s->VTABLE->HandlesPadAxis(s, axis, x, y)) return;
+	}
+}
+
 
 /*########################################################################################################################*
 *------------------------------------------------------Gui component------------------------------------------------------*
 *#########################################################################################################################*/
 static void GuiPngProcess(struct Stream* stream, const cc_string* name) {
-	Game_UpdateTexture(&Gui.GuiTex, stream, name, NULL);
+	int heightDivisor = 2; /* only top half of gui png is used */
+	Game_UpdateTexture(&Gui.GuiTex, stream, name, NULL, &heightDivisor);
 }
 static struct TextureEntry gui_entry = { "gui.png", GuiPngProcess };
 
 static void GuiClassicPngProcess(struct Stream* stream, const cc_string* name) {
-	Game_UpdateTexture(&Gui.GuiClassicTex, stream, name, NULL);
+	int heightDivisor = 2; /* only top half of gui png is used */
+	Game_UpdateTexture(&Gui.GuiClassicTex, stream, name, NULL, &heightDivisor);
 }
 static struct TextureEntry guiClassic_entry = { "gui_classic.png", GuiClassicPngProcess };
 
 static void IconsPngProcess(struct Stream* stream, const cc_string* name) {
-	Game_UpdateTexture(&Gui.IconsTex, stream, name, NULL);
+	int heightDivisor = 4; /* only top quarter of icons png is used */
+	Game_UpdateTexture(&Gui.IconsTex, stream, name, NULL, &heightDivisor);
 }
 static struct TextureEntry icons_entry = { "icons.png", IconsPngProcess };
 
 static void TouchPngProcess(struct Stream* stream, const cc_string* name) {
-	Game_UpdateTexture(&Gui.TouchTex, stream, name, NULL);
+	Game_UpdateTexture(&Gui.TouchTex, stream, name, NULL, NULL);
 }
 static struct TextureEntry touch_entry = { "touch.png", TouchPngProcess };
 
-
 static void OnFontChanged(void* obj) { Gui_RefreshAll(); }
 
 static void OnKeyPress(void* obj, int cp) {
@@ -594,6 +707,24 @@ static void OnInit(void) {
 	TextureEntry_Register(&icons_entry);
 	TextureEntry_Register(&touch_entry);
 
+	Event_Register_(&InputEvents.Wheel,   NULL, OnMouseWheel);
+	Event_Register_(&PointerEvents.Moved, NULL, OnPointerMove);
+	Event_Register_(&ControllerEvents.AxisUpdate, NULL, OnAxisUpdate);
+
+#ifdef CC_BUILD_DUALSCREEN
+	struct Context2D ctx;
+	Context2D_Alloc(&ctx, 32, 32);
+	Gradient_Noise(&ctx, BitmapColor_RGB(0x40, 0x30, 0x20), 6, 0, 0, ctx.width, ctx.height);
+	Context2D_MakeTexture(&touchBgTex, &ctx);
+	Context2D_Free(&ctx);
+	
+	// Tile the texture to fill the entire screen
+	int tilesX = Math_CeilDiv(Window_Alt.Width,  ctx.width);
+	int tilesY = Math_CeilDiv(Window_Alt.Height, ctx.height);
+	touchBgTex.width *= tilesX; touchBgTex.height *= tilesY;
+	touchBgTex.uv.u2 *= tilesX; touchBgTex.uv.v2  *= tilesY;
+#endif
+
 	Event_Register_(&ChatEvents.FontChanged,     NULL, OnFontChanged);
 	Event_Register_(&GfxEvents.ContextLost,      NULL, OnContextLost);
 	Event_Register_(&GfxEvents.ContextRecreated, NULL, OnContextRecreated);
@@ -601,13 +732,6 @@ static void OnInit(void) {
 	Event_Register_(&WindowEvents.Resized,       NULL, OnResize);
 	Event_Register_(&InputEvents.TextChanged,    NULL, OnTextChanged);
 
-#ifdef CC_BUILD_TOUCH
-	#define DEFAULT_SP_ONSCREEN (ONSCREEN_BTN_FLY | ONSCREEN_BTN_SPEED)
-	#define DEFAULT_MP_ONSCREEN (ONSCREEN_BTN_FLY | ONSCREEN_BTN_SPEED | ONSCREEN_BTN_CHAT)
-	Gui._onscreenButtons = Options_GetInt(OPT_TOUCH_BUTTONS, 0, Int32_MaxValue,
-											Server.IsSinglePlayer ? DEFAULT_SP_ONSCREEN : DEFAULT_MP_ONSCREEN);
-#endif
-
 	LoadOptions();
 	Gui_ShowDefault();
 }
