@@ -1,6 +1,8 @@
 #ifndef CC_HELDBLOCKRENDERER_H
 #define CC_HELDBLOCKRENDERER_H
 #include "Core.h"
+CC_BEGIN_HEADER
+
 /* 
 Renders the held block/arm at bottom right of game
 Copyright 2014-2023 ClassiCube | Licensed under BSD-3
@@ -11,5 +13,7 @@ extern struct IGameComponent HeldBlockRenderer_Component;
 extern cc_bool HeldBlockRenderer_Show;
 
 void HeldBlockRenderer_ClickAnim(cc_bool digging);
-void HeldBlockRenderer_Render(double delta);
+void HeldBlockRenderer_Render(float delta);
+
+CC_END_HEADER
 #endif
