@@ -22,6 +22,7 @@
 #include "PackedCol.h"
 #include "SystemFonts.h"
 #include "TexturePack.h"
+#include "Gui.h"
 
 struct LScreen* Launcher_Active;
 cc_bool Launcher_ShouldExit, Launcher_ShouldUpdate;
@@ -35,10 +36,10 @@ static struct Bitmap dirtBmp, stoneBmp;
 #define TILESIZE 48
 
 static void CloseActiveScreen(void) {
-	Window_CloseKeyboard();
+	OnscreenKeyboard_Close();
 	if (!Launcher_Active) return;
 	
-	Launcher_Active->Free(Launcher_Active);
+	Launcher_Active->Deactivated(Launcher_Active);
 	LBackend_CloseScreen(Launcher_Active);
 	Launcher_Active = NULL;
 }
@@ -46,11 +47,13 @@ static void CloseActiveScreen(void) {
 void Launcher_SetScreen(struct LScreen* screen) {
 	CloseActiveScreen();
 	Launcher_Active = screen;
-	if (!screen->numWidgets) screen->Init(screen);
 
-	screen->Show(screen);
+	screen->Activated(screen);
 	screen->Layout(screen);
 
+	if (!screen->everShown) screen->LoadState(screen);
+	screen->everShown = true;
+
 	LBackend_SetScreen(screen);
 	LBackend_Redraw();
 }
@@ -62,8 +65,7 @@ void Launcher_DisplayHttpError(struct HttpRequest* req, const char* action, cc_s
 	if (res) {
 		/* Non HTTP error - this is not good */
 		Http_LogError(action, req);
-		String_Format2(dst, res >= 0x80000000 ? "&cError %h when %c" : "&cError %i when %c",
-						&res, action);
+		String_Format2(dst, "&cError %e when %c", &res, action);
 	} else if (status != 200) {
 		String_Format2(dst, "&c%i error when %c", &status, action);
 	} else {
@@ -76,7 +78,7 @@ void Launcher_DisplayHttpError(struct HttpRequest* req, const char* action, cc_s
 *--------------------------------------------------------Starter/Updater--------------------------------------------------*
 *#########################################################################################################################*/
 static cc_uint64 lastJoin;
-cc_bool Launcher_StartGame(const cc_string* user, const cc_string* mppass, const cc_string* ip, const cc_string* port, const cc_string* server) {
+cc_bool Launcher_StartGame(const cc_string* user, const cc_string* mppass, const cc_string* ip, const cc_string* port, const cc_string* server, int numStates) {
 	cc_string args[4]; int numArgs;
 	cc_uint64 now;
 	cc_result res;
@@ -88,11 +90,12 @@ cc_bool Launcher_StartGame(const cc_string* user, const cc_string* mppass, const
 	/* Save resume info */
 	if (server->length) {
 		Options_PauseSaving();
-		Options_Set(ROPT_SERVER, server);
-		Options_Set(ROPT_USER,   user);
-		Options_Set(ROPT_IP,     ip);
-		Options_Set(ROPT_PORT,   port);
-		Options_SetSecure(ROPT_MPPASS, mppass);
+			Options_Set(ROPT_SERVER, server);
+			Options_Set(ROPT_USER,   user);
+			Options_Set(ROPT_IP,     ip);
+			Options_Set(ROPT_PORT,   port);
+			Options_SetSecure(ROPT_MPPASS, mppass);
+		Options_ResumeSaving();
 	}
 	/* Save options BEFORE starting new game process */
 	/* Otherwise can get 'file already in use' errors on startup */
@@ -107,10 +110,15 @@ cc_bool Launcher_StartGame(const cc_string* user, const cc_string* mppass, const
 		numArgs = 4;
 	}
 
+#ifdef CC_BUILD_SPLITSCREEN
+	Game_NumStates = numStates;
+#endif
+
 	res = Process_StartGame2(args, numArgs);
 	if (res) { Logger_SysWarn(res, "starting game"); return false; }
 
 	Launcher_ShouldExit = Platform_SingleProcess || Options_GetBool(LOPT_AUTO_CLOSE, false);
+
 	return true;
 }
 
@@ -119,7 +127,7 @@ CC_NOINLINE static void StartFromInfo(struct ServerInfo* info) {
 	String_InitArray(port, portBuffer);
 
 	String_AppendInt(&port, info->port);
-	Launcher_StartGame(&Launcher_Username, &info->mppass, &info->ip, &port, &info->name);
+	Launcher_StartGame(&Launcher_Username, &info->mppass, &info->ip, &port, &info->name, 1);
 }
 
 static void ConnectToServerError(struct HttpRequest* req) {
@@ -181,15 +189,21 @@ static cc_bool IsShutdown(int key) {
 #endif
 }
 
-static void OnInputDown(void* obj, int key, cc_bool was) {
+static void OnInputDown(void* obj, int key, cc_bool was, struct InputDevice* device) {
+	if (Input.DownHook) { Input.DownHook(key, device); return; }
+
 	if (IsShutdown(key)) Launcher_ShouldExit = true;
-	Launcher_Active->KeyDown(Launcher_Active, key, was);
+	Launcher_Active->KeyDown(Launcher_Active, key, was, device);
 }
 
 static void OnMouseWheel(void* obj, float delta) {
 	Launcher_Active->MouseWheel(Launcher_Active, delta);
 }
 
+static void OnClosing(void* obj) {
+	Launcher_ShouldExit = true;
+}
+
 
 /*########################################################################################################################*
 *-----------------------------------------------------------Main body-----------------------------------------------------*
@@ -197,8 +211,9 @@ static void OnMouseWheel(void* obj, float delta) {
 static void Launcher_Init(void) {
 	Event_Register_(&WindowEvents.Resized,      NULL, OnResize);
 	Event_Register_(&WindowEvents.StateChanged, NULL, OnResize);
+	Event_Register_(&WindowEvents.Closing,      NULL, OnClosing);
 
-	Event_Register_(&InputEvents.Down,          NULL, OnInputDown);
+	Event_Register_(&InputEvents.Down2,         NULL, OnInputDown);
 	Event_Register_(&InputEvents.Wheel,         NULL, OnMouseWheel);
 
 	Utils_EnsureDirectory("texpacks");
@@ -231,7 +246,6 @@ void Launcher_Run(void) {
 		Options_Set("update-dirty", NULL);
 	}
 #endif
-
 	Drawer2D_Component.Init();
 	SystemFonts_Component.Init();
 	Drawer2D.BitmappedText    = false;
@@ -241,6 +255,8 @@ void Launcher_Run(void) {
 	LBackend_InitFramebuffer();
 	Launcher_ShowEmptyServers = Options_GetBool(LOPT_SHOW_EMPTY, true);
 	Options_Get(LOPT_USERNAME, &Launcher_Username, "");
+	/* Some window backends require priming a few frames in case e.g. returning from in-game */
+	LBackend_AddDirtyFrames(4);
 
 	LWebTasks_Init();
 	Session_Load();
@@ -253,21 +269,22 @@ void Launcher_Run(void) {
 	Http_Component.Init();
 	CheckUpdateTask_Run();
 
-#ifdef CC_BUILD_FLATPAK
-	MainScreen_SetActive();
-#else
+#ifdef CC_BUILD_RESOURCES
 	Resources_CheckExistence();
 
-	if (Resources_Count) {
+	if (Resources_MissingCount) {
 		CheckResourcesScreen_SetActive();
 	} else {
 		MainScreen_SetActive();
 	}
+#else
+	MainScreen_SetActive();
 #endif
 
 	for (;;) {
-		Window_ProcessEvents(10 / 1000.0);
-		if (!WindowInfo.Exists || Launcher_ShouldExit) break;
+		Window_ProcessEvents(10 / 1000.0f);
+		Gamepad_Tick(10 / 1000.0f);
+		if (!Window_Main.Exists || Launcher_ShouldExit) break;
 
 		Launcher_Active->Tick(Launcher_Active);
 		LBackend_Tick();
@@ -283,15 +300,14 @@ void Launcher_Run(void) {
 	Platform_LogConst("undoing components");
 	Drawer2D_Component.Free();
 	Http_Component.Free();
-#else
+#endif
+
 	if (Launcher_ShouldUpdate) {
 		const char* action;
 		cc_result res = Updater_Start(&action);
 		if (res) Logger_SysWarn(res, action);
 	}
-
-	if (WindowInfo.Exists) Window_Close();
-#endif
+	Window_Destroy();
 }
 
 
@@ -326,9 +342,7 @@ const struct LauncherTheme Launcher_NordicTheme = {
 
 CC_NOINLINE static void ParseColor(const char* key, BitmapCol* color) {
 	cc_uint8 rgb[3];
-	cc_string value;
-	if (!Options_UNSAFE_Get(key, &value))    return;
-	if (!PackedCol_TryParseHex(&value, rgb)) return;
+	if (!Options_GetColor(key, rgb)) return;
 
 	*color = BitmapColor_RGB(rgb[0], rgb[1], rgb[2]);
 }
@@ -359,12 +373,14 @@ CC_NOINLINE static void SaveColor(const char* key, BitmapCol color) {
 }
 
 void Launcher_SaveTheme(void) {
-	SaveColor("launcher-back-col",                   Launcher_Theme.BackgroundColor);
-	SaveColor("launcher-btn-border-col",             Launcher_Theme.ButtonBorderColor);
-	SaveColor("launcher-btn-fore-active-col",        Launcher_Theme.ButtonForeActiveColor);
-	SaveColor("launcher-btn-fore-inactive-col",      Launcher_Theme.ButtonForeColor);
-	SaveColor("launcher-btn-highlight-inactive-col", Launcher_Theme.ButtonHighlightColor);
-	Options_SetBool("nostalgia-classicbg",           Launcher_Theme.ClassicBackground);
+	Options_PauseSaving();
+		SaveColor("launcher-back-col",                   Launcher_Theme.BackgroundColor);
+		SaveColor("launcher-btn-border-col",             Launcher_Theme.ButtonBorderColor);
+		SaveColor("launcher-btn-fore-active-col",        Launcher_Theme.ButtonForeActiveColor);
+		SaveColor("launcher-btn-fore-inactive-col",      Launcher_Theme.ButtonForeColor);
+		SaveColor("launcher-btn-highlight-inactive-col", Launcher_Theme.ButtonHighlightColor);
+		Options_SetBool("nostalgia-classicbg",           Launcher_Theme.ClassicBackground);
+	Options_ResumeSaving();
 }
 
 
@@ -415,6 +431,7 @@ static cc_result Launcher_ProcessZipEntry(const cc_string* path, struct Stream*
 	struct Bitmap bmp;
 	cc_result res;
 
+
 	if (String_CaselessEqualsConst(path, "default.png")) {
 		if (hasBitmappedFont) return 0;
 		hasBitmappedFont = false;
@@ -442,6 +459,7 @@ static cc_result Launcher_ProcessZipEntry(const cc_string* path, struct Stream*
 }
 
 static cc_result ExtractTexturePack(const cc_string* path) {
+	struct ZipEntry entries[32];
 	struct Stream stream;
 	cc_result res;
 
@@ -450,8 +468,8 @@ static cc_result ExtractTexturePack(const cc_string* path) {
 	if (res) { Logger_SysWarn(res, "opening texture pack"); return res; }
 
 	res = Zip_Extract(&stream, 
-			Launcher_SelectZipEntry, Launcher_ProcessZipEntry);
-
+			Launcher_SelectZipEntry, Launcher_ProcessZipEntry,
+			entries, Array_Elems(entries));
 	if (res) { Logger_SysWarn(res, "extracting texture pack"); }
 	/* No point logging error for closing readonly file */
 	(void)stream.Close(&stream);
@@ -520,23 +538,56 @@ cc_bool Launcher_BitmappedText(void) {
 	return (useBitmappedFont || Launcher_Theme.ClassicBackground) && hasBitmappedFont;
 }
 
-void Launcher_DrawTitle(struct FontDesc* font, const char* text, struct Context2D* ctx) {
+static void DrawTitleText(struct FontDesc* font, const char* text, struct Context2D* ctx, 
+				cc_uint8 horAnchor, cc_uint8 verAnchor) {
 	cc_string title = String_FromReadonly(text);
 	struct DrawTextArgs args;
-	int x;
-
-	/* Skip dragging logo when very small window to save space */
-	if (WindowInfo.Height < 300) return;
-
+	int x, y;
+	
 	DrawTextArgs_Make(&args, &title, font, false);
-	x = ctx->width / 2 - Drawer2D_TextWidth(&args) / 2;
-
+	x = Gui_CalcPos(horAnchor, 0, Drawer2D_TextWidth(&args),  ctx->width);
+	y = Gui_CalcPos(verAnchor, 0, Drawer2D_TextHeight(&args), ctx->height);
+	
 	Drawer2D.Colors['f'] = BITMAPCOLOR_BLACK;
-	Context2D_DrawText(ctx, &args, x + Display_ScaleX(4), Display_ScaleY(4));
+	Context2D_DrawText(ctx, &args, x + Display_ScaleX(4), y + Display_ScaleY(4));
 	Drawer2D.Colors['f'] = BITMAPCOLOR_WHITE;
-	Context2D_DrawText(ctx, &args, x,                     0);
+	Context2D_DrawText(ctx, &args, x,                     y);
 }
 
+#ifdef CC_BUILD_DUALSCREEN
+extern cc_bool launcherTop;
+
+void Launcher_DrawTitle(struct FontDesc* font, const char* text, struct Context2D* ctx) {
+	/* Put title on top screen */
+	struct Context2D topCtx;
+	struct Bitmap bmp;
+	int width  = Window_Alt.Width;
+	int height = Window_Alt.Height;
+	launcherTop = true;
+
+	ctx = &topCtx;
+	Window_AllocFramebuffer(&bmp, width, height);
+	Context2D_Wrap(ctx, &bmp);
+	topCtx.width  = width;
+	topCtx.height = height;
+	
+	Launcher_DrawBackgroundAll(ctx);
+	DrawTitleText(font, text, ctx, ANCHOR_CENTRE, ANCHOR_CENTRE);
+	Rect2D rect = { 0, 0, bmp.width, bmp.height };
+	Window_DrawFramebuffer(rect, &bmp);
+	
+	Window_FreeFramebuffer(&bmp);
+	launcherTop = false;
+}
+#else
+void Launcher_DrawTitle(struct FontDesc* font, const char* text, struct Context2D* ctx) {
+	/* Skip dragging logo when very small window to save space */
+	if (Window_Main.Height < 240) return;
+	
+	DrawTitleText(font, text, ctx, ANCHOR_CENTRE, ANCHOR_MIN);
+}
+#endif
+
 void Launcher_MakeTitleFont(struct FontDesc* font) {
 	Drawer2D.BitmappedText = Launcher_BitmappedText();
 	Font_Make(font, 32, FONT_FLAGS_NONE);
