@@ -2,6 +2,8 @@
 #define CC_SELECTIONBOX_H
 #include "Vectors.h"
 #include "PackedCol.h"
+CC_BEGIN_HEADER
+
 /* Describes a selection box, and contains methods related to the selection box.
    Copyright 2014-2023 ClassiCube | Licensed under BSD-3
 */
@@ -13,4 +15,6 @@ void Selections_Render(void);
 CC_API void Selections_Add(cc_uint8 id, const IVec3* p1, const IVec3* p2, PackedCol color);
 /* Removes the selection box with the givne ID */
 CC_API void Selections_Remove(cc_uint8 id);
+
+CC_END_HEADER
 #endif
