@@ -3,6 +3,7 @@
 #include "Window.h"
 #include "Platform.h"
 #include "Input.h"
+#include "InputHandler.h"
 #include "Event.h"
 #include "String.h"
 #include "Funcs.h"
@@ -10,34 +11,30 @@
 #include "Errors.h"
 #include "ExtMath.h"
 #include "Graphics.h"
+#include "VirtualKeyboard.h"
 #include <gccore.h>
 #if defined HW_RVL
+#include <ogc/usbmouse.h>
 #include <wiiuse/wpad.h>
 #include <wiikeyboard/keyboard.h>
 #endif
 
+static cc_bool needsFBUpdate;
 static cc_bool launcherMode;
+static int mouseSupported;
+#include "VirtualCursor.h"
 static void* xfb;
 static GXRModeObj* rmode;
 void* Window_XFB;
 struct _DisplayData DisplayInfo;
-struct _WinData WindowInfo;
-// no DPI scaling on Wii/GameCube
-int Display_ScaleX(int x) { return x; }
-int Display_ScaleY(int y) { return y; }
+struct cc_window WindowInfo;
 
 
 static void OnPowerOff(void) {
-	Event_RaiseVoid(&WindowEvents.Closing);
-	WindowInfo.Exists = false;
+	Window_Main.Exists = false;
+	Window_RequestClose();
 }
-
-void Window_Init(void) {	
-	// TODO: SYS_SetResetCallback(reload); too? not sure how reset differs on GC/WII
-	#if defined HW_RVL
-	SYS_SetPowerCallback(OnPowerOff);
-	#endif
-	
+static void InitVideo(void) {
 	// Initialise the video system
 	VIDEO_Init();
 
@@ -61,106 +58,120 @@ void Window_Init(void) {
 	VIDEO_Flush();
 	// Wait for Video setup to complete
 	VIDEO_WaitVSync();
-	
+}
+
+void Window_PreInit(void) {
+	// TODO: SYS_SetResetCallback(reload); too? not sure how reset differs on GC/WII
+	#if defined HW_RVL
+	SYS_SetPowerCallback(OnPowerOff);
+	#endif
+	InitVideo();
+}
+
+void Window_Init(void) {
 	DisplayInfo.Width  = rmode->fbWidth;
 	DisplayInfo.Height = rmode->xfbHeight;
 	DisplayInfo.ScaleX = 1;
 	DisplayInfo.ScaleY = 1;
 	
-	WindowInfo.Width   = rmode->fbWidth;
-	WindowInfo.Height  = rmode->xfbHeight;
-	WindowInfo.Focused = true;
-	WindowInfo.Exists  = true;
+	Window_Main.Width    = rmode->fbWidth;
+	Window_Main.Height   = rmode->xfbHeight;
+	Window_Main.Focused  = true;
+	
+	Window_Main.Exists   = true;
+	Window_Main.UIScaleX = DEFAULT_UI_SCALE_X;
+	Window_Main.UIScaleY = DEFAULT_UI_SCALE_Y;
+
+	DisplayInfo.ContentOffsetX = 10;
+	DisplayInfo.ContentOffsetY = 10;
+	Window_Main.SoftKeyboard   = SOFT_KEYBOARD_VIRTUAL;
 
-	Input.GamepadSource = true;
 	#if defined HW_RVL
-	WPAD_Init();
-	WPAD_SetDataFormat(0, WPAD_FMT_BTNS_ACC_IR);
 	KEYBOARD_Init(NULL);
+	mouseSupported = MOUSE_Init() >= 0;
 	#endif
-	PAD_Init();
 }
 
-void Window_Create2D(int width, int height) { launcherMode = true;  }
-void Window_Create3D(int width, int height) { launcherMode = false; }
+void Window_Free(void) { }
 
-void Window_Close(void) {
-	/* TODO implement */
+void Window_Create2D(int width, int height) {
+	needsFBUpdate = true;
+	launcherMode  = true;  
+}
+
+void Window_Create3D(int width, int height) { 
+	launcherMode = false; 
+}
+
+void Window_Destroy(void) { }
+
+void Window_RequestClose(void) {
+	Event_RaiseVoid(&WindowEvents.Closing);
 }
 
 
 /*########################################################################################################################*
 *---------------------------------------------GameCube controller processing----------------------------------------------*
 *#########################################################################################################################*/
-static PADStatus gc_pad;
-
-static void ProcessPAD_LeftJoystick(void) {
-	int dx = gc_pad.stickX;
-	int dy = gc_pad.stickY;
+static PADStatus gc_pads[PAD_CHANMAX];
 
+#define PAD_AXIS_SCALE 8.0f
+static void ProcessPAD_Joystick(int port, int axis, int x, int y, float delta) {
 	// May not be exactly 0 on actual hardware
-	if (Math_AbsI(dx) <= 8) dx = 0;
-	if (Math_AbsI(dy) <= 8) dy = 0;
+	if (Math_AbsI(x) <= 8) x = 0;
+	if (Math_AbsI(y) <= 8) y = 0;
 	
-	if (dx == 0 && dy == 0) return;	
-	Input.JoystickMovement = true;
-	Input.JoystickAngle    = Math_Atan2(dx, -dy);
+	Gamepad_SetAxis(port, axis, x / PAD_AXIS_SCALE, -y / PAD_AXIS_SCALE, delta);
 }
 
-static void ProcessPAD_RightJoystick(double delta) {
-	float scale = (delta * 60.0) / 8.0f;
-	int dx = gc_pad.substickX;
-	int dy = gc_pad.substickY;
-
-	// May not be exactly 0 on actual hardware
-	if (Math_AbsI(dx) <= 8) dx = 0;
-	if (Math_AbsI(dy) <= 8) dy = 0;
+static void ProcessPAD_Buttons(int port, int mods) {
+	Gamepad_SetButton(port, CCPAD_L, mods & PAD_TRIGGER_L);
+	Gamepad_SetButton(port, CCPAD_R, mods & PAD_TRIGGER_R);
 	
-	Event_RaiseRawMove(&PointerEvents.RawMoved, dx * scale, -dy * scale);		
+	Gamepad_SetButton(port, CCPAD_1, mods & PAD_BUTTON_A);
+	Gamepad_SetButton(port, CCPAD_2, mods & PAD_BUTTON_B);
+	Gamepad_SetButton(port, CCPAD_3, mods & PAD_BUTTON_X);
+	Gamepad_SetButton(port, CCPAD_4, mods & PAD_BUTTON_Y);
+	
+	Gamepad_SetButton(port, CCPAD_START,  mods & PAD_BUTTON_START);
+	Gamepad_SetButton(port, CCPAD_SELECT, mods & PAD_TRIGGER_Z);
+	
+	Gamepad_SetButton(port, CCPAD_LEFT,   mods & PAD_BUTTON_LEFT);
+	Gamepad_SetButton(port, CCPAD_RIGHT,  mods & PAD_BUTTON_RIGHT);
+	Gamepad_SetButton(port, CCPAD_UP,     mods & PAD_BUTTON_UP);
+	Gamepad_SetButton(port, CCPAD_DOWN,   mods & PAD_BUTTON_DOWN);
 }
 
-static void ProcessPAD_Buttons(void) {
-	int mods = gc_pad.button;
-	Input_SetNonRepeatable(CCPAD_L, mods & PAD_TRIGGER_L);
-	Input_SetNonRepeatable(CCPAD_R, mods & PAD_TRIGGER_R);
-	
-	Input_SetNonRepeatable(CCPAD_A, mods & PAD_BUTTON_A);
-	Input_SetNonRepeatable(CCPAD_B, mods & PAD_BUTTON_B);
-	Input_SetNonRepeatable(CCPAD_X, mods & PAD_BUTTON_X);
-	Input_SetNonRepeatable(CCPAD_Y, mods & PAD_BUTTON_Y);
-	
-	Input_SetNonRepeatable(CCPAD_START,  mods & PAD_BUTTON_START);
-	Input_SetNonRepeatable(CCPAD_SELECT, mods & PAD_TRIGGER_Z);
-	
-	Input_SetNonRepeatable(CCPAD_LEFT,   mods & PAD_BUTTON_LEFT);
-	Input_SetNonRepeatable(CCPAD_RIGHT,  mods & PAD_BUTTON_RIGHT);
-	Input_SetNonRepeatable(CCPAD_UP,     mods & PAD_BUTTON_UP);
-	Input_SetNonRepeatable(CCPAD_DOWN,   mods & PAD_BUTTON_DOWN);
-}
-
-static void ProcessPADInput(double delta) {
-	PADStatus pads[4];
-	PAD_Read(pads);
-	int error = pads[0].err;
+static void ProcessPADInput(PADStatus* pad, int i, float delta) {
+	int error = pad->err;
 
 	if (error == 0) {
-		gc_pad = pads[0]; // new state arrived
+		gc_pads[i] = *pad; // new state arrived
 	} else if (error == PAD_ERR_TRANSFER) {
 		// usually means still busy transferring state - use last state
 	} else {
 		return; // not connected, still busy, etc
 	}
 	
-	ProcessPAD_Buttons();
-	if (Input.RawMode) {
-		ProcessPAD_LeftJoystick();
-		ProcessPAD_RightJoystick(delta);
+	int port = Gamepad_Connect(0x5C + i, PadBind_Defaults);
+	ProcessPAD_Buttons(port, gc_pads[i].button);
+	ProcessPAD_Joystick(port, PAD_AXIS_LEFT,  gc_pads[i].stickX,    gc_pads[i].stickY,    delta);
+	ProcessPAD_Joystick(port, PAD_AXIS_RIGHT, gc_pads[i].substickX, gc_pads[i].substickY, delta);
+}
+
+static void ProcessPADInputs(float delta) {
+	PADStatus pads[PAD_CHANMAX];
+	PAD_Read(pads);
+
+	for (int i = 0; i < PAD_CHANMAX; i++)
+	{
+		ProcessPADInput(&pads[i], i, delta);
 	}
 }
 
 
 /*########################################################################################################################*
-*--------------------------------------------------Kebyaord processing----------------------------------------------------*
+*--------------------------------------------------Keyboard processing----------------------------------------------------*
 *#########################################################################################################################*/
 #if defined HW_RVL
 static const cc_uint8 key_map[] = {
@@ -204,6 +215,9 @@ static void ProcessKeyboardInput(void) {
 	int res = KEYBOARD_GetEvent(&ke);
 	int key;
 	
+	if (!res) return;
+	Input.Sources |= INPUT_SOURCE_NORMAL;
+	
 	if (res && ke.type == KEYBOARD_PRESSED)
 	{
 		key = MapNativeKey(ke.keycode);
@@ -220,6 +234,35 @@ static void ProcessKeyboardInput(void) {
 #endif
 
 
+/*########################################################################################################################*
+*---------------------------------------------------Mouse processing------------------------------------------------------*
+*#########################################################################################################################*/
+#if defined HW_RVL
+static void ProcessMouseInput(float delta) {
+	if (!mouseSupported)      return;
+	if (!MOUSE_IsConnected()) return;
+
+	mouse_event event;
+    if (MOUSE_GetEvent(&event) == 0) return;
+
+	Input_SetNonRepeatable(CCMOUSE_L, event.button & 1);
+	Input_SetNonRepeatable(CCMOUSE_R, event.button & 2);
+	Input_SetNonRepeatable(CCMOUSE_M, event.button & 4);
+	Mouse_ScrollVWheel(event.rz * 0.5f);
+
+	if (!vc_hooked) {
+		Pointer_SetPosition(0, Window_Main.Width / 2, Window_Main.Height / 2);
+	}
+	VirtualCursor_SetPosition(Pointers[0].x + event.rx, Pointers[0].y + event.ry);
+	
+	if (!Input.RawMode) return;	
+	float scale = (delta * 60.0) / 2.0f;
+	Event_RaiseRawMove(&PointerEvents.RawMoved, 
+				event.rx * scale, event.ry * scale);
+}
+#endif
+
+
 /*########################################################################################################################*
 *----------------------------------------------------Input processing-----------------------------------------------------*
 *#########################################################################################################################*/
@@ -228,118 +271,11 @@ static int dragCurX, dragCurY;
 static int dragStartX, dragStartY;
 static cc_bool dragActive;
 
-static void ProcessWPAD_Buttons(int mods) {
-	Input_SetNonRepeatable(CCPAD_L, mods & WPAD_BUTTON_1);
-	Input_SetNonRepeatable(CCPAD_R, mods & WPAD_BUTTON_2);
-      
-	Input_SetNonRepeatable(CCPAD_A, mods & WPAD_BUTTON_A);
-	Input_SetNonRepeatable(CCPAD_B, mods & WPAD_BUTTON_B);
-	Input_SetNonRepeatable(CCPAD_X, mods & WPAD_BUTTON_PLUS);
-      
-	Input_SetNonRepeatable(CCPAD_START,  mods & WPAD_BUTTON_HOME);
-	Input_SetNonRepeatable(CCPAD_SELECT, mods & WPAD_BUTTON_MINUS);
-
-	Input_SetNonRepeatable(CCPAD_LEFT,   mods & WPAD_BUTTON_LEFT);
-	Input_SetNonRepeatable(CCPAD_RIGHT,  mods & WPAD_BUTTON_RIGHT);
-	Input_SetNonRepeatable(CCPAD_UP,     mods & WPAD_BUTTON_UP);
-	Input_SetNonRepeatable(CCPAD_DOWN,   mods & WPAD_BUTTON_DOWN);
+void Window_ProcessEvents(float delta) {
+	ProcessKeyboardInput();
+    ProcessMouseInput(delta);
 }
 
-static void ProcessNunchuck_Game(int mods, double delta) {
-	WPADData* wd = WPAD_Data(0);
-	joystick_t analog = wd->exp.nunchuk.js;
-
-	Input_SetNonRepeatable(CCPAD_L, mods & WPAD_NUNCHUK_BUTTON_C);
-	Input_SetNonRepeatable(CCPAD_R, mods & WPAD_NUNCHUK_BUTTON_Z);
-      
-	Input_SetNonRepeatable(CCPAD_A, mods & WPAD_BUTTON_A);
-	Input_SetNonRepeatable(CCPAD_Y, mods & WPAD_BUTTON_1);
-	Input_SetNonRepeatable(CCPAD_X, mods & WPAD_BUTTON_2);
-
-	Input_SetNonRepeatable(CCPAD_START,  mods & WPAD_BUTTON_HOME);
-	Input_SetNonRepeatable(CCPAD_SELECT, mods & WPAD_BUTTON_MINUS);
-
-	Input_SetNonRepeatable(KeyBinds[KEYBIND_FLY], mods & WPAD_BUTTON_LEFT);
-
-	if (mods & WPAD_BUTTON_RIGHT) {
-		Mouse_ScrollWheel(1.0*delta);
-	}
-
-	Input_SetNonRepeatable(KeyBinds[KEYBIND_THIRD_PERSON], mods & WPAD_BUTTON_UP);
-	Input_SetNonRepeatable(KeyBinds[KEYBIND_FLY_DOWN],    mods & WPAD_BUTTON_DOWN);
-
-	const float ANGLE_DELTA = 50;
-	bool nunchuckUp    = (analog.ang > -ANGLE_DELTA)    && (analog.ang < ANGLE_DELTA)     && (analog.mag > 0.5);
-	bool nunchuckDown  = (analog.ang > 180-ANGLE_DELTA) && (analog.ang < 180+ANGLE_DELTA) && (analog.mag > 0.5);
-	bool nunchuckLeft  = (analog.ang > -90-ANGLE_DELTA) && (analog.ang < -90+ANGLE_DELTA) && (analog.mag > 0.5);
-	bool nunchuckRight = (analog.ang > 90-ANGLE_DELTA)  && (analog.ang < 90+ANGLE_DELTA)  && (analog.mag > 0.5);
-
-	Input_SetNonRepeatable(CCPAD_LEFT,  nunchuckLeft);
-	Input_SetNonRepeatable(CCPAD_RIGHT, nunchuckRight);
-	Input_SetNonRepeatable(CCPAD_UP,    nunchuckUp);
-	Input_SetNonRepeatable(CCPAD_DOWN,  nunchuckDown);
-}
-
-
-static void ProcessClassic_LeftJoystick(struct joystick_t* js) {
-	// TODO: need to account for min/max??
-	int dx = js->pos.x - js->center.x;
-	int dy = js->pos.y - js->center.y;
-	
-	if (Math_AbsI(dx) <= 8) dx = 0;
-	if (Math_AbsI(dy) <= 8) dy = 0;
-	
-	if (dx == 0 && dy == 0) return;
-	Input.JoystickMovement = true;
-	Input.JoystickAngle    = (js->ang - 90) * MATH_DEG2RAD;
-}
-
-static void ProcessClassic_RightJoystick(struct joystick_t* js, double delta) {
-	float scale = (delta * 60.0) / 2.0f;
-	// TODO: need to account for min/max??
-	int dx = js->pos.x - js->center.x;
-	int dy = js->pos.y - js->center.y;
-	
-	if (Math_AbsI(dx) <= 8) dx = 0;
-	if (Math_AbsI(dy) <= 8) dy = 0;
-	
-	Event_RaiseRawMove(&PointerEvents.RawMoved, dx * scale, -dy * scale);
-}
-
-static void ProcessClassicButtons(int mods) {
-	Input_SetNonRepeatable(CCPAD_L, mods & CLASSIC_CTRL_BUTTON_FULL_L);
-	Input_SetNonRepeatable(CCPAD_R, mods & CLASSIC_CTRL_BUTTON_FULL_R);
-      
-	Input_SetNonRepeatable(CCPAD_A, mods & CLASSIC_CTRL_BUTTON_A);
-	Input_SetNonRepeatable(CCPAD_B, mods & CLASSIC_CTRL_BUTTON_B);
-	Input_SetNonRepeatable(CCPAD_X, mods & CLASSIC_CTRL_BUTTON_X);
-	Input_SetNonRepeatable(CCPAD_Y, mods & CLASSIC_CTRL_BUTTON_Y);
-      
-	Input_SetNonRepeatable(CCPAD_START,  mods & CLASSIC_CTRL_BUTTON_PLUS);
-	Input_SetNonRepeatable(CCPAD_SELECT, mods & CLASSIC_CTRL_BUTTON_MINUS);
-
-	Input_SetNonRepeatable(CCPAD_LEFT,   mods & CLASSIC_CTRL_BUTTON_LEFT);
-	Input_SetNonRepeatable(CCPAD_RIGHT,  mods & CLASSIC_CTRL_BUTTON_RIGHT);
-	Input_SetNonRepeatable(CCPAD_UP,     mods & CLASSIC_CTRL_BUTTON_UP);
-	Input_SetNonRepeatable(CCPAD_DOWN,   mods & CLASSIC_CTRL_BUTTON_DOWN);
-	
-	Input_SetNonRepeatable(CCPAD_ZL, mods & CLASSIC_CTRL_BUTTON_ZL);
-	Input_SetNonRepeatable(CCPAD_ZR, mods & CLASSIC_CTRL_BUTTON_ZR);
-}
-
-static void ProcessClassicInput(double delta) {
-	WPADData* wd = WPAD_Data(0);
-	classic_ctrl_t ctrls = wd->exp.classic;
-	int mods = ctrls.btns | ctrls.btns_held;
-
-	ProcessClassicButtons(mods);
-	if (Input.RawMode) {
-		ProcessClassic_LeftJoystick(&ctrls.ljs);
-		ProcessClassic_RightJoystick(&ctrls.rjs, delta);
-	}
-}
-
-
 static void GetIRPos(int res, int* x, int* y) {
 	if (res == WPAD_ERR_NONE) {
 		WPADData* wd = WPAD_Data(0);
@@ -352,23 +288,16 @@ static void GetIRPos(int res, int* x, int* y) {
 	}
 }
 
-static void ProcessWPADInput(double delta) {	
-	WPAD_ScanPads();
-	u32 mods = WPAD_ButtonsDown(0) | WPAD_ButtonsHeld(0);
+static void ScanAndGetIRPos(int* x, int* y) {
 	u32 type;
-	int res  = WPAD_Probe(0, &type);
-	if (res) return;
+	WPAD_ScanPads();
+	
+	int res = WPAD_Probe(0, &type);
+	GetIRPos(res, x, y);
+}
 
-	if (type == WPAD_EXP_CLASSIC) {
-		ProcessClassicInput(delta);
-	} else if (launcherMode) {
-		ProcessWPAD_Buttons(mods);
-	} else if (type == WPAD_EXP_NUNCHUK) {
-		ProcessNunchuck_Game(mods, delta);
-	} else {
-		ProcessWPAD_Buttons(mods);
-	}
 
+static void ProcessWPADDrag(int res, u32 mods) {
 	int x, y;
 	GetIRPos(res, &x, &y);
 	
@@ -381,24 +310,9 @@ static void ProcessWPADInput(double delta) {
 	} else {
 		dragActive = false;
 	}
-	Pointer_SetPosition(0, x, y);
+	VirtualCursor_SetPosition(x, y);
 }
 
-void Window_ProcessEvents(double delta) {
-	Input.JoystickMovement = false;
-
-	ProcessWPADInput(delta);
-	ProcessPADInput(delta);
-	ProcessKeyboardInput();
-}
-
-static void ScanAndGetIRPos(int* x, int* y) {
-	u32 type;
-	WPAD_ScanPads();
-	
-	int res = WPAD_Probe(0, &type);
-	GetIRPos(res, x, y);
-}
 #define FACTOR 2
 void Window_UpdateRawMouse(void)  {
 	if (!dragActive) return;
@@ -416,10 +330,7 @@ void Window_UpdateRawMouse(void)  {
 	dragCurX = x; dragCurY = y;
 }
 #else
-void Window_ProcessEvents(double delta) {
-	Input.JoystickMovement = false;
-	
-	ProcessPADInput(delta);
+void Window_ProcessEvents(float delta) {
 }
 
 void Window_UpdateRawMouse(void) { }
@@ -431,13 +342,173 @@ void Window_EnableRawMouse(void)  { Input.RawMode = true;  }
 void Window_DisableRawMouse(void) { Input.RawMode = false; }
 
 
+/*########################################################################################################################*
+*-------------------------------------------------------Gamepads----------------------------------------------------------*
+*#########################################################################################################################*/
+void Gamepads_Init(void) {
+	Input.Sources |= INPUT_SOURCE_GAMEPAD;
+
+	#if defined HW_RVL
+	WPAD_Init();
+	for (int i = 0; i < 4; i++)
+		WPAD_SetDataFormat(i, WPAD_FMT_BTNS_ACC_IR);
+	#endif
+	PAD_Init();
+}
+
+#if defined HW_RVL
+static const BindMapping default_nunchuck[BIND_COUNT] = {
+	[BIND_FORWARD] = { CCPAD_CUP,    0 },
+	[BIND_BACK]    = { CCPAD_CDOWN,  0 },
+	[BIND_LEFT]    = { CCPAD_CLEFT,  0 },
+	[BIND_RIGHT]   = { CCPAD_CRIGHT, 0 },
+	
+	[BIND_THIRD_PERSON] = { CCPAD_UP,    0 },
+	[BIND_FLY_DOWN]     = { CCPAD_DOWN,  0 },
+	[BIND_FLY]          = { CCPAD_LEFT,  0 },
+	[BIND_HOTBAR_RIGHT] = { CCPAD_RIGHT, 0 },
+	
+	[BIND_JUMP]         = { CCPAD_1, 0 },
+	[BIND_CHAT]         = { CCPAD_L, 0 },
+	[BIND_INVENTORY]    = { CCPAD_R, 0 },
+	
+	[BIND_PLACE_BLOCK]  = { CCPAD_5, 0 },
+	[BIND_DELETE_BLOCK] = { CCPAD_6, 0 },
+	[BIND_SET_SPAWN]    = { CCPAD_START, 0 },
+	[BIND_SEND_CHAT]    = { CCPAD_START, 0 },
+};
+
+static int dragCurX, dragCurY;
+static int dragStartX, dragStartY;
+static cc_bool dragActive;
+
+static void ProcessWPAD_Buttons(int port, int mods) {
+	Gamepad_SetButton(port, CCPAD_L, mods & WPAD_BUTTON_1);
+	Gamepad_SetButton(port, CCPAD_R, mods & WPAD_BUTTON_2);
+      
+	Gamepad_SetButton(port, CCPAD_1, mods & WPAD_BUTTON_A);
+	Gamepad_SetButton(port, CCPAD_2, mods & WPAD_BUTTON_B);
+	Gamepad_SetButton(port, CCPAD_3, mods & WPAD_BUTTON_PLUS);
+      
+	Gamepad_SetButton(port, CCPAD_START,  mods & WPAD_BUTTON_HOME);
+	Gamepad_SetButton(port, CCPAD_SELECT, mods & WPAD_BUTTON_MINUS);
+
+	Gamepad_SetButton(port, CCPAD_LEFT,   mods & WPAD_BUTTON_LEFT);
+	Gamepad_SetButton(port, CCPAD_RIGHT,  mods & WPAD_BUTTON_RIGHT);
+	Gamepad_SetButton(port, CCPAD_UP,     mods & WPAD_BUTTON_UP);
+	Gamepad_SetButton(port, CCPAD_DOWN,   mods & WPAD_BUTTON_DOWN);
+}
+
+static void ProcessNunchuck(int port, WPADData* wd, int mods) {
+	joystick_t analog = wd->exp.nunchuk.js;
+	Gamepad_SetButton(port, CCPAD_5, mods & WPAD_NUNCHUK_BUTTON_Z);
+	Gamepad_SetButton(port, CCPAD_6, mods & WPAD_NUNCHUK_BUTTON_C);
+
+	const float ANGLE_DELTA = 50;
+	bool nunchuckUp    = (analog.ang > -ANGLE_DELTA)    && (analog.ang < ANGLE_DELTA)     && (analog.mag > 0.5);
+	bool nunchuckDown  = (analog.ang > 180-ANGLE_DELTA) && (analog.ang < 180+ANGLE_DELTA) && (analog.mag > 0.5);
+	bool nunchuckLeft  = (analog.ang > -90-ANGLE_DELTA) && (analog.ang < -90+ANGLE_DELTA) && (analog.mag > 0.5);
+	bool nunchuckRight = (analog.ang > 90-ANGLE_DELTA)  && (analog.ang < 90+ANGLE_DELTA)  && (analog.mag > 0.5);
+
+	Gamepad_SetButton(port, CCPAD_CLEFT,  nunchuckLeft);
+	Gamepad_SetButton(port, CCPAD_CRIGHT, nunchuckRight);
+	Gamepad_SetButton(port, CCPAD_CUP,    nunchuckUp);
+	Gamepad_SetButton(port, CCPAD_CDOWN,  nunchuckDown);
+}
+
+#define CLASSIC_AXIS_SCALE 2.0f
+static void ProcessClassic_Joystick(int port, int axis, struct joystick_t* js, float delta) {
+	// TODO: need to account for min/max?? see libogc
+	int x = js->pos.x - js->center.x;
+	int y = js->pos.y - js->center.y;
+	
+	if (Math_AbsI(x) <= 8) x = 0;
+	if (Math_AbsI(y) <= 8) y = 0;
+	
+	Gamepad_SetAxis(port, axis, x / CLASSIC_AXIS_SCALE, -y / CLASSIC_AXIS_SCALE, delta);
+}
+
+static void ProcessClassicButtons(int port, int mods) {
+	Gamepad_SetButton(port, CCPAD_L, mods & CLASSIC_CTRL_BUTTON_FULL_L);
+	Gamepad_SetButton(port, CCPAD_R, mods & CLASSIC_CTRL_BUTTON_FULL_R);
+      
+	Gamepad_SetButton(port, CCPAD_1, mods & CLASSIC_CTRL_BUTTON_A);
+	Gamepad_SetButton(port, CCPAD_2, mods & CLASSIC_CTRL_BUTTON_B);
+	Gamepad_SetButton(port, CCPAD_3, mods & CLASSIC_CTRL_BUTTON_X);
+	Gamepad_SetButton(port, CCPAD_4, mods & CLASSIC_CTRL_BUTTON_Y);
+      
+	Gamepad_SetButton(port, CCPAD_START,  mods & CLASSIC_CTRL_BUTTON_PLUS);
+	Gamepad_SetButton(port, CCPAD_SELECT, mods & CLASSIC_CTRL_BUTTON_MINUS);
+
+	Gamepad_SetButton(port, CCPAD_LEFT,   mods & CLASSIC_CTRL_BUTTON_LEFT);
+	Gamepad_SetButton(port, CCPAD_RIGHT,  mods & CLASSIC_CTRL_BUTTON_RIGHT);
+	Gamepad_SetButton(port, CCPAD_UP,     mods & CLASSIC_CTRL_BUTTON_UP);
+	Gamepad_SetButton(port, CCPAD_DOWN,   mods & CLASSIC_CTRL_BUTTON_DOWN);
+	
+	Gamepad_SetButton(port, CCPAD_ZL, mods & CLASSIC_CTRL_BUTTON_ZL);
+	Gamepad_SetButton(port, CCPAD_ZR, mods & CLASSIC_CTRL_BUTTON_ZR);
+}
+
+static void ProcessClassicInput(int port, WPADData* wd, float delta) {
+	classic_ctrl_t ctrls = wd->exp.classic;
+	int mods = ctrls.btns | ctrls.btns_held;
+
+	ProcessClassicButtons(port, mods);
+	ProcessClassic_Joystick(port, PAD_AXIS_LEFT,  &ctrls.ljs, delta);
+	ProcessClassic_Joystick(port, PAD_AXIS_RIGHT, &ctrls.rjs, delta);
+}
+
+static int frame;
+static void ProcessWPADInput(int i, float delta) {
+	WPAD_ScanPads();
+	// First time WPADs are scanned, type is 0 even for classic/nunchuck it seems
+	//  (in Dolphin at least). So delay for a little bit
+	if (frame < 4 * 5) { frame++; return; }
+
+	u32 type;
+	int res = WPAD_Probe(i, &type);
+	if (res) return;
+
+	WPADData* wd = WPAD_Data(i);
+	u32 mods = wd->btns_h | wd->btns_d; // buttons held | buttons down now
+	int port;
+
+	if (type == WPAD_EXP_CLASSIC) {
+		port = Gamepad_Connect(0xC1 + i, PadBind_Defaults);
+		ProcessClassicInput(port, wd, delta);
+	} else if (type == WPAD_EXP_NUNCHUK) {
+		port = Gamepad_Connect(0xCC + i, default_nunchuck);
+		ProcessWPAD_Buttons(port, mods);
+		ProcessNunchuck(port, wd, mods);
+	} else {
+		port = Gamepad_Connect(0x11 + i, PadBind_Defaults);
+		ProcessWPAD_Buttons(port, mods);
+	}
+
+	ProcessWPADDrag(res, mods);
+}
+
+void Gamepads_Process(float delta) {
+	for (int i = 0; i < 4; i++)
+	{
+		ProcessWPADInput(i, delta);
+	}
+	ProcessPADInputs(delta);
+}
+#else
+void Gamepads_Process(float delta) {
+	ProcessPADInputs(delta);
+}
+#endif
+
+
 /*########################################################################################################################*
 *------------------------------------------------------Framebuffer--------------------------------------------------------*
 *#########################################################################################################################*/
-static struct Bitmap fb_bmp;
-void Window_AllocFramebuffer(struct Bitmap* bmp) {
-	bmp->scan0 = (BitmapCol*)Mem_Alloc(bmp->width * bmp->height, 4, "window pixels");
-	fb_bmp     = *bmp;
+void Window_AllocFramebuffer(struct Bitmap* bmp, int width, int height) {
+	bmp->scan0  = (BitmapCol*)Mem_Alloc(width * height, BITMAPCOLOR_SIZE, "window pixels");
+	bmp->width  = width;
+	bmp->height = height;
 }
 
 // TODO: Get rid of this complexity and use the 3D API instead..
@@ -446,13 +517,13 @@ static u32 CvtRGB (u8 r1, u8 g1, u8 b1, u8 r2, u8 g2, u8 b2)
 {
   int y1, cb1, cr1, y2, cb2, cr2, cb, cr;
  
-  y1  = (299 * r1 + 587 * g1 + 114 * b1) / 1000;
+  y1  = (299    * r1 +   587 * g1 +   114 * b1) / 1000;
   cb1 = (-16874 * r1 - 33126 * g1 + 50000 * b1 + 12800000) / 100000;
-  cr1 = (50000 * r1 - 41869 * g1 - 8131 * b1 + 12800000) / 100000;
+  cr1 = (50000  * r1 - 41869 * g1 -  8131 * b1 + 12800000) / 100000;
  
-  y2  = (299 * r2 + 587 * g2 + 114 * b2) / 1000;
+  y2  = (299    * r2 +   587 * g2 +   114 * b2) / 1000;
   cb2 = (-16874 * r2 - 33126 * g2 + 50000 * b2 + 12800000) / 100000;
-  cr2 = (50000 * r2 - 41869 * g2 - 8131 * b2 + 12800000) / 100000;
+  cr2 = (50000  * r2 - 41869 * g2 -  8131 * b2 + 12800000) / 100000;
  
   cb = (cb1 + cb2) >> 1;
   cr = (cr1 + cr2) >> 1;
@@ -460,17 +531,24 @@ static u32 CvtRGB (u8 r1, u8 g1, u8 b1, u8 r2, u8 g2, u8 b2)
   return (y1 << 24) | (cb << 16) | (y2 << 8) | cr;
 }
 
-void Window_DrawFramebuffer(Rect2D r) {
+void Window_DrawFramebuffer(Rect2D r, struct Bitmap* bmp) {
+	// When coming back from the 3D game, framebuffer might have changed
+	if (needsFBUpdate) {
+		VIDEO_SetNextFramebuffer(xfb);
+		VIDEO_Flush();
+		needsFBUpdate = false;
+	}
+	
 	VIDEO_WaitVSync();
-	r.X &= ~0x01; // round down to nearest even horizontal index
+	r.x &= ~0x01; // round down to nearest even horizontal index
 	
 	// TODO XFB is raw yuv, but is absolutely a pain to work with..
-	for (int y = r.Y; y < r.Y + r.Height; y++) 
+	for (int y = r.y; y < r.y + r.height; y++) 
 	{
-		cc_uint32* src = fb_bmp.scan0 + y * fb_bmp.width   + r.X;
-		u16* dst       = (u16*)xfb    + y * rmode->fbWidth + r.X;
+		cc_uint32* src = Bitmap_GetRow(bmp, y)           + r.x;
+		u16* dst       = (u16*)xfb  + y * rmode->fbWidth + r.x;
 		
-		for (int x = 0; x < r.Width / 2; x++) {
+		for (int x = 0; x < r.width / 2; x++) {
 			cc_uint32 rgb0 = src[(x<<1) + 0];
 			cc_uint32 rgb1 = src[(x<<1) + 1];
 			
@@ -485,6 +563,23 @@ void Window_FreeFramebuffer(struct Bitmap* bmp) {
 }
 
 
+/*########################################################################################################################*
+*------------------------------------------------------Soft keyboard------------------------------------------------------*
+*#########################################################################################################################*/
+void OnscreenKeyboard_Open(struct OpenKeyboardArgs* args) {
+	if (Input.Sources & INPUT_SOURCE_NORMAL) return;
+	VirtualKeyboard_Open(args, launcherMode);
+}
+
+void OnscreenKeyboard_SetText(const cc_string* text) {
+	VirtualKeyboard_SetText(text);
+}
+
+void OnscreenKeyboard_Close(void) {
+	VirtualKeyboard_Close();
+}
+
+
 /*########################################################################################################################*
 *-------------------------------------------------------Misc/Other--------------------------------------------------------*
 *#########################################################################################################################*/
@@ -501,10 +596,6 @@ void Window_Show(void) { }
 void Window_SetSize(int width, int height) { }
 
 
-void Window_OpenKeyboard(struct OpenKeyboardArgs* args) { /* TODO implement */ }
-void Window_SetKeyboardText(const cc_string* text) { }
-void Window_CloseKeyboard(void) { /* TODO implement */ }
-
 void Window_ShowDialog(const char* title, const char* msg) {
 	/* TODO implement */
 	Platform_LogConst(title);
@@ -518,4 +609,4 @@ cc_result Window_OpenFileDialog(const struct OpenFileDialogArgs* args) {
 cc_result Window_SaveFileDialog(const struct SaveFileDialogArgs* args) {
 	return ERR_NOT_SUPPORTED;
 }
-#endif
\ No newline at end of file
+#endif
