@@ -45,6 +45,18 @@
 
 FT_BEGIN_HEADER
 
+/* ClassiCube patch - don't export FreeType symbols in debug builds to avoid conflicting with system */
+#if defined(__GNUC__) && !defined(_WIN32) && !defined(__sun__)
+	#define FT_EXPORT( x ) __attribute__((visibility("hidden"))) extern x
+	#define FT_BASE( x )   __attribute__((visibility("hidden"))) extern x
+#elif defined(__cplusplus)
+	#define FT_EXPORT( x ) extern "C"  x
+	#define FT_BASE( x )   extern "C"  x
+#else
+	#define FT_EXPORT( x ) extern x
+	#define FT_BASE( x )   extern x
+#endif
+
 
   /*************************************************************************/
   /*                                                                       */
